//version=1.0
(function(window,document){
(function (global, factory) {
factory();
}(this, (function () { 'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
};











var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();





var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

/**
 * @class utils 工具类
 * @description 视频组件常用的方法
 */
var utils = {
    debounce: function debounce(func, wait, immediate) {
        var timeout, result;
        return function () {
            var context = this,
                args = arguments;
            var later = function later() {
                timeout = null;
                if (!immediate) result = func.apply(context, args);
            };
            var callNow = immediate && !timeout;
            clearTimeout(timeout);
            timeout = setTimeout(later, wait);
            if (callNow) result = func.apply(context, args);
            return result;
        };
    },
    simple_throttle: function simple_throttle(func, mustRunDelay) {
        var timer = null;
        var t_start;
        return function () {
            var context = this,
                args = arguments,
                t_curr = +new Date();
            clearTimeout(timer);
            if (!t_start) {
                t_start = t_curr;
            }
            if (t_curr - t_start >= mustRunDelay) {
                func.apply(context, args);
                t_start = t_curr;
            }
        };
    },
    throttle: function throttle(func, wait) {
        var context, args, timeout, result;
        var previous = 0;
        var later = function later() {
            previous = new Date();
            timeout = null;
            result = func.apply(context, args);
        };
        return function () {
            var now = new Date();
            var remaining = wait - (now - previous);
            context = this;
            args = arguments;
            if (remaining <= 0) {
                clearTimeout(timeout);
                timeout = null;
                previous = now;
                result = func.apply(context, args);
            } else if (!timeout) {
                timeout = setTimeout(later, remaining);
            }
            return result;
        };
    },
    /**
     * @method addEventHandler 事件监听
     * @param {dom} objectelement element
     * @param {string} eventname 事件 eg:"mousedown touchstart"
     * @param {function} eventhandler function(e) {}
     * @return {void} void
     */
    addEventHandler: function addEventHandler(objectelement, eventname, eventhandler) {
        var eventnames = eventname.split(' ');
        for (var i = 0, l = eventnames.length; i < l; i++) {
            var name = eventnames[i];
            if (document.attachEvent) {
                //ie浏览器
                objectelement.attachEvent("on" + name, eventhandler);
            } else if (document.addEventListener) {
                //非ie浏览器
                objectelement.addEventListener(name, eventhandler, false);
            }
        }
    },
    removeEventHandler: function removeEventHandler(objectelement, eventname, eventhandler) {
        var eventnames = eventname.split(' ');
        for (var i = 0, l = eventnames.length; i < l; i++) {
            var name = eventnames[i];
            if (document.detachevent) {
                //ie浏览器
                objectelement.detachevent("on" + name, eventhandler);
            } else if (document.addEventListener) {
                //非ie浏览器
                objectelement.removeEventListener(name, eventhandler, false);
            }
        }
    },
    /**
     * @method getFileName 把秒解析成 00:00 格式
     * @param {number} secs 秒
     * @param {number} total secs 秒
     * @return {String} 00:00 format
     */
    currentSecondToTime: function currentSecondToTime(secs, total) {
        var add0 = function add0(num) {
            return num < 10 ? '0' + num : '' + num;
        };

        secs = Math.round(secs);
        var totalSecs = Math.round(total);
        var hours = Math.floor(secs / (60 * 60));
        var totalHours = Math.floor(totalSecs / (60 * 60));

        var divisor_for_minutes = secs % (60 * 60);
        var minutes = Math.floor(divisor_for_minutes / 60);

        var divisor_for_seconds = divisor_for_minutes % 60;
        var seconds = Math.ceil(divisor_for_seconds);

        if (totalHours > 0) {
            return add0(hours) + ":" + add0(minutes) + ':' + add0(seconds);
        } else {
            return add0(minutes) + ':' + add0(seconds);
        }
    },
    /**
     * @method getFileName 把秒解析成 00:00 格式
     * @param {number} secs 秒
     * @return {String} 00:00 format
     */
    secondToTime: function secondToTime(secs) {

        if (isNaN(secs)) {
            return '00:00';
        }

        var add0 = function add0(num) {
            return num < 10 ? '0' + num : '' + num;
        };

        secs = Math.round(secs);
        var hours = Math.floor(secs / (60 * 60));

        var divisor_for_minutes = secs % (60 * 60);
        var minutes = Math.floor(divisor_for_minutes / 60);

        var divisor_for_seconds = divisor_for_minutes % 60;
        var seconds = Math.ceil(divisor_for_seconds);

        if (hours > 0) {
            return add0(hours) + ":" + add0(minutes) + ':' + add0(seconds);
        } else {
            return add0(minutes) + ':' + add0(seconds);
        }
    },
    /**
     * @method getFileName 根据路径获取文件名
     * @param {path} path 路径
     * @return {String} 00:00 format
     */
    getFileName: function getFileName(path) {
        var paths = path.split('/');
        if (path.indexOf('\\') > -1) {
            paths = path.split('\\');
        }
        return paths[paths.length - 1];
    },
    /**
     * @method createDom 创建dom
     * @param {str} html 
     * @return {dom} 返回dom对象
     */
    createDom: function createDom(str) {
        var dom = document && document.createElement('div');
        dom.innerHTML = str;
        return dom.firstChild;
    },
    insertAfter: function insertAfter(newElement, targetElement) {
        var parent = targetElement.parentNode;
        if (parent.lastChild == targetElement) {
            // 如果最后的节点是目标元素，则直接添加。因为默认是最后  
            parent.appendChild(newElement);
        } else {
            parent.insertBefore(newElement, targetElement.nextSibling);
            //如果不是，则插入在目标元素的下一个兄弟节点 的前面。也就是目标元素的后面  
        }
    },

    /**
     * @method getScrollPosition 获取window 滚动位置
     * @return {number} 返回window 滚动位置
     */
    getScrollPosition: function getScrollPosition() {
        return {
            left: window.pageXOffset || document.documentElement.scrollLeft || document.body.scrollLeft || 0,
            top: window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop || 0
        };
    },

    /**
     * @method setScrollPosition 获取window 滚动位置
     * @param {left} 
     * @param {top} 
     * @return {number} 返回window 滚动位置
     */
    setScrollPosition: function setScrollPosition(_ref) {
        var _ref$left = _ref.left,
            left = _ref$left === undefined ? 0 : _ref$left,
            _ref$top = _ref.top,
            top = _ref$top === undefined ? 0 : _ref$top;

        if (this.isFirefox) {
            document.documentElement.scrollLeft = left;
            document.documentElement.scrollTop = top;
        } else {
            window.scrollTo(left, top);
        }
    },


    /**
     * 检查是否用户是否用手机
     */
    isMobile: /mobile/i.test(window.navigator.userAgent),
    /**
     * 检查是否用户是否用Firefox
     */
    isFirefox: /firefox/i.test(window.navigator.userAgent),
    /**
     * 检查是否用户是否用chrome
     */
    isChrome: /chrome/i.test(window.navigator.userAgent),
    /**
     * localStorage 存储
     */
    storage: {
        set: function set$$1(key, value) {
            localStorage.setItem(key, value);
        },

        get: function get$$1(key) {
            return localStorage.getItem(key);
        }
    },

    getOffset: function getOffset(elem) {
        var docElem,
            win,
            box = {
            top: 0,
            left: 0
        },
            doc = elem && elem.ownerDocument;

        if (!doc) {
            return;
        }

        docElem = doc.documentElement;
        // If we don't have gBCR, just use 0,0 rather than error
        // BlackBerry 5, iOS 3 (original iPhone)
        if (_typeof(elem.getBoundingClientRect) !== (typeof undefined === 'undefined' ? 'undefined' : _typeof(undefined))) {
            box = elem.getBoundingClientRect();
        }
        win = doc == doc.window ? doc : doc.nodeType === 9 ? doc.defaultView || doc.parentWindow : false;
        return {
            top: box.top + (win.pageYOffset || docElem.scrollTop) - (docElem.clientTop || 0),
            left: box.left + (win.pageXOffset || docElem.scrollLeft) - (docElem.clientLeft || 0)
        };
    }
};

/**
 * @class i18n 国际化
 * @description 多语言处理
 */
function i18n(lang) {
    var _this = this;

    this.lang = lang;
    if (typeof lang === 'string') {
        this.tran = function (text) {
            if (tranTxt[_this.lang] && tranTxt[_this.lang][text]) {
                return tranTxt[_this.lang][text];
            } else {
                return text;
            }
        };
    } else {
        this.tran = function (text) {
            return _this.lang[text];
        };
    }
}

var tranTxt = {
    "zh_cn": {
        'text_fails_load': '视频加载失败'
    },
    "zh_hk": {
        'text_fails_load': '視頻載入失敗'
    },
    'ar_sa': {
        'text_fails_load': 'فشل تحميل الفيديو'
    },
    'en_us': {
        'text_fails_load': 'Video loading failed'
    },
    'ms_my': {
        'text_fails_load': 'Video gagal dimuat'
    },
    'ru_ru': {
        'text_fails_load': 'загрузка видео не удалась'
    },
    'th_th': {
        'text_fails_load': 'โหลดการแสดงผลไม่สำเร็จ'
    },
    'tr_tr': {
        'text_fails_load': 'Video yüklenemedi'
    }

};

/**
 * @class html html处理类
 * @description 处理视频相关的html及dom
 */
// ${html.video(true, options.video.pic, options.preload, options.video.url, options)}
var html = {
    main: function main(options, tran) {
        return '<span class="video-mediaelement" cid="video-mediaelement">\n    </span>\n    <span class="video-overlay" cid="video-layer">\n        <!-- \u64AD\u653E\u754C\u9762\u7684\u5176\u4ED6\u5C55\u793A -->\n        <!-- \u5C01\u9762 -->\n        <span class="video-layer ' + (options.video.pic ? '' : 'hide_dom') + '" cid="video-layer-cover">\n            <img class="video-layer-poster" src="' + (options.video.pic ? options.video.pic : '') + '" alt=""  cid="video-layer-cover">\n        </span>\n        <!-- \u52A0\u8F7D -->\n        <span class="video-layer hide_dom" cid="video-layer-loading">\n            <span class="video-layer-loading"><span class="loadgif"></span></span>\n        </span>\n        <!-- \u9519\u8BEF -->\n        <span class="video-layer hide_dom" cid="video-layer-error">\n            <span class="video-layer-error">\n                <span class="video-layer-error__content">\n                    <span class="video-layer-error__text" cid="video-layer-error-text">' + tran('text_fails_load') + '</span>\n                </span>\n            </span>\n        </span>\n        <!-- \u64AD\u653E\u6309\u94AE -->\n        <span class="video-layer hide_dom" cid="video-layer-player-button">\n            <span class="video-layer-button" cid="video-layer-button"></span>\n        </span>\n    </span>\n\n    <!-- \u9876\u90E8\u89C6\u9891\u540D\u79F0\u4E0E\u5173\u95ED\u6309\u94AE -->\n    <span class="video-header" cid="video-title-bar">\n        <span class="video-title">\n            <span class="video-title__text" cid="video-title-text">' + options.video.title + '</span>\n        </span>\n    </span>\n\n    <!-- \u64AD\u653E\u63A7\u5236\u680F\uFF0C\u9690\u85CF\u52A0 hide_dom -->\n    <span class="video-controls" cid="video-controls-bar">\n        <span class="video-btnwrap video-button">\n            <!-- \n                video-btn-pause  \u6682\u505C\n                video-btn-play  \u64AD\u653E\n            -->\n            <i class="video-btn video-btn-play" cid="video-btn-play"></i>\n        </span>\n        <span class="video-rail" cid="video-rail-bar">\n            <span class="video-rail__loaded" cid="video-rail-loaded" style="width: 0%;"></span>\n            <span class="video-rail__current" cid="video-rail-current" style="width: 0%;">\n                <i class="video-silde-point"></i>\n            </span>\n            <span class="video-rail__float hide_dom" cid="video-rail-float" style="left: 60%;">\n                <span class="video-text" cid="video-text-float">00:00:00</span>\n            </span>\n        </span>\n        <span class="video-time">\n            <span class="video-time__word">\n                <span class="video-text" cid="video-text-current">00:00</span>\n                <span class="video-text video-text--split">/</span>\n                <span class="video-text video-text--total" cid="video-text-total">/00:00</span>\n            </span>\n        </span>\n        <span class="video-btnwrap video-volume" cid="video-volume">\n            <!-- \n                video-btn-sound  \u6709\u58F0\u97F3\n                video-btn-nosound  \u9759\u97F3\n            -->\n            <i class="video-btn video-btn-sound" cid="video-btn-sound"></i>\n            <span class="video-silde" cid="video-silde">\n                <span class="video-silde__bg"></span>\n                <span class="video-silde__content" cid="video-silde-content">\n                    <span class="video-silde__bar" cid="video-silde-bar">\n                        <span class="video-silde__value" cid="video-silde-value" style="height: ' + options.volume * 100 + '%;">\n                            <span class="video-silde-point video-silde-point--small"></span>\n                        </span>\n                    </span>\n                </span>\n            </span>\n        </span>\n        <!-- \u5168\u5C4F -->\n        <span class="video-btnwrap video-screen ' + (options.showFullscreenBtn ? '' : 'hide_dom') + '">\n            <!-- \n                video-btn-full-screen  \u8FDB\u5165\u5168\u5C4F\n                video-btn-exit-screen  \u9000\u51FA\u5168\u5C4F\n            -->\n            <i class="video-btn video-btn-full-screen" cid="video-btn-full-screen"></i>\n        </span>\n    </span>\n    ';
    },
    video: function video(current, pic, preload, url, options) {
        var i = new Date().getTime();
        return '\n        <video \n        cid ="player-video"\n        title="' + (options.video.title ? options.video.title : '') + '"\n        class="player-video ' + (current ? 'player-video-current' : '') + '"\n        preload="' + preload + '"\n        duration="' + options.video.duration + '"\n        src="' + url + '"\n        </video>';
    },
    show: function show(obj) {
        obj.classList.remove('hide_dom');
    },
    hide: function hide(obj) {
        obj.classList.add('hide_dom');
    },
    getDoms: function getDoms(obj) {
        var video = obj.querySelector("video[cid='player-video']");

        var overlay = {};
        overlay.video = obj.querySelector("span[cid='video-layer']");
        overlay.loading = obj.querySelector("span[cid='video-layer-loading']");
        overlay.cover = obj.querySelector("span[cid='video-layer-cover']");
        overlay.error = obj.querySelector("span[cid='video-layer-error']");
        overlay.button = obj.querySelector("span[cid='video-layer-player-button']");

        var bar = {};
        bar.rail = obj.querySelector("span[cid='video-rail-bar']");
        bar.played = obj.querySelector("span[cid='video-rail-current']");
        bar.loaded = obj.querySelector("span[cid='video-rail-loaded']");
        bar.volume = obj.querySelector("span[cid='video-silde-value']");
        bar.title = obj.querySelector("span[cid='video-title-bar']");
        bar.controler = obj.querySelector("span[cid='video-controls-bar']");
        bar.volumeSilde = obj.querySelector("span[cid='video-silde-bar']");

        var timetip = obj.querySelector("span[cid='video-rail-float']");

        var buttons = {};
        buttons.layer = overlay.button.querySelector("span[cid='video-layer-button']");
        buttons.play = obj.querySelector("i[cid='video-btn-play']");
        buttons.fullscreen = obj.querySelector("i[cid='video-btn-full-screen']");
        buttons.icon = obj.querySelector("i[cid='video-btn-sound']");

        var volume = {};
        volume.controler = obj.querySelector("span[cid='video-volume']");
        volume.sildeContent = obj.querySelector("span[cid='video-silde-content']");
        volume.sildeBar = obj.querySelector("span[cid='video-silde-bar']");

        var texts = {};
        texts.current = obj.querySelector("span[cid='video-text-current']");
        texts.total = obj.querySelector("span[cid='video-text-total']");
        texts.timetip = obj.querySelector("span[cid='video-text-float']");
        texts.error = obj.querySelector("span[cid='video-layer-error-text']");
        texts.title = obj.querySelector("span[cid='video-title-text']");

        return {
            video: video,
            overlays: overlay,
            texts: texts,
            bars: bar,
            buttons: buttons,
            timetip: timetip,
            volume: volume
        };
    }
};

/*
 * xhr.status ---> fail
 * response.code === 1 ---> success
 * response.code !== 1 ---> error
 * */
var SendXMLHttpRequest = function SendXMLHttpRequest(url, data, success, error, fail) {
    var xhr = new XMLHttpRequest();

    xhr.onreadystatechange = function () {
        if (xhr.readyState === 4) {
            if (xhr.status >= 200 && xhr.status < 300 || xhr.status === 304) {
                var response = JSON.parse(xhr.responseText);

                if (response.code !== 1) {
                    return error(xhr, response);
                }

                return success(xhr, response);
            }

            fail(xhr);
        }
    };

    xhr.open(data !== null ? 'POST' : 'GET', url, true);
    xhr.send(data !== null ? JSON.stringify(data) : null);
};

var defaultApiBackend = {
    send: function send(endpoint, danmakuData, callback) {
        SendXMLHttpRequest(endpoint, danmakuData, function (xhr, response) {
            console.log('Post danmaku: ', response);
            if (callback) {
                callback();
            }
        }, function (xhr, response) {
            alert(response.msg);
        }, function (xhr) {
            console.log('Request was unsuccessful: ' + xhr.status);
        });
    },

    read: function read(endpoint, callback) {
        SendXMLHttpRequest(endpoint, null, function (xhr, response) {
            callback(null, response.danmaku);
        }, function (xhr, response) {
            callback({
                status: xhr.status,
                response: response
            });
        }, function (xhr) {
            callback({
                status: xhr.status,
                response: null
            });
        });
    }
};

/**
 * @class options 视频选项处理类
 * @description 视频选项处理类
 */
var handleOption = function (options) {
    var isMobile = /mobile/i.test(window.navigator.userAgent);
    if (isMobile) {
        options.autoplay = false;
    }
    var defaultOption = {
        container: options.element || document.getElementsByClassName('nd-video-player')[0],
        fullContainer: document.body,
        autoplay: false,
        allowSelected: false,
        style: '',
        theme: '#b7daff',
        extendStyle: '',
        loop: false,
        lang: (navigator.language || navigator.browserLanguage).replace('-', '_').toLowerCase(),
        screenshot: false,
        hotkey: true,
        preload: 'auto',
        volume: 0.7,
        apiBackend: defaultApiBackend,
        video: {},
        muted: true,
        showFullscreenBtn: true, // 是否支持全屏
        duration: 0,
        width: '', //12.5em
        height: '' //10em
    };

    options.width = options.width == 0 || options.width == undefined ? defaultOption.width : options.width;
    options.height = options.height == 0 || options.height == undefined ? defaultOption.height : options.height;

    for (var defaultKey in defaultOption) {
        if (defaultOption.hasOwnProperty(defaultKey) && !options.hasOwnProperty(defaultKey)) {
            options[defaultKey] = defaultOption[defaultKey];
        }
    }

    if (options.video) {
        !options.video.type && (options.video.type = 'auto');
    }

    if (options.subtitle) {
        !options.subtitle.type && (options.subtitle.type = 'webvtt');
        !options.subtitle.fontSize && (options.subtitle.fontSize = '20px');
        !options.subtitle.bottom && (options.subtitle.bottom = '40px');
        !options.subtitle.color && (options.subtitle.color = '#fff');
    }

    if (options.lang) {
        options.lang = options.lang.toLowerCase();
    }

    return options;
};

/**
 * @class Events 事件处理类
 * @description 事件相关处理
 */
var Events = function () {
    function Events() {
        classCallCheck(this, Events);

        this.events = {};

        this.videoEvents = ['abort', 'canplay', 'canplaythrough', 'durationchange', 'emptied', 'ended', 'error', 'loadeddata', 'loadedmetadata', 'loadstart', 'mozaudioavailable', 'pause', 'play', 'playing', 'progress', 'ratechange', 'seeked', 'seeking', 'stalled', 'suspend', 'timeupdate', 'volumechange', 'waiting'];
        this.playerEvents = ['screenshot', 'thumbnails_show', 'thumbnails_hide', 'danmaku_show', 'danmaku_hide', 'danmaku_clear', 'danmaku_loaded', 'danmaku_send', 'danmaku_opacity', 'contextmenu_show', 'contextmenu_hide', 'notice_show', 'notice_hide', 'quality_start', 'quality_end', 'destroy', 'resize', 'fullscreen', 'fullscreen_cancel', 'webfullscreen', 'webfullscreen_cancel', 'title_show', 'title_hide', 'title_change', 'imageopen'];
    }

    createClass(Events, [{
        key: 'on',
        value: function on(name, callback) {
            if (this.type(name) && typeof callback === 'function') {
                if (!this.events[name]) {
                    this.events[name] = [];
                }
                this.events[name].push(callback);
            }
        }
    }, {
        key: 'trigger',
        value: function trigger(name, info) {
            if (this.events[name] && this.events[name].length) {
                for (var i = 0; i < this.events[name].length; i++) {
                    this.events[name][i](info);
                }
            }
        }
    }, {
        key: 'type',
        value: function type(name) {
            if (this.playerEvents.indexOf(name) !== -1) {
                return 'player';
            } else if (this.videoEvents.indexOf(name) !== -1) {
                return 'video';
            }

            console.error('Unknown event name: ' + name);
            return null;
        }
    }]);
    return Events;
}();

var classList = {
    remove: function remove(dom, clazz) {
        dom.classList.remove(clazz);
    },
    add: function add(dom, clazz) {
        dom.classList.add(clazz);
    },
    button_fullscreen: function button_fullscreen(dom) {
        this.remove(dom, 'video-btn-full-screen');
        this.add(dom, 'video-btn-exit-screen');
    },
    button_exitFullscreen: function button_exitFullscreen(dom) {
        this.remove(dom, 'video-btn-exit-screen');
        this.add(dom, 'video-btn-full-screen');
    },
    container_fullscreen: function container_fullscreen(dom, isFullBody) {
        if (isFullBody) {
            this.add(dom, 'component__video--fullscreen');
        } else {
            this.add(dom, 'component__video--full-specified-container');
        }
    },
    container_exitFullscreen: function container_exitFullscreen(dom, isFullBody) {
        if (isFullBody) {
            this.remove(dom, 'component__video--fullscreen');
        } else {
            this.remove(dom, 'component__video--full-specified-container');
        }
    }
};

// import { setTimeout } from 'timers';
/**
 * @class FullScreen 全屏处理类
 * @description 处理视频的全屏和取消全屏
 */

var FullScreen = function () {
    function FullScreen(player) {
        var _this = this;

        classCallCheck(this, FullScreen);

        this.player = player;
        this.paused = this.player.player.video.paused;
        this.player.events.on('webfullscreen', function () {
            _this.player.resize();
        });
        this.player.events.on('webfullscreen_cancel', function () {
            _this.player.resize();
        });

        var fullscreenchange = function fullscreenchange() {
            _this.player.resize();
            if (_this.isFullScreen('browser')) {
                _this.player.events.trigger('fullscreen');
            } else {
                _this.player.events.trigger('fullscreen_cancel');
            }
        };
        this.player.container.addEventListener('fullscreenchange', fullscreenchange);
        this.player.container.addEventListener('mozfullscreenchange', fullscreenchange);
        this.player.container.addEventListener('webkitfullscreenchange', fullscreenchange);
        this.parentContainer = this.player.container.parentNode;
    }

    createClass(FullScreen, [{
        key: 'isFullScreen',
        value: function isFullScreen() {
            var type = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'web';

            switch (type) {
                case 'browser':
                    return document.fullscreenElement || document.mozFullScreenElement || document.webkitFullscreenElement;
                case 'web':
                    return this.player.container.classList.contains('component__video--fullscreen') || this.player.container.classList.contains('component__video--full-specified-container');
            }
        }
    }, {
        key: 'request',
        value: function request() {
            var type = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'web';

            var self = this;
            var isFullBody = this.player.options.fullContainer.tagName.toUpperCase() === 'BODY';
            this.paused = this.player.player.video.paused;
            this.lastScrollPosition = utils.getScrollPosition();
            classList.container_fullscreen(this.player.container, isFullBody);
            classList.button_fullscreen(this.player.player.buttons.fullscreen);
            var container = this.player.container;
            this.nullContainer = utils.createDom('<div cid="component-video-null" />');
            utils.insertAfter(this.nullContainer, this.player.container);
            this.player.options.fullContainer.appendChild(container);
            if (!this.paused) {
                setTimeout(function () {
                    self.player.player.video.play();
                }, 400);
            }
            // this.player.events.trigger('webfullscreen');   
            if (this.player.options.width != null || this.player.options.height != null) {
                this.player.container.removeAttribute('style');
            }

            if (this.player.options.fsStyle != null) {
                for (var defaultKey in this.player.options.fsStyle) {
                    this.player.container.style[defaultKey] = this.player.options.fsStyle[defaultKey];
                }
            }
        }
    }, {
        key: 'cancel',
        value: function cancel() {
            var type = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'web';

            var self = this;
            var isFullBody = this.player.options.fullContainer.tagName.toUpperCase() === 'BODY';
            this.paused = this.player.player.video.paused;
            utils.setScrollPosition(this.lastScrollPosition);
            classList.container_exitFullscreen(this.player.container, isFullBody);
            classList.button_exitFullscreen(this.player.player.buttons.fullscreen);
            utils.insertAfter(this.player.container, this.nullContainer);
            this.parentContainer.removeChild(this.nullContainer);
            if (!this.paused) {
                setTimeout(function () {
                    self.player.player.video.play();
                }, 400);
            }
            // this.player.events.trigger('webfullscreen_cancel');
            this.player.container.removeAttribute('style');
            if (this.player.options.width != null) {
                this.player.container.style['width'] = this.player.options.width;
            }
            if (this.player.options.height != null) {
                this.player.container.style['height'] = this.player.options.width;
            }
        }

        // 自定义函数向后插入

    }, {
        key: 'insertAfter',
        value: function insertAfter(newElement, targetElement) {
            var parent = targetElement.parentNode;
            if (parent.lastChild == targetElement) {
                // 如果最后的节点是目标元素，则直接添加。因为默认是最后
                parent.a(newElement);
            } else {
                //如果不是，则插入在目标元素的下一个兄弟节点的前面。也就是目标元素的后面
                parent.insertBefore(newElement, targetElement.nextSibling);
            }
        }
    }, {
        key: 'toggle',
        value: function toggle() {
            var type = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'web';

            var isFull = true;
            if (this.isFullScreen(type)) {
                this.cancel(type);
                isFull = false;
            } else {
                this.request(type);
            }

            if (this.player.options.onFullScreenChange) {
                this.player.options.onFullScreenChange(isFull);
            }
        }
    }]);
    return FullScreen;
}();

var EXTEND_STYLE = {
    'blue': {
        'className': 'style--pop-blue'
    }
};

// var firstNameS = Symbol("firstName");
// var lastNameS = Symbol("lastName");
// var ageS = Symbol("age");

// const VAL_OFFSET_TOP = 5.5
var VAL_OFFSET_TOP = 0;
/**
 * @class VPlayer 视频组件
 * @description 视频 基本功能
 */

var VPlayer$1 = function () {
    function VPlayer(options) {
        classCallCheck(this, VPlayer);

        if (!options) {
            throw new Error("至少需要一个参数");
            return;
        }

        this.options = handleOption(options);
        this.tran = new i18n(this.options.lang).tran;
        this.events = new Events();
        this.container = this.options.container;
        this.durationCache = 0;
        this.loaded = false;

        var src = this.container.getAttribute('src') ? this.container.getAttribute('src') : this.container.querySelector('source').getAttribute('src');
        var duration = this.options.video.duration ? this.options.video.duration : this.container.getAttribute('duration') ? this.container.getAttribute('duration') : 0;

        var title = this.options.video.title ? this.options.video.title : this.container.getAttribute('title') ? this.container.getAttribute('title') : utils.getFileName(src);
        var cover = this.container.getAttribute('cover');

        this.options.video.url = src;
        this.options.video.pic = cover;
        this.options.video.title = title ? title : this.options.video.title ? this.options.video.title : utils.getFileName(src);
        this.options.video.duration = duration == "undefined" ? 0 : duration;
    }

    /**
     * @method change 改变options
     * @returns {void} 
     */


    createClass(VPlayer, [{
        key: 'change',
        value: function change(options) {
            if (!options) {
                throw new Error("至少需要一个参数");
                return;
            }

            this.pause();
            this.seek(0);

            this.options = handleOption(options);
            this.tran = new i18n(this.options.lang).tran;
            // this.events = new events();

            this.options.video.url = options.video.url;
            var title = options.video.title ? options.video.title : utils.getFileName(options.video.url);
            this.options.video.title = title ? title : this.options.video.title ? this.options.video.title : utils.getFileName(src);
            this.options.video.duration = options.video.duration ? options.video.duration : 0;

            this.player.texts.title.innerText = this.options.video.title;
            this.updateTotalPlayTime(0);
            this.ended = false;
            this.loaded = false;

            this.player.video.src = options.video.url;
            this.player.video.load();
            // this.bindEvent();
            // this.video
        }

        /**
         * @method render 渲染组件
         * @returns {void} 
         */

    }, {
        key: 'render',
        value: function render() {
            var _this = this;

            var resetVideo = function resetVideo() {
                while (_this.container.hasChildNodes()) {
                    _this.container.removeChild(_this.container.firstChild);
                }
                _this.container.removeAttribute("controls");
                // this.container.setAttribute("crossorigin", "anonymous");
                _this.container.setAttribute("cid", "player-video");
                if (_this.container.getAttribute('src') !== _this.options.video.url) {
                    _this.container.setAttribute("src", _this.options.video.url);
                }
                _this.container.setAttribute("title", _this.options.video.title);
                _this.container.setAttribute("class", "player-video player-video-current");
                _this.container.setAttribute("preload", _this.options.preload);
                _this.container.setAttribute("duration", _this.options.video.duration);
                _this.container.setAttribute("webkit-playsinline", true);
                _this.container.setAttribute("playsinline", true);
                _this.container.setAttribute("width", "1");
                _this.container.setAttribute("height", "1");
                _this.container.setAttribute("x-webkit-airplay", true);
            };

            resetVideo();

            var videoHtml = html.main(this.options, this.tran);
            var topContains = utils.createDom('<span class="component__video" cid="component-video" />');
            topContains.innerHTML = videoHtml;
            utils.insertAfter(topContains, this.container);
            var videoContains = topContains.querySelector("span[cid='video-mediaelement']");
            videoContains.appendChild(this.container);
            this.container = topContains;
            this.componentVideo = topContains;

            var doms = html.getDoms(this.container);

            if (this.options.width != null) {
                this.container.style['width'] = this.options.width;
            }
            if (this.options.height != null) {
                this.container.style['height'] = this.options.width;
            }

            // 扩展皮肤，增加相应类名
            if (this.options.extendStyle && EXTEND_STYLE[this.options.extendStyle]) {
                this.componentVideo.classList.add(EXTEND_STYLE[this.options.extendStyle].className);
            }

            this.player = doms;

            this.bindEvent();
        }
    }, {
        key: 'bindEvent',
        value: function bindEvent() {
            var _this2 = this;

            //vide init timetip 
            this.isTimeTipsShow = true;

            this.fullScreen = new FullScreen(this);

            var that = this;

            var mouseEvents = utils.isMobile ? {
                down: 'touchstart mousedown',
                move: 'touchmove mousemove',
                up: 'touchend mouseup',
                over: 'touchstart',
                out: 'touchend',
                tap: 'touchstart click'
            } : {
                down: 'mousedown touchstart',
                move: 'mousemove touchmove',
                up: 'mouseup touchend',
                over: 'mouseover',
                out: 'mouseout',
                tap: 'click touchstart'
            };

            var barWidth = void 0;
            this.updateBar = function (type, percentage, direction) {
                percentage = percentage > 0 ? percentage : 0;
                percentage = percentage < 1 ? percentage : 1;
                _this2.player.bars[type].style[direction] = percentage * 100 + '%';
            };

            var stopPropagation = function stopPropagation(e) {
                if (document.all) {
                    //只有ie识别
                    e.cancelBubble = true;
                } else {
                    e.stopPropagation();
                }
            };

            var cancelBubble = function cancelBubble(e) {
                //取消事件的默认动作
                e.preventDefault && e.preventDefault();
                stopPropagation(e);
            };

            var playerToggele = function playerToggele(e) {
                var e = e || window.event;
                cancelBubble(e);
                utils.throttle(that.toggle(), 500);
            };

            utils.addEventHandler(document, mouseEvents.tap, utils.throttle(function () {
                if (_this2.player != null && _this2.player.volume != null) {
                    _this2.player.volume.controler.classList.remove("ui-btn-active");
                }
            }, 500));

            if (!utils.isMobile) {
                utils.addEventHandler(this.player.buttons.play, "mouseover", function () {
                    _this2.player.buttons.play.classList.add("ui-btn-hover");
                });
                utils.addEventHandler(this.player.buttons.play, "mouseout", function () {
                    _this2.player.buttons.play.classList.remove("ui-btn-hover");
                });
                utils.addEventHandler(this.player.volume.controler, "mouseover", function () {
                    _this2.player.buttons.icon.classList.add("ui-btn-hover");
                    _this2.player.volume.controler.classList.add("ui-btn-active");
                });
                utils.addEventHandler(this.player.volume.controler, "mouseout", function () {
                    _this2.player.buttons.icon.classList.remove("ui-btn-hover");
                    _this2.player.volume.controler.classList.remove("ui-btn-active");
                });
                utils.addEventHandler(this.player.buttons.fullscreen, "mouseover", function () {
                    _this2.player.buttons.fullscreen.classList.add("ui-btn-hover");
                });
                utils.addEventHandler(this.player.buttons.fullscreen, "mouseout", function () {
                    _this2.player.buttons.fullscreen.classList.remove("ui-btn-hover");
                });
            }

            utils.addEventHandler(this.player.buttons.layer, mouseEvents.tap, playerToggele);

            utils.addEventHandler(this.player.buttons.play, mouseEvents.tap, playerToggele);

            this.mouseHandler = this.mouseHandler(this.player.bars.rail, this.player.timetip);
            utils.addEventHandler(this.player.bars.rail, mouseEvents.move, this.mouseHandler);
            utils.addEventHandler(this.player.bars.rail, 'mouseenter', this.mouseHandler);
            utils.addEventHandler(this.player.bars.rail, 'mouseleave', this.mouseHandler);
            utils.addEventHandler(this.player.bars.rail, 'mouseout', this.mouseHandler);
            utils.addEventHandler(this.componentVideo, 'mouseleave', function () {
                if (that.player != null) {
                    that.player.timetip.classList.add('hide_dom');
                }
            });

            var getPercentage = function getPercentage(e) {
                var e = e || window.event;
                var clientX = e.changedTouches ? e.changedTouches[0].clientX : e.clientX;
                var percentage = (clientX - utils.getOffset(_this2.player.bars.rail).left) / _this2.getScale() / barWidth;
                percentage = percentage > 0 ? percentage : 0;
                percentage = percentage < 1 ? percentage : 1;
                return percentage;
            };

            var thumbMove = function thumbMove(e) {
                var e = e || window.event;
                cancelBubble(e);
                if (1 == e.which || 0 == e.which) {
                    var percentage = getPercentage();
                    _this2.updateBar('played', percentage, 'width');
                    var duration = _this2.getDuration();
                    _this2.player.texts.timetip.innerHTML = utils.currentSecondToTime(percentage * duration, duration);
                    _this2.seek(parseFloat(_this2.player.bars.played.style.width) / 100 * duration);
                    _this2.setTime();
                }
            };

            var thumbUp = function thumbUp(e) {
                var e = e || window.event;
                cancelBubble(e);
                if (1 == e.which || 0 == e.which) {
                    var percentage = getPercentage();
                    _this2.updateBar('played', percentage, 'width');
                    var duration = _this2.getDuration();
                    _this2.seek(parseFloat(_this2.player.bars.played.style.width) / 100 * duration);
                    _this2.setTime();
                    utils.removeEventHandler(document, mouseEvents.up, thumbUp);
                    utils.removeEventHandler(document, mouseEvents.move, thumbMove);
                    document.onmousemove = null;
                    document.onmouseup = null;
                }
            };

            var screenMove = function screenMove(e) {
                var e = e || window.event;
                cancelBubble(e);
                if (1 == e.which || 0 == e.which) {
                    var percentage = getPercentage();
                    _this2.updateBar('played', percentage, 'width');
                    var duration = _this2.getDuration();
                    _this2.player.texts.timetip.innerHTML = utils.currentSecondToTime(percentage * duration, duration);
                    _this2.seek(parseFloat(_this2.player.bars.played.style.width) / 100 * duration);
                    _this2.setTime();
                }
            };

            var screenUp = function screenUp(e) {
                var e = e || window.event;
                cancelBubble(e);
                if (1 == e.which || 0 == e.which) {
                    utils.removeEventHandler(document, mouseEvents.up, screenUp);
                    utils.removeEventHandler(document, mouseEvents.move, screenMove);
                    document.onmousemove = null;
                    document.onmouseup = null;
                }
            };

            // utils.throttle(() => {
            //     if (this.player != null && this.player.volume != null) {
            //         this.player.volume.controler.classList.remove("ui-btn-active");
            //     }
            // }, 500)
            utils.addEventHandler(this.player.bars.rail, mouseEvents.down, function (e) {
                var e = e || window.event;
                cancelBubble(e);
                if (1 == e.which || 0 == e.which) {
                    barWidth = _this2.player.bars.rail.clientWidth;
                    _this2.clearTime();
                    utils.addEventHandler(document, mouseEvents.move, thumbMove);
                    utils.addEventHandler(document, mouseEvents.up, thumbUp);
                }
            });

            if (!this.options.allowSelected) {
                utils.addEventHandler(this.player.overlays.video, mouseEvents.down, utils.simple_throttle(function (e) {
                    var e = e || window.event;
                    cancelBubble(e);
                    if (1 == e.which || 0 == e.which) {
                        _this2.isScreenDown = true;
                        barWidth = _this2.player.bars.rail.clientWidth;
                        _this2.clearTime();
                        utils.addEventHandler(document, mouseEvents.move, screenMove);
                        utils.addEventHandler(document, mouseEvents.up, screenUp);
                    }
                }, 1000));
            }

            /**
             * control volume
             */
            var vWidth = 80;
            this.switchVolumeIcon = function () {
                if (_this2.volume() > 0) {
                    _this2.player.buttons.icon.classList.remove('video-btn-nosound');
                    _this2.player.buttons.icon.classList.add('video-btn-sound');
                } else {
                    _this2.player.buttons.icon.classList.remove('video-btn-sound');
                    _this2.player.buttons.icon.classList.add('video-btn-nosound');
                }
            };

            var volumeMove = function volumeMove(e) {
                var e = e || window.event;
                cancelBubble(e);
                if (1 == e.which || 0 == e.which) {
                    var clientY = e.changedTouches ? e.changedTouches[0].clientY : e.clientY;
                    var percentage = 1 - (clientY - _this2.player.volume.sildeBar.getBoundingClientRect().top - VAL_OFFSET_TOP) / _this2.getScale() / vWidth;
                    _this2.volume(percentage);
                }
            };
            var volumeUp = function volumeUp(e) {
                var e = e || window.event;
                cancelBubble(e);
                if (1 == e.which || 0 == e.which) {
                    utils.removeEventHandler(document, mouseEvents.up, volumeUp);
                    utils.removeEventHandler(document, mouseEvents.move, volumeMove);
                    document.onmousemove = null;
                    document.onmouseup = null;
                }
            };

            utils.addEventHandler(this.player.volume.sildeContent, mouseEvents.tap, function (e) {
                var e = e || window.event;
                cancelBubble(e);
                if (1 == e.which || 0 == e.which) {
                    var clientY = e.changedTouches ? e.changedTouches[0].clientY : e.clientY;
                    var percentage = 1 - (clientY - _this2.player.volume.sildeBar.getBoundingClientRect().top - VAL_OFFSET_TOP) / _this2.getScale() / vWidth;
                    utils.throttle(_this2.volume(percentage), 500);
                }
            });

            utils.addEventHandler(this.player.volume.sildeContent, mouseEvents.down, function (e) {
                var e = e || window.event;
                cancelBubble(e);
                utils.addEventHandler(document, mouseEvents.move, volumeMove);
                utils.addEventHandler(document, mouseEvents.up, volumeUp);
            });

            utils.addEventHandler(this.player.buttons.icon, mouseEvents.tap, utils.throttle(function (e) {
                var e = e || window.event;
                cancelBubble(e);
                _this2.player.volume.controler.classList.add("ui-btn-active");
                if (_this2.player.video.muted) {
                    _this2.player.video.muted = false;
                    _this2.switchVolumeIcon();
                    _this2.updateBar('volume', _this2.volume(), 'height');
                } else {
                    _this2.player.video.muted = true;
                    _this2.player.buttons.icon.classList.remove('video-btn-sound');
                    _this2.player.buttons.icon.classList.add('video-btn-nosound');
                    _this2.updateBar('volume', 0, 'height');
                }
            }, 500));

            /**
             * auto hide controller
             */
            this.hideTime = 0;
            var hideController = function hideController(e) {
                var e = e || window.event;
                cancelBubble(e);
                html.show(_this2.player.bars.title);
                html.show(_this2.player.bars.controler);
                clearTimeout(_this2.hideTime);
                _this2.hideTime = setTimeout(function () {
                    if (_this2.player.video.currentTime > 0 && !_this2.disableHideController) {
                        html.hide(_this2.player.bars.title);
                        html.hide(_this2.player.bars.controler);
                    }
                }, 2000);
            };

            utils.addEventHandler(this.player.overlays.video, mouseEvents.move, utils.throttle(hideController, 500));
            utils.addEventHandler(this.player.overlays.video, mouseEvents.tap, utils.throttle(hideController, 500));
            utils.addEventHandler(this.player.overlays.video, 'mouseenter', function () {
                _this2.disableHideController = true;
            });
            utils.addEventHandler(this.player.overlays.video, 'mouseleave', function () {
                _this2.disableHideController = false;
            });

            utils.addEventHandler(this.player.volume.controler, 'mouseenter', function () {
                _this2.disableHideController = true;
            });

            utils.addEventHandler(this.player.volume.controler, 'mouseleave', function () {
                _this2.disableHideController = false;
            });

            // if allowSelected = false support click event
            if (!this.options.allowSelected) {
                utils.addEventHandler(this.player.overlays.video, mouseEvents.tap, utils.throttle(playerToggele, 500));
            }

            // web full screen
            utils.addEventHandler(this.player.buttons.fullscreen, mouseEvents.tap, utils.throttle(function (e) {
                var e = e || window.event;
                cancelBubble(e);
                _this2.fullScreen.toggle('web');
            }, 500));

            var lastPlayPos = 0;
            var currentPlayPos = 0;
            var bufferingDetected = false;
            window.requestAnimationFrame = function () {
                return window.requestAnimationFrame || window.webkitRequestAnimationFrame || window.mozRequestAnimationFrame || window.oRequestAnimationFrame || window.msRequestAnimationFrame || function (callback) {
                    window.setTimeout(callback, 1000 / 60);
                };
            }();

            var setCheckLoadingTime = function setCheckLoadingTime() {
                _this2.checkLoading = setInterval(function () {
                    currentPlayPos = _this2.player.video.currentTime;
                    if (!bufferingDetected && currentPlayPos === lastPlayPos && !_this2.player.video.paused) {
                        _this2.container.classList.add('player-loading');
                        bufferingDetected = true;
                    }
                    if (bufferingDetected && currentPlayPos > lastPlayPos && !_this2.player.video.paused) {
                        _this2.container.classList.remove('player-loading');
                        bufferingDetected = false;
                    }
                    lastPlayPos = currentPlayPos;
                }, 100);
            };

            var clearCheckLoadingTime = function clearCheckLoadingTime() {
                clearInterval(_this2.checkLoading);
            };

            this.playedTime = false;
            this.animationFrame = function () {
                if (_this2.playedTime) {
                    _this2.updateBar('played', _this2.player.video.currentTime / _this2.getDuration(), 'width');
                    _this2.updatePlayTime(_this2.player.video.currentTime, _this2.getDuration());
                    _this2.updateTotalPlayTime(_this2.getDuration());
                }

                if (window.requestAnimationFrame && _this2.animationFrame) {
                    window.requestAnimationFrame(_this2.animationFrame);
                }
            };
            window.requestAnimationFrame(this.animationFrame);

            this.setTime = function (type) {
                if (!type) {
                    _this2.playedTime = true;
                    setCheckLoadingTime();
                } else {
                    _this2[type + 'Time'] = true;
                    if (type === 'played') {
                        setCheckLoadingTime();
                    }
                }
            };
            this.clearTime = function (type) {
                if (!type) {
                    _this2.playedTime = false;
                    clearCheckLoadingTime();
                } else {
                    _this2[type + 'Time'] = false;
                    if (type === 'played') {
                        clearCheckLoadingTime();
                    }
                }
            };

            html.show(this.player.overlays.loading);
            // set duration time
            if (this.player.video.duration !== 1) {
                html.hide(this.player.overlays.loading);
                this.updatePlayTime(0, this.getDuration());
                this.updateTotalPlayTime(this.getDuration());
            }

            // autoplay
            if (this.options.autoplay) {
                this.play();
            } else {
                this.pause();
            }

            this.volume(this.options.volume);

            this.initVideo(this.player.video, this.options.video.type);
        }

        /**
         * @method Seek seek video
         * @param {video} video 视频vid
         * @param {type} type  
         * @returns {void} void
         */

    }, {
        key: 'initVideo',
        value: function initVideo(video, type) {
            var _this3 = this;

            // this.setLock(true);
            // show video time: the metadata has loaded or changed
            this.on('durationchange', function () {
                if (video.duration !== 1) {
                    // compatibility: Android browsers will output 1 at first
                    // this.setLock(false);
                    if (!isNaN(video.duration)) {
                        _this3.durationCache = video.duration;
                    }
                    var currentTime = video.currentTime || 0;
                    _this3.updatePlayTime(currentTime, _this3.player.video.duration);
                    _this3.updateTotalPlayTime(video.duration);
                }
            });

            // show video loaded bar: to inform interested parties of progress downloading the media
            this.on('progress', function () {
                var percentage = video.buffered.length ? video.buffered.end(video.buffered.length - 1) / video.duration : 0;
                _this3.updateBar('loaded', percentage, 'width');
            });

            this.on('waiting', function () {
                html.show(_this3.player.overlays.loading);
            });

            // video end
            this.ended = false;
            this.on('ended', function () {
                _this3.updateBar('played', 1, 'width');
                if (!_this3.loop) {
                    _this3.ended = true;
                    _this3.pause();
                    _this3.seek(0);
                    _this3.disableHideController = true;
                    html.show(_this3.player.overlays.cover);
                    html.show(_this3.player.bars.title);
                    html.show(_this3.player.bars.controler);
                } else {
                    _this3.seek(0);
                    video.play();
                }

                if (typeof _this3.options.onEnd !== 'undefined') {
                    _this3.options.onEnd();
                }
            });

            this.on('play', function () {
                if (_this3.player.video.paused) {
                    _this3.play();
                }
            });

            this.on('pause', function () {
                if (!_this3.player.video.paused) {
                    _this3.pause();
                }
            });

            this.on('loadedmetadata', function () {
                console.log("loadedmetadata");
                _this3.loaded = true;
            });

            this.on('timeupdate', function () {
                html.hide(_this3.player.overlays.loading);
                _this3.player.video.currentTime > 0 ? html.hide(_this3.player.overlays.cover) : html.show(_this3.player.overlays.cover);
                if (typeof _this3.options.onTimeUpdate !== 'undefined') {
                    _this3.options.onTimeUpdate(_this3.player.video.currentTime);
                }
            });

            this.on('error', function () {
                html.show(_this3.player.overlays.error);
            });

            var _loop = function _loop(i) {
                video.addEventListener(_this3.events.videoEvents[i], function () {
                    if (_this3.events && _this3.events.trigger) {
                        _this3.events.trigger(_this3.events.videoEvents[i]);
                    }
                });
            };

            for (var i = 0; i < this.events.videoEvents.length; i++) {
                _loop(i);
            }
        }

        /**
         * @method updatePlayTime 更新播放时间
         * @param {current} current  eg: 02:29 
         * @param {total} total  eg: 02:29 
         * @returns {void} void
         */

    }, {
        key: 'updatePlayTime',
        value: function updatePlayTime(current, total) {
            this.player.texts.current.innerHTML = utils.currentSecondToTime(current, total);
        }

        /**
         * @method updatePlayTime 更新视频的总播放时间
         * @param {total} total  eg:  /04:29
         * @returns {void} void
         */

    }, {
        key: 'updateTotalPlayTime',
        value: function updateTotalPlayTime(total) {
            this.player.texts.total.innerHTML = utils.secondToTime(total);
        }

        /**
         * @method getStatus get video status
         * @returns {bool} 是否暂停
         */

    }, {
        key: 'getStatus',
        value: function getStatus() {
            return this.player.video.paused;
        }

        /**
         * @method Seek seek video
         * @param {time} time  eg: 80(s)
         * @returns {void} void
         */

    }, {
        key: 'seek',
        value: function seek(time) {
            if (this.player.video != null) {
                //TODO:被重新loaded之后获取不到duration值，导致时间无法显示
                if (!this.loaded) {
                    this.player.video.pause();
                    this.player.video.load();
                    this.loaded = true;
                }

                time = Math.max(time, 0);
                if (this.getDuration()) {
                    time = Math.min(time, this.getDuration());
                }

                this.player.video.currentTime = time;
                if (time > 0) {
                    html.hide(this.player.overlays.cover);
                } else {
                    html.show(this.player.overlays.cover);
                }

                this.updateBar('played', time / this.getDuration(), 'width');
                this.updatePlayTime(time, this.getDuration());

                if (typeof this.options.onSeek !== 'undefined') {
                    this.options.onSeek(time);
                }
            }
        }

        /**
         * @method end seek video
         * @param {time} time  eg: 80(s)
         * @returns {void} void
         */

    }, {
        key: 'end',
        value: function end(time) {
            if (this.player.video != null) {
                this.seek(this.getDuration());
            }
        }

        /**
         * @method getMedia 获取video对象
         * @returns {video} video
         */

    }, {
        key: 'getMedia',
        value: function getMedia() {
            return this.player.video;
        }

        /**
         * @method getView 获取组件componentVideo
         * @returns {void} void
         */

    }, {
        key: 'getView',
        value: function getView() {
            return this.componentVideo;
        }

        /**
         * @method play 播放视频
         * @returns {void} void
         */

    }, {
        key: 'play',
        value: function play() {
            var that = this;
            if (this.player.video != null) {
                if (!this.loaded) {
                    this.player.video.pause();
                    this.player.video.load();
                    this.loaded = true;
                }
                if (this.player.video.paused && !this.options.allowSelected) {
                    html.hide(this.player.overlays.button);
                }
                this.player.video.play();
                this.setTime();
                this.player.buttons.play.classList.remove('video-btn-play');
                this.player.buttons.play.classList.add('video-btn-pause');
                html.hide(this.player.overlays.cover);
                this.container.classList.add('player-playing');

                if (typeof this.options.onPlay !== 'undefined') {
                    this.options.onPlay();
                }
            }
        }
    }, {
        key: 'getDuration',
        value: function getDuration() {
            if (!isNaN(this.player.video.duration) && this.player.video.duration) {
                return this.player.video.duration;
            } else if (this.options.video.duration) {
                return this.options.video.duration;
            } else {
                return this.durationCache;
            }
        }

        /**
         * @method pause 暂停视频
         * @returns {void} void
         */

    }, {
        key: 'pause',
        value: function pause() {
            if (this.player.video != null) {
                var state = this.player.video.paused;
                if (!state && !this.options.allowSelected) {
                    html.show(this.player.overlays.button);
                }
                this.ended = false;
                this.player.buttons.play.classList.remove('video-btn-pause');
                this.player.buttons.play.classList.add('video-btn-play');
                if (this.player.video.currentTime > 0) {
                    html.hide(this.player.overlays.cover);
                }
                if (!state) {
                    this.player.video.pause();
                }
                this.clearTime();
                this.container.classList.remove('player-playing');

                if (typeof this.options.onPause !== 'undefined' && !state) {
                    this.options.onPause();
                }
            }
        }

        /**
         * @method toggle 播放/暂停视频
         * @returns {void} void
         */

    }, {
        key: 'toggle',
        value: function toggle() {
            if (this.player.video.paused) {
                this.play();
            } else {
                this.pause();
            }
        }

        /**
         * @method volume 设置音量
         * @param {percentage} 音量 eg: (0.1-1)
         * @returns {void} void
         */

    }, {
        key: 'volume',
        value: function volume(percentage) {
            percentage = parseFloat(percentage);
            if (!isNaN(percentage)) {
                percentage = percentage > 0 ? percentage : 0;
                percentage = percentage < 1 ? percentage : 1;
                this.updateBar('volume', percentage, 'height');
                var formatPercentage = (percentage * 100).toFixed(0) + '%';
                this.player.bars.volume.style.height = formatPercentage;

                this.player.video.volume = percentage;
                if (this.player.video.muted) {
                    this.player.video.muted = false;
                }
                this.switchVolumeIcon();
                if (typeof this.options.onVolumeChange !== 'undefined') {
                    this.options.onVolumeChange(percentage);
                }
            }

            return this.player.video.volume;
        }

        /**
         * @method changeVolume 设置音量
         * @param {percentage} 音量 eg: (0.1-1)
         * @returns {void} void
         */

    }, {
        key: 'changeVolume',
        value: function changeVolume(percentage) {
            this.volume(percentage, 'height');
        }

        /**
         * @method setLock 锁定
         * @param {lock} lock 锁定：true, 不锁定 ：false
         * @returns {void} void
         */

    }, {
        key: 'setLock',
        value: function setLock(lock) {
            if (lock) {
                this.componentVideo.classList.add('pointer-events-none');
            } else {
                this.componentVideo.classList.remove('pointer-events-none');
            }

            if (typeof this.options.onSetLock !== 'undefined') {
                this.options.onSetLock(lock);
            }
        }
    }, {
        key: 'on',
        value: function on(name, callback) {
            this.events.on(name, callback);
        }

        /**
         * @method reset 重置
         * @param {pbar} 播放轨迹栏
         * @param {timeTips} 时间tip 
         * @returns {void} void
         */

    }, {
        key: 'mouseHandler',
        value: function mouseHandler(pbar, timeTips) {
            var _this4 = this;

            var cumulativeOffset = function cumulativeOffset(element) {
                var top = 0,
                    left = 0;
                do {
                    top += element.offsetTop || 0;
                    left += element.offsetLeft || 0;
                    element = element.offsetParent;
                } while (element);

                return {
                    top: top,
                    left: left
                };
            };

            return function (e) {
                if (!_this4.player.video.duration) {
                    return;
                }
                var clientX = e.clientX;

                var px = cumulativeOffset(pbar).left;
                var tx = clientX - px;
                if (tx < 0 || tx > pbar.offsetWidth) {
                    return;
                }

                //是否显示当前时间tip 
                var timeTipsDisplay = function timeTipsDisplay(show, timeTips) {
                    if (show) {
                        if (_this4.isTimeTipsShow) {
                            return;
                        }
                        html.show(timeTips);
                        _this4.isTimeTipsShow = true;
                    } else {
                        if (!_this4.isTimeTipsShow) {
                            return;
                        }
                        html.hide(timeTips);
                        _this4.isTimeTipsShow = false;
                    }
                };

                var time = _this4.getDuration() * (tx / pbar.offsetWidth);

                if (tx - timeTips.offsetWidth / 2 < 0) {
                    tx = timeTips.offsetWidth / 2;
                }

                if (tx + timeTips.offsetWidth / 2 > pbar.offsetWidth) {
                    tx = pbar.offsetWidth - 20;
                }

                timeTips.style.left = tx + 'px';

                switch (e.type) {
                    case 'mouseenter':
                        break;
                    case 'mousemove':
                        _this4.player.texts.timetip.innerText = utils.currentSecondToTime(time, _this4.getDuration());
                        timeTipsDisplay(true, timeTips);
                        break;
                    case 'mouseleave':
                        timeTipsDisplay(false, timeTips);
                        break;
                    case 'mouseout':
                        timeTipsDisplay(false, timeTips);
                        break;
                }
            };
        }

        /**
         * @method reset 重置
         * @returns {void} void
         */

    }, {
        key: 'reset',
        value: function reset() {
            if (this.player.video != null) {
                this.pause();
            }
            if (this.fullScreen.isFullScreen()) {
                this.fullScreen.cancel();
            }
        }

        /**
         * @method resize 调整大小 fullscreen.resize 
         * @returns {void} void
         */

    }, {
        key: 'resize',
        value: function resize() {
            this.events.trigger('resize');
        }

        /**
         * @method fullscreen 全屏
         * @returns {void} void
         */

    }, {
        key: 'fullscreen',
        value: function fullscreen() {
            this.fullScreen.request();
        }

        /**
         * @method exitFullscreen 退出全屏
         * @returns {void} void
         */

    }, {
        key: 'exitFullscreen',
        value: function exitFullscreen() {
            this.fullScreen.cancel();
        }
    }, {
        key: 'getScale',
        value: function getScale() {
            var scale = this.container.getBoundingClientRect().width / this.container.offsetWidth || 1;
            return scale;
        }

        /**
         * @method destroy 销毁
         * @returns {void} void
         */

    }, {
        key: 'destroy',
        value: function destroy() {
            this.pause();
            clearTimeout(this.hideTime);
            this.player.video.src = '';
            this.container.innerHTML = '';
            this.events.trigger('destroy');

            for (var key in this) {
                if (this.hasOwnProperty(key) && key !== 'paused') {
                    delete this[key];
                }
            }
        }
    }]);
    return VPlayer;
}();

var _$1 = {
    TOP_VIEWER: null,
    INSTANCES: {},
    clear: function clear(key) {
        var instance = this.INSTANCES[key];
        if (instance) {
            instance.destroy();
        }
        this.INSTANCES[key] = null;
        delete this.INSTANCES[key];
    },
    getKeyId: function getKeyId(key) {
        return key.split('|')[1];
    }

    /**
     * @class VideoPlayer 视频播放器
     * @description 对VPlayer的封装
     */

};var VideoPlayer = {

    setLoggerLevel: function setLoggerLevel(level) {},
    render: function render(container, options) {
        container = container.length > 0 ? container[0] : container;
        for (var key in _$1.INSTANCES) {
            if (!document.querySelector('[data-id="' + _$1.getKeyId(key) + '"]')) {
                _$1.clear(key);
            }
        }
        options = options || {};
        if (container) {
            var items = container.nodeName.toLowerCase() == 'video' ? [container] : container.getElementsByTagName('video');
            $.each(items, function (item, index) {
                var dataId = $.getRandomId('media');
                var mediaId = 'video';
                var containerId = options.containerId || '';
                mediaId = mediaId + '-' + containerId + '-' + index;

                var key = mediaId + '|' + dataId;
                var newOptions = $.extend({}, options, {
                    dataId: dataId,
                    mediaId: mediaId,
                    key: key
                });

                var instance = new VPlayer$1(newOptions);
                instance.render();
                _$1.INSTANCES[key] = instance;
            });
        } else {
            Logger.debug('no container');
        }
        return _$1.INSTANCES;
    },
    destroy: function destroy() {
        var instances = _$1.INSTANCES;
        for (var i = 0, len = instances.length; i < len; i++) {
            instances.destroy();
        }
        _$1.INSTANCES = {};
    },
    reset: function reset() {
        // this.getInstance().reset();
    },
    setLock: function setLock(lock) {
        // this.getInstance().setLock(lock);
    },
    getMedia: function getMedia() {
        // return this.getInstance().getMedia();
    },
    mediaPlay: function mediaPlay() {
        // this.getInstance().play();
    },
    mediaPause: function mediaPause() {
        // this.getInstance().pause();
    },
    changeVolume: function changeVolume(volume) {
        // this.getInstance().volume(volume);
    },
    resizeMedia: function resizeMedia(width, height) {
        // this.getInstance().changeSize(width, height);
    },
    create: function create(video, options) {
        return new VPlayer$1(options);
    },
    Video: {
        create: function create(options) {
            return new VPlayer$1(options);
        }
    }
};

var utils$1 = defineProperty({

    throttle: function throttle(func, wait) {
        var context, args, timeout, result;
        var previous = 0;
        var later = function later() {
            previous = new Date();
            timeout = null;
            result = func.apply(context, args);
        };
        return function () {
            var now = new Date();
            var remaining = wait - (now - previous);
            context = this;
            args = arguments;
            if (remaining <= 0) {
                clearTimeout(timeout);
                timeout = null;
                previous = now;
                result = func.apply(context, args);
            } else if (!timeout) {
                timeout = setTimeout(later, remaining);
            }
            return result;
        };
    },
    /**
     * @method addEventHandler 事件监听
     * @param {dom} objectelement element
     * @param {string} eventname 事件 eg:"mousedown touchstart"
     * @param {function} eventhandler function(e) {}
     * @return {void} void
     */
    addEventHandler: function addEventHandler(objectelement, eventname, eventhandler) {
        var eventnames = eventname.split(' ');
        for (var i = 0, l = eventnames.length; i < l; i++) {
            var name = eventnames[i];
            if (document.attachEvent) {
                //ie浏览器
                objectelement.attachEvent("on" + name, eventhandler);
            } else if (document.addEventListener) {
                //非ie浏览器
                objectelement.addEventListener(name, eventhandler, false);
            }
        }
    },
    removeEventHandler: function removeEventHandler(objectelement, eventname, eventhandler) {
        var eventnames = eventname.split(' ');
        for (var i = 0, l = eventnames.length; i < l; i++) {
            var name = eventnames[i];
            if (document.detachevent) {
                //ie浏览器
                objectelement.detachevent("on" + name, eventhandler);
            } else if (document.addEventListener) {
                //非ie浏览器
                objectelement.removeEventListener(name, eventhandler, false);
            }
        }
    },
    insertAfter: function insertAfter(newElement, targetElement) {
        var parent = targetElement.parentNode;
        if (parent.lastChild == targetElement) {
            // 如果最后的节点是目标元素，则直接添加。因为默认是最后  
            parent.appendChild(newElement);
        } else {
            parent.insertBefore(newElement, targetElement.nextSibling);
            //如果不是，则插入在目标元素的下一个兄弟节点 的前面。也就是目标元素的后面  
        }
    },

    // 是否不是用户输入
    isNotUserInput: function isNotUserInput(textinput) {
        return textinput == null || textinput.toLowerCase() !== "userinput";
    },
    // 是否用户输入
    isUserInput: function isUserInput(textinput) {
        return textinput != null && textinput.toLowerCase() == "userinput";
    },
    /**
    * @method secondToTime 把 秒 解析 成 00:00 格式 
    * @param {Number} second
    * @return {String} 00:00 format
    */
    timeFormat: function timeFormat(timeString) {
        var time = timeString.trim();
        var h = '0';
        var m = '00';
        var s = '00';
        var ms = '00';
        var times = time.split(/(?:(\d{1,2}):)?(\d{2}):(\d{2})[,\.](\d{2,3})/);
        if (times.length >= 6) {
            h = times[1] ? times[1].substring(0, 1) : '0';
            m = times[2];
            s = times[3];
            ms = times[4].length === 3 ? times[4].substring(0, 2) : times[4];
        }

        return h + ':' + m + ':' + s + '.' + ms;
    },
    /**
    * @method secondToTime 把 秒 解析 成 00:00 格式 
    * @param {Number} second
    * @return {String} 00:00 format
    */
    secondToTime: function secondToTime(second) {
        if (isNaN(second)) {
            return '00:00';
        }
        var add0 = function add0(num) {
            return num < 10 ? '0' + num : '' + num;
        };
        var min = parseInt(second / 60);
        var sec = parseInt(second - min * 60);
        return add0(min) + ':' + add0(sec);
    },
    /**
    * @method getRandomId 随机数
    * @param {Number} second
    * @return {String} 00:00 format
    */
    getRandomId: function getRandomId(name) {
        return name + (Math.random() * 10 + new Date() * 1).toString().replace('.', '');
    },
    getOffset: function getOffset(text) {
        var offset = 0;
        try {
            var offsetPattern = /\[offset:\-?\+?\d+\]/g,
                offset_line = text.match(offsetPattern)[0],
                offset_str = offset_line.split(':')[1];
            offset = parseInt(offset_str);
        } catch (err) {
            offset = 0;
        }
        return offset;
    },
    /**
    * @method timeToSecond [00:04.00]
    * @param {string} text
    * @return {string} result
    */
    timeToSecond: function timeToSecond(text) {
        //get each line from the text
        var result = 0;
        var t = text.slice(1, -1).split(':');
        result = parseInt(t[0] == '' ? 0 : t[0], 10) * 60 * 60 + parseInt(t[1] == '' ? 0 : t[1], 10) * 60 + parseFloat(t[2]);
        return result;
    },
    /**
    * @method parse [00:04.00]歌词歌词歌词
    * @param {text} text
    * @return {array} result
    */
    parse: function parse(text) {
        //get each line from the text
        var lines = text.split('\n'),

        //this regex mathes the time [00.12.78]
        pattern = /\[\d{2}:\d{2}.\d{2}\]/g,
            result = [];

        var offset = this.getOffset(text);
        while (!pattern.test(lines[0])) {
            lines = lines.slice(1);
        }
        lines[lines.length - 1].length === 0 && lines.pop();
        lines.forEach(function (v, i, a) {
            var time = v.match(pattern),
                value = v.replace(pattern, '').replace(/^\s+|\s+$/g, '');
            if (time != null) {
                time.forEach(function (v1, i1, a1) {
                    //convert the [min:sec] to secs format then store into result
                    var t = v1.slice(1, -1).split(':');
                    var newOffset = parseInt(t[0], 10) * 60 + parseFloat(t[1]) + parseInt(offset) / 1000;
                    result.push({ offset: newOffset, text: value.replace('', '&nbsp;') });
                });
            }
        });
        result.sort(function (a, b) {
            return a.offset - b.offset;
        });
        return result;
    },
    /**
    * @method getFileName 根据路径获取文件名
    * @param {path} path 路径
    * @return {String} 00:00 format
    */
    getFileName: function getFileName(path) {
        var paths = path.split('/');
        if (path.indexOf('\\') > -1) {
            paths = path.split('\\');
        }
        return paths[paths.length - 1];
    },
    /**
    * @method createDom 创建dom
    * @param {str} html 
    * @return {dom} 返回dom对象
    */
    createDom: function createDom(str) {
        var dom = document && document.createElement('div');
        dom.innerHTML = str;
        return dom.firstChild;
    },
    hideOverflow: function hideOverflow(element) {
        var current = element.parentElement;
        while (current !== null) {
            current = current.parentElement;
            current.style.overflow = 'hidden';
        }
        return scrollTop;
    },
    autoOverflow: function autoOverflow(element) {
        var current = element.parentElement;
        while (current !== null) {
            current = current.parentElement;
            current.style.overflow = 'auto';
        }
        return scrollTop;
    },
    /**
    * @method getScrollPosition 获取window 滚动位置
    * @return {number} 返回window 滚动位置
    */
    getScrollPosition: function getScrollPosition() {
        return {
            left: window.pageXOffset || document.documentElement.scrollLeft || document.body.scrollLeft || 0,
            top: window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop || 0
        };
    },

    /**
    * @method setScrollPosition 获取window 滚动位置
    * @param {left} 
    * @param {top} 
    * @return {number} 返回window 滚动位置
    */
    setScrollPosition: function setScrollPosition(_ref) {
        var _ref$left = _ref.left,
            left = _ref$left === undefined ? 0 : _ref$left,
            _ref$top = _ref.top,
            top = _ref$top === undefined ? 0 : _ref$top;

        if (this.isFirefox) {
            document.documentElement.scrollLeft = left;
            document.documentElement.scrollTop = top;
        } else {
            window.scrollTo(left, top);
        }
    },

    /**
    * @method scroll 获取window 滚动位置
    * @param {timeId} timeId timer ID
    * @param {obj} 滚动对象
    * @param {crt} 当前位置
    * @param {dst} 最终位置
    * @param {step} 步长
    * @param {fr} 步长
    * @return {number} 返回window 滚动位置
    */
    scroll: function scroll(timeId, obj, crt, dst, step, frequency) {
        clearInterval(timeId);
        timeId = setInterval(function () {
            if (crt < dst) {
                obj.scrollTop += step;
                crt += step;
            }
        }, frequency);
        return timeId;
    },

    /**
     * 检查是否用户是否用手机
     */
    isMobile: /mobile/i.test(window.navigator.userAgent),

    isMobile1: /Android|webOS|iPhone|iPod|BlackBerry/i.test(window.navigator.userAgent),
    /**
     * 检查是否用户是否用Firefox
     */
    isFirefox: /firefox/i.test(window.navigator.userAgent),
    /**
     * 检查是否用户是否用chrome
     */
    isChrome: /chrome/i.test(window.navigator.userAgent),
    /**
     * localStorage 存储
     */
    storage: {
        set: function set$$1(key, value) {
            localStorage.setItem(key, value);
        },

        get: function get$$1(key) {
            return localStorage.getItem(key);
        }
    }
}, 'getOffset', function getOffset(elem) {
    var docElem,
        win,
        box = {
        top: 0,
        left: 0
    },
        doc = elem && elem.ownerDocument;

    if (!doc) {
        return;
    }

    docElem = doc.documentElement;
    // If we don't have gBCR, just use 0,0 rather than error
    // BlackBerry 5, iOS 3 (original iPhone)
    if (_typeof(elem.getBoundingClientRect) !== (typeof undefined === 'undefined' ? 'undefined' : _typeof(undefined))) {
        box = elem.getBoundingClientRect();
    }
    win = doc == doc.window ? doc : doc.nodeType === 9 ? doc.defaultView || doc.parentWindow : false;
    return {
        top: box.top + (win.pageYOffset || docElem.scrollTop) - (docElem.clientTop || 0),
        left: box.left + (win.pageXOffset || docElem.scrollLeft) - (docElem.clientLeft || 0)
    };
});

/*
 * xhr.status ---> fail
 * response.code === 1 ---> success
 * response.code !== 1 ---> error
 * */

var SendXMLHttpRequest$1 = function SendXMLHttpRequest(url, data, success, error, fail) {
    var xhr = new XMLHttpRequest();

    xhr.onreadystatechange = function () {
        if (xhr.readyState === 4) {
            if (xhr.status >= 200 && xhr.status < 300 || xhr.status === 304) {
                return success(xhr, xhr);
            }
            fail(xhr);
        }
    };

    xhr.open(data !== null ? 'POST' : 'GET', url, true);
    xhr.send(data !== null ? JSON.stringify(data) : null);
};

var API = {
    send: function send(endpoint, danmakuData, callback) {
        SendXMLHttpRequest$1(endpoint, danmakuData, function (xhr, response) {
            console.log('Request success: ', response);
            if (callback) {
                callback();
            }
        }, function (xhr, response) {
            console.log('Request was error: ' + xhr.status);
        }, function (xhr) {
            console.log('Request was unsuccessful: ' + xhr.status);
        });
    },

    read: function read(endpoint, callback) {
        SendXMLHttpRequest$1(endpoint, null, function (xhr, response) {
            callback({ status: xhr.status, response: response.responseText });
        }, function (xhr, response) {
            callback({ status: xhr.status, response: response });
        }, function (xhr) {
            callback({ status: xhr.status, response: xhr.responseText });
        });
    }
};

// const http = (uri, method, data, headers) => {
//     return new Promise(function (resolve, reject) {
//         var xhr = new XMLHttpRequest();
//         xhr.open(method, uri, true);
//         if (headers) {
//             for (var p in headers) {
//                 xhr.setRequestHeader(p, headers[p]);
//             }
//         }
//         xhr.addEventListener('readystatechange', function (e) {
//             if (xhr.readyState === 4) {
//                 if (String(xhr.status).match(/^2\d\d$/)) {
//                     resolve(xhr.responseText);
//                 } else {
//                     resolve(xhr.responseText);
//                 }
//             }
//         });
//         xhr.send(data);
//     })
// }

// export default {
//     get: (uri) => {
//         return http(uri, 'GET', null);
//     }
// }

// ${html.audio(options.preload, options.audio.url, options)}
var html$2 = {
    main: function main(options, tran) {
        return '\n    <!-- audio\u6807\u7B7E\u5BB9\u5668 -->\n    <span class="audio-resources-wrap"  cid="audio-mediaelement">\n    </span>\n    <span class="audio-layer" cid="audio-layer-btn-play">\n        <span class="audio-layer-button" cid="audio-layer-play-btn"></span>\n    </span>\n    <!-- \u5DE6\u5074\u64AD\u653E\u63A7\u5236 -->\n    <span class="audio-tool">\n        <span class="audio-btn audio-btn-play">\n            <i class="audio-icon audio-icon-play"  cid="audio-btn-play"></i>\n        </span>\n    </span>\n    <span class="audio-play" cid="audio-play">\n        <span class="audio-play__name">\n            <span class="audio-play__name-word" cid="audio-play-name">' + options.audio.title + '</span>\n        </span>\n        <span class="audio-play__view">\n            <span class="audio-rail" cid="audio-rail">\n                <span class="audio-rail__current" cid="audio-rail-current" style="width: 0%;">\n                    <span class="audio-point"></span>\n                </span>\n            </span>\n            <span class="audio-play__time">\n                <span class="audio-play__time-word">\n                    <span class="audio-text audio-text-current" cid="audio-play-time">00:05:36</span>\n                    <span class="audio-text audio-text-split">/</span>\n                    <span class="audio-text audio-text-total" cid="audio-time-total">00:10:00</span>\n                </span>\n            </span>\n            <span class="audio-play__other">\n                <span class="audio-volume" cid="audio-volume">\n                    <i class="audio-icon audio-icon-volume" cid="audio-icon-volume"></i>\n                    <span class="audio-silde" cid="audio-silde">\n                        <span class="audio-silde__bg"></span>\n                        <span class="audio-silde__content" cid="audio-silde-content">\n                            <span class="audio-silde__bar" cid="audio-silde-bar">\n                                <span class="audio-silde__value" cid="audio-silde-value" style="height: ' + options.volume * 100 + '%;">\n                                    <span class="audio-point audio-point--small"></span>\n                                </span>\n                            </span>\n                        </span>\n                    </span>\n                </span>\n                <span class="audio-screen ' + (options.showFullscreenBtn ? '' : 'hide_dom') + '" cid="audio-screen-container">\n                    <i class="audio-icon audio-icon-screen audio-icon-enter-screen" cid="audio-screen"></i>\n                </span>\n                <span class="audio-lrc" cid="audio-lrc-container">\n                    <i class="audio-icon audio-icon-lrc" cid="audio-lrc"></i>\n                </span>\n            </span>\n        </span>\n    </span>\n    <span class="autext-pop hide_dom" cid="autext-pop">\n        <span class="autext-pop-wrap" cid="autext-pop-wrap">\n            <span class="autext-pop-paper ">\n                <span class="autext-textwrap" cid="autext-textwrap">\n                    <span class="autext-content" cid="autext-content">\n                    </span>\n                </span>\n                <a href="javascript:;" class="autext-pop-close" cid="autext-pop-close"></a>\n            </span>\n        </span>\n    </span>\n    ';
    },
    audio: function audio(preload, url, options) {
        var version = new Date().getTime();
        return '<audio \n        class="player-audio player-audio-current"\n        cid="player-audio"\n        title="' + (options.audio.title ? options.audio.title : '') + '"\n        lyric="' + (options.audio.lyric ? options.audio.lyric : '') + '"\n        preload="' + preload + '"\n        duration="' + options.audio.duration + '"\n        src="' + url + '"\n        ></audio>';
    },
    subtitle: function subtitle(textTracks) {
        var result = "";
        if (textTracks != null) {
            for (var k = 0, length = textTracks.length; k < length; k++) {
                result += '<span class="autext-content__item"><span class="autext-content__item-text">' + textTracks[k].text + '</span></span>';
            }
        }
        return result;
    },
    show: function show(obj) {
        obj.classList.remove('hide_dom');
    },
    hide: function hide(obj) {
        obj.classList.add('hide_dom');
    },
    getDoms: function getDoms(obj) {
        var audio = obj.querySelector("audio[cid='player-audio']");

        var buttons = {};
        buttons.layerPlay = obj.querySelector("[cid='audio-layer-btn-play']");
        buttons.layerPlayBtn = obj.querySelector("[cid='audio-layer-play-btn']");
        buttons.play = obj.querySelector("i[cid='audio-btn-play']");
        buttons.subtitle = obj.querySelector("i[cid='audio-lrc']");
        buttons.subtitleContainer = obj.querySelector("span[cid='audio-lrc-container']");
        buttons.closeSubtitle = obj.querySelector("a[cid='autext-pop-close']");
        buttons.volume = obj.querySelector("i[cid='audio-icon-volume']");
        buttons.screen = obj.querySelector("i[cid='audio-screen']");

        var bar = {};
        bar.played = obj.querySelector("span[cid='audio-rail-current']");
        bar.volume = obj.querySelector("span[cid='audio-silde-value']");
        bar.rail = obj.querySelector("span[cid='audio-rail']");
        bar.volumeSilde = obj.querySelector("span[cid='audio-silde-bar']");

        var timetip = obj.querySelector("span[cid='video-rail-float']");

        var pop = {};
        pop.container = obj.querySelector("span[cid='autext-pop']");
        pop.wrap = obj.querySelector("span[cid='autext-pop-wrap']");
        pop.textwrap = obj.querySelector("span[cid='autext-textwrap']");

        var volume = {};
        // 先判断是否为iOS
        var u = navigator.userAgent;
        var isIOS = !!u.match(/\(i[^;]+;( U;)? CPU.+Mac OS X/); //ios终端
        if (isIOS) {
            // 隐藏音量调整条
            obj.querySelector("span[cid='audio-silde']").setAttribute('style', 'display: none !important');
            volume.sildeBarHidden = true;
        } else {
            volume.sildeBarHidden = false;
        }

        volume.sildeContent = obj.querySelector("span[cid='audio-silde-content']");
        volume.sildeBar = obj.querySelector("span[cid='audio-silde-bar']");
        volume.controler = obj.querySelector("span[cid='audio-volume']");

        var texts = {};
        texts.time = obj.querySelector("span[cid='audio-play-time']");
        texts.total = obj.querySelector("span[cid='audio-time-total']");
        texts.title = obj.querySelector("span[cid='audio-play-name']");

        return {
            audio: audio,
            bars: bar,
            texts: texts,
            buttons: buttons,
            timetip: timetip,
            volume: volume,
            pop: pop

        };
    }
};

function i18n$1(lang) {
    var _this = this;

    this.lang = lang;
    if (typeof lang === 'string') {
        this.tran = function (text) {
            if (tranTxt$1[_this.lang] && tranTxt$1[_this.lang][text]) {
                return tranTxt$1[_this.lang][text];
            } else {
                return text;
            }
        };
    } else {
        this.tran = function (text) {
            return _this.lang[text];
        };
    }
}

var tranTxt$1 = {
    "zh-cn": {}
};

var handleOption$1 = function (options) {
    var isMobile = /mobile/i.test(window.navigator.userAgent);
    if (isMobile) {
        options.autoplay = false;
    }
    var defaultOption = {
        container: options.element || document.getElementsByClassName('nd-audio-player')[0],
        fullContainer: document.body,
        autoplay: false,
        theme: '#b7daff',
        loop: false,
        hotkey: true,
        preload: 'auto',
        volume: 0.7,
        apiBackend: API,
        extendStyle: '',
        audio: {},
        muted: true,
        native: true,
        showFullscreenBtn: true // 是否显示全屏按钮
    };

    options.width = options.width == 0 || options.width == undefined ? defaultOption.width : options.width;

    for (var defaultKey in defaultOption) {
        if (defaultOption.hasOwnProperty(defaultKey) && !options.hasOwnProperty(defaultKey)) {
            options[defaultKey] = defaultOption[defaultKey];
        }
    }
    if (options.video) {
        !options.video.type && (options.video.type = 'auto');
    }
    if (options.subtitle) {
        !options.subtitle.type && (options.subtitle.type = 'webvtt');
        !options.subtitle.fontSize && (options.subtitle.fontSize = '20px');
        !options.subtitle.bottom && (options.subtitle.bottom = '40px');
        !options.subtitle.color && (options.subtitle.color = '#fff');
    }

    return options;
};

var Events$1 = function () {
    function Events() {
        classCallCheck(this, Events);

        this.events = {};

        this.audioEvents = ['timeupdate', 'abort', 'canplay', 'canplaythrough', 'durationchange', 'emptied', 'ended', 'error', 'loadeddata', 'loadedmetadata', 'loadstart', 'mozaudioavailable', 'pause', 'play', 'playing', 'progress', 'ratechange', 'seeked', 'seeking', 'stalled', 'suspend', 'volumechange', 'waiting'];
        this.playerEvents = ['screenshot', 'thumbnails_show', 'thumbnails_hide', 'danmaku_show', 'danmaku_hide', 'danmaku_clear', 'danmaku_loaded', 'danmaku_send', 'danmaku_opacity', 'contextmenu_show', 'contextmenu_hide', 'notice_show', 'notice_hide', 'quality_start', 'quality_end', 'destroy', 'resize', 'fullscreen', 'fullscreen_cancel', 'webfullscreen', 'webfullscreen_cancel', 'title_show', 'title_hide', 'title_change', 'imageopen'];
    }

    createClass(Events, [{
        key: 'on',
        value: function on(name, callback) {
            if (this.type(name) && typeof callback === 'function') {
                if (!this.events[name]) {
                    this.events[name] = [];
                }
                this.events[name].push(callback);
            }
        }
    }, {
        key: 'trigger',
        value: function trigger(name, info) {
            if (this.events[name] && this.events[name].length) {
                for (var i = 0; i < this.events[name].length; i++) {
                    this.events[name][i](info);
                }
            }
        }
    }, {
        key: 'type',
        value: function type(name) {
            if (this.playerEvents.indexOf(name) !== -1) {
                return 'player';
            } else if (this.audioEvents.indexOf(name) !== -1) {
                return 'audio';
            }

            console.error('Unknown event name: ' + name);
            return null;
        }
    }]);
    return Events;
}();

//base64加密 解密

/* //1.加密  
var result = Base.encode('125中文');  //--> "MTI15Lit5paH"
  
//2.解密  
var result2 = Base.decode(result); //--> '125中文'
*/

var Parser = {
  /**
   * @method 解析歌词
   * @param {string} caption 歌词文本
   * @returns {object} 歌词信息
   */
  parse: function parse(caption) {
    var result = [];

    // 解析歌词
    var regex = /((?:\[(?:\d:)?\d{2}:\d{2}\.\d{2,3}])+)/g;
    var texts = caption.replace(/\r/g, '').split(regex);
    texts.shift();
    for (var i = 0; i < texts.length; i += 2) {
      var times = texts[i].trim().split(/\[(\d:\d{2}:\d{2}\.\d{2,3})]/g);
      for (var k = 0; k < times.length; k++) {
        if (times[k]) {
          result.push({
            id: '',
            startTime: utils$1.timeFormat(times[k]),
            endTime: null,
            text: texts[i + 1].trim()
          });
        }
      }
    }

    // 解析信息
    var infoRegex = /\[([a-zA-Z]+):(.+)]/g;
    var info = caption.replace(/\r/g, '').split(infoRegex);
    var infoResult = {};
    info.shift();
    for (var _i = 0; _i < info.length; _i += 3) {
      infoResult[info[_i]] = info[_i + 1];
    }

    // 重新排序
    result.sort(function (a, b) {
      return a.startTime > b.startTime ? 1 : -1;
    });

    // 添加id属性
    for (var _i2 = 0; _i2 < result.length; _i2++) {
      result[_i2].id = _i2 + 1 + '';
    }

    return {
      info: infoResult,
      texts: result
    };
  }
};

/**
 * @class Subtitle 字幕
 * @description 字幕 基本功能
 */

var Subtitle = function () {
    function Subtitle(container, player, options, events, tran) {
        classCallCheck(this, Subtitle);

        this.container = container;
        this.wrap = player.pop.wrap;
        this.player = player;
        this.audio = player.audio;
        this.options = options;
        this.events = events;
        this.tran = tran;
        this.popShow = false;
        this.init();
        this.parentContainer = this.container.parentElement;
    }

    /**
     * @method getLyric 渲染组件
     * @returns {lyric} 字幕
     */


    createClass(Subtitle, [{
        key: 'getLyric',
        value: function getLyric() {
            return this.list;
        }

        /**
         * @method init 初始化
         * @param {options} 选项
         * @returns {void} void
         */

    }, {
        key: 'init',
        value: function init() {
            var _this = this;

            if (!this.options.audio.lyric) {
                throw new Error("获取歌词数据失败");
                return;
            }

            this.autextContent = this.container.querySelector("span[cid='autext-content']");
            this.autextTextwrap = this.container.querySelector("span[cid='autext-textwrap']");
            this.preLineNo = 0;
            this.lineNo = 0;
            this.autextItemHeight = 54;
            this.frequency = 20;
            var that = this;

            var render = function render(text) {

                if (text == null || text == "" || text == undefined) {
                    _this.player.buttons.subtitle.classList.add("click_disabled");
                    return;
                }

                var obj = Parser.parse(text);
                _this.list = obj.texts;
                _this.info = obj.info;

                if (_this.list.length == 0) {
                    _this.player.buttons.subtitle.classList.add("click_disabled");
                    return;
                }
                var innerHTML = html$2.subtitle(_this.list);
                _this.autextContent.innerHTML = '';
                _this.autextContent.innerHTML = innerHTML;
                if (_this.list != null && _this.list.length > 0) {
                    _this.autextItemHeight = _this.autextContent.children[0].clientHeight == 0 ? 54 : _this.autextContent.children[0].clientHeight;
                }

                if (utils$1.isUserInput(_this.info.textinput)) {
                    var mouseEvents = utils$1.isMobile ? {
                        down: 'touchstart mousedown',
                        move: 'touchmove mousemove',
                        up: 'touchend mouseup',
                        over: 'touchstart',
                        out: 'touchend',
                        tap: 'touchstart click'
                    } : {
                        down: 'mousedown touchstart',
                        move: 'mousemove touchmove',
                        up: 'mouseup touchend',
                        over: 'mouseover',
                        out: 'mouseout',
                        tap: 'click touchstart'
                    };

                    _this.drag = false;
                    _this.scrollable = false;
                    utils$1.addEventHandler(_this.autextTextwrap, mouseEvents.down, function () {
                        _this.drag = true;
                    });

                    utils$1.addEventHandler(_this.autextTextwrap, mouseEvents.up, function () {
                        _this.drag = false;
                        _this.initScroll();
                    });
                }
            };

            // API.get(this.options.audio.lyric).then(function(data){
            //     if (data.length === 0) {
            //         that.player.buttons.subtitle.classList.add("click_disabled");
            //         return;
            //     }
            //     render(data);
            // }, function(data){
            //     that.player.buttons.subtitle.classList.add("click_disabled");
            //     throw new Error("加载歌词出错");
            // })

            API.read(this.options.audio.lyric, function (data) {
                if (data.status == 200 || data.status == 0 && data.response !== "") {
                    if (data.response.length === 0) {
                        that.player.buttons.subtitle.classList.add("click_disabled");
                        return;
                    }
                    render(data.response);
                } else {
                    that.player.buttons.subtitle.classList.add("click_disabled");
                    throw new Error("加载歌词出错");
                }
            });
        }
    }, {
        key: 'load',
        value: function load() {
            this.init();
        }

        /**
        * @method syncLyric 同步
        * @returns {void} void
        */

    }, {
        key: 'syncLyric',
        value: function syncLyric() {
            this.init(this.options);
        }
    }, {
        key: 'initScroll',
        value: function initScroll() {

            if (this.timeId != null) {
                clearInterval(this.timeId);
                // var time = this.audio.currentTime == null ? 0: this.audio.currentTime;
                // this.autextTextwrap.scrollTop = (time/this.audio.duration)*(this.autextContent.clientHeight - this.autextTextwrap.clientHeight/2);
            }
            if (this.popShow && !this.audio.paused && utils$1.isUserInput(this.info.textinput)) {
                var duration = isNaN(this.audio.duration) ? this.options.audio.duration : this.audio.duration;
                this.timeId = utils$1.scroll(this.timeId, this.autextTextwrap, this.autextTextwrap.scrollTop, this.autextContent.clientHeight, 1, duration - this.audio.currentTime);
            }

            if (this.popShow && this.audio.paused) {
                this.pause();
            }
        }

        /**
         * @method go 滚动到当前歌词行（lineno）
         * @param {lineNo} lineNo 行号
         * @param {isSeek} 是否seek
         * @returns {void} void
         */

    }, {
        key: 'go',
        value: function go(lineno) {
            var isSeek = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;


            this.lineNo = lineno;

            var currentItem = this.autextContent.children[this.lineNo];
            var preItem = this.autextContent.children[this.lineNo - 1];

            if (this.lineNo > 0) {
                preItem && preItem.classList.remove("ui_btn_active");
            }

            if (currentItem != null) {
                currentItem && currentItem.classList.add("ui_btn_active");
            }

            var preItemOffsetTop = typeof preItem == 'undefined' ? 0 : preItem.offsetTop;

            if (isSeek) {
                this.autextTextwrap.scrollTop = preItemOffsetTop;
                this.timeId = utils$1.scroll(this.timeId, this.autextTextwrap, preItemOffsetTop, currentItem.offsetTop, 1, this.frequency);
            }

            if (this.preLineNo != this.lineNo) {
                this.autextTextwrap.scrollTop = preItemOffsetTop;
                this.timeId = utils$1.scroll(this.timeId, this.autextTextwrap, preItemOffsetTop, currentItem.offsetTop, 1, this.frequency);
            }

            if (this.lineNo != this.preLineNo) {
                this.preLineNo = this.lineNo;
            }
        }
    }, {
        key: 'end',


        /**
         * @method end 播放结束
         * @returns {void} void
         */
        value: function end() {
            clearInterval(this.timeId);
            this.autextTextwrap.scrollTop = 0;
        }

        /**
         * @method sync 同步滚动到当前歌词行（time）
         * @param {time} time 
         * @param {isSeek} 是否seek
         * @returns {void} void
         */

    }, {
        key: 'sync',
        value: function sync(time) {
            var isSeek = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

            if (this.list != null && utils$1.isNotUserInput(this.info.textinput)) {
                var obj = this.find(time, isSeek);
                if (obj.flag) {
                    this.go(obj.lineNo, isSeek);
                }
            }
        }

        /**
         * @method find 查找
         * @returns {void} void
         */

    }, {
        key: 'find',
        value: function find(time) {
            var lineNo = 0,
                flag = false;
            for (var i = 0, l = this.list.length; i < l; i++) {
                if (this.list[i + 1] != null) {
                    var startTime = utils$1.timeToSecond(this.list[i].startTime);
                    var endTime = utils$1.timeToSecond(this.list[i + 1].startTime);
                    if (time > startTime && time < endTime) {
                        lineNo = i;
                        flag = true;
                    }
                } else {
                    if (time > this.list[i].startTime) {
                        lineNo = i;
                        flag = true;
                    }
                }
            }
            return {
                lineNo: lineNo,
                flag: flag
            };
        }

        /**
         * @method play 播放
         * @returns {void} void
         */

    }, {
        key: 'play',
        value: function play() {
            this.initScroll();
        }

        /**
         * @method pause 暂停
         * @returns {void} void
         */

    }, {
        key: 'pause',
        value: function pause() {
            clearInterval(this.timeId);
        }

        /**
         * @method seek 跳转
         * @param {time} time 
         * @returns {void} void
         */

    }, {
        key: 'seek',
        value: function seek(time) {
            this.clearHightLigth();
            this.sync(time, true);
        }

        /**
         * @method clearHightLigth 清空高亮
         * @returns {void} void
         */

    }, {
        key: 'clearHightLigth',
        value: function clearHightLigth() {
            if (this.autextContent.children != null) {
                for (var i = 0, l = this.autextContent.children.length; i < l; i++) {
                    var ep = this.autextContent.children[i];
                    ep.classList.remove("ui_btn_active");
                }
            }
        }

        /**
         * @method show 显示字幕
         * @returns {void} void
         */

    }, {
        key: 'show',
        value: function show() {
            this.popShow = true;
            this.container.classList.remove('hide_dom');
            this.player.buttons.subtitle.classList.add("ui-btn-active");
            document.body.appendChild(this.container);
            this.initScroll();
            this.events.trigger('title_show');
        }

        /**
         * @method offset 位置
         * @returns {void} void
         */

    }, {
        key: 'offset',
        value: function offset(left, top) {
            this.wrap.style.top = top;
            this.wrap.style.left = left;
        }

        /**
         * @method hide 隐藏字幕
         * @returns {void} void
         */

    }, {
        key: 'hide',
        value: function hide() {
            this.popShow = false;
            this.container.classList.add('hide_dom');
            this.player.buttons.subtitle.classList.remove("ui-btn-active");
            this.parentContainer.appendChild(this.container);
            clearInterval(this.timeId);
            this.events.trigger('title_hide');
        }

        /**
         * @method toggle 显示/隐藏字幕
         * @returns {void} void
         */

    }, {
        key: 'toggle',
        value: function toggle() {
            if (this.container.classList.contains('hide_dom')) {
                this.show();
            } else {
                this.hide();
            }
        }
    }, {
        key: 'destroy',
        value: function destroy() {
            clearInterval(this.timeId);
        }
    }]);
    return Subtitle;
}();

var JsBridge = {
    callNative: function callNative(method, params) {
        if (typeof Bridge !== "undefined") {
            return Bridge.callNative("com.nd.pad.icr.ui.IcrJsBridge", method, params || {});
        }
    },
    registerListener: function registerListener(eventName, func, scope) {
        if (typeof Bridge !== "undefined") {
            return Bridge.registerListener(eventName, func, scope);
        }
    }

};

/**
 * @class Subtitle 字幕
 * @description 字幕 基本功能
 */

var NativeSubtitle = function () {
    function NativeSubtitle(container, player, options, events, tran) {
        classCallCheck(this, NativeSubtitle);

        this.container = container;
        this.wrap = player.pop.wrap;
        this.player = player;
        this.audio = player.audio;
        this.options = options;
        this.events = events;
        this.tran = tran;
        this.preLineNo = 0;
        this.lineNumber = 0;
        this.popShow = false;
        this.mediaId = utils$1.getRandomId("avi_media_palyer");
        this.init(options);
        this.parentContainer = this.container.parentElement;
    }

    /**
     * @method getLyric 渲染组件
     * @returns {lyric} 字幕
     */


    createClass(NativeSubtitle, [{
        key: 'getLyric',
        value: function getLyric() {
            return this.list;
        }

        /**
         * @method init 初始化
         * @param {options} 选项
         * @returns {void} void
         */

    }, {
        key: 'init',
        value: function init(options) {
            var _this = this;

            var that = this;
            if (!this.options.audio.lyric) {
                throw new Error("获取歌词数据失败");
                return;
            }

            var render = function render(text) {
                if (text == null || text == "" || text == undefined) {
                    _this.player.buttons.subtitle.classList.add("click_disabled");
                    return;
                }
                var obj = Parser.parse(text);
                _this.list = obj.texts;

                if (_this.list.length == 0) {
                    _this.player.buttons.subtitle.classList.add("click_disabled");
                    return;
                }
                _this.info = obj.info;
                obj.textinput = obj.info.textinput ? obj.info.textinput : "";
                obj.mediaId = _this.mediaId;
                _this.createPresenter(obj);
            };

            // API.get(this.options.audio.lyric).then(function(data){
            //     if (data.length === 0) {
            //         that.player.buttons.subtitle.classList.add("click_disabled");
            //         return;
            //     }
            //     render(data);
            // }, function(data){
            //     that.player.buttons.subtitle.classList.add("click_disabled");
            //     throw new Error("加载歌词出错");
            // })

            API.read(this.options.audio.lyric, function (data) {
                if (data.status == 200 || data.status == 0 && data.response !== "") {
                    if (data.response.length === 0) {
                        that.player.buttons.subtitle.classList.add("click_disabled");
                        return;
                    }
                    render(data.response);
                } else {
                    that.player.buttons.subtitle.classList.add("click_disabled");
                    throw new Error("加载歌词出错");
                }
            });
        }
    }, {
        key: 'syncLyric',
        value: function syncLyric() {
            this.init(this.options);
        }
    }, {
        key: 'createPresenter',
        value: function createPresenter(data) {
            var that = this;
            if (JsBridge != null) {
                JsBridge.callNative("createPresenter", {
                    presenterId: "avi_media_palyer",
                    instanceId: that.mediaId,
                    initData: data
                });
            }
        }
    }, {
        key: 'destroyPresenter',
        value: function destroyPresenter() {
            var that = this;
            if (JsBridge != null) {
                JsBridge.callNative("destroyPresenter", {
                    instanceId: this.mediaId
                });
            }
        }
    }, {
        key: 'sendToPresenter',
        value: function sendToPresenter(type, data) {
            var params = {
                instanceId: this.mediaId,
                type: type,
                data: data
            };
            if (JsBridge != null) {
                JsBridge.callNative("sendToPresenter", params);
            }
        }
    }, {
        key: 'registerShowLyricBoxCallback',
        value: function registerShowLyricBoxCallback(callback) {
            if (JsBridge != null) {
                JsBridge.registerListener("showLyricBoxCallback", callback);
            }
        }
    }, {
        key: 'showLyricBoxCallback',
        value: function showLyricBoxCallback(data) {
            console.log("showLyricBoxCallback");
            if (data.mediaId == this.mediaId) {
                if (data.isShow) {
                    this.popShow = true;
                    this.player.buttons.subtitle.classList.add("ui-btn-active");
                } else {
                    this.popShow = false;
                    this.player.buttons.subtitle.classList.remove("ui-btn-active");
                }
            }
        }
    }, {
        key: 'locateLyricCallback',
        value: function locateLyricCallback(time) {
            console.log("locateLyricCallback");
            console.log(time);
        }
    }, {
        key: 'registerLocateLyricCallback',
        value: function registerLocateLyricCallback(callback) {
            if (JsBridge != null) {
                JsBridge.registerListener("locateLyricCallback", callback);
            }
        }
    }, {
        key: 'toggle',
        value: function toggle() {
            console.log("toggle");
            if (this.popShow) {
                this.hide();
            } else {
                this.show();
            }
        }
    }, {
        key: 'play',
        value: function play(time) {
            console.log("pause");
            this.sendToPresenter("locateLyric", {
                totalTime: this.player.audio.duration * 1000,
                currentTime: time
            });
        }
    }, {
        key: 'pause',
        value: function pause() {
            console.log("pause");
            this.sendToPresenter("stopLocateLyric", {
                totalTime: this.player.audio.duration * 1000,
                currentTime: this.player.audio.currentTime * 1000,
                lineNumber: this.lineNumber
            });
        }
    }, {
        key: 'show',
        value: function show() {
            console.log("show");
            this.popShow = true;
            this.player.buttons.subtitle.classList.add("ui-btn-active");
            this.sendToPresenter("showLyricBox", {
                totalTime: this.player.audio.duration * 1000,
                currentTime: this.player.audio.currentTime * 1000
            });
        }
    }, {
        key: 'hide',
        value: function hide() {
            console.log("hide");
            this.popShow = false;
            this.player.buttons.subtitle.classList.remove("ui-btn-active");
            this.sendToPresenter("dismissLyricBox", "");
        }
    }, {
        key: 'end',
        value: function end() {}

        /**
         * @method find 查找
         * @returns {void} void
         */

    }, {
        key: 'find',
        value: function find(time) {
            var lineNo = 0,
                preLineNo = 0,
                flag = false;
            for (var i = 0, l = this.list.length; i < l; i++) {
                if (this.list[i + 1] != null) {
                    var startTime = utils$1.timeToSecond(this.list[i].startTime);
                    var endTime = utils$1.timeToSecond(this.list[i + 1].startTime);
                    if (time > startTime && time < endTime) {
                        lineNo = i;
                        if (lineNo != this.preLineNo) {
                            this.preLineNo = lineNo;
                            flag = true;
                        } else {
                            flag = false;
                        }
                    }
                } else {
                    if (time > this.list[i].startTime) {
                        lineNo = i;
                        if (lineNo != this.preLineNo) {
                            this.preLineNo = lineNo;
                            flag = true;
                        } else {
                            flag = false;
                        }
                    }
                }
            }
            return {
                lineNo: lineNo,
                flag: flag
            };
        }
    }, {
        key: 'sync',
        value: function sync(time) {
            console.log("sync");
            if (this.list != null && utils$1.isNotUserInput(this.info.textinput)) {
                var obj = this.find(time);
                if (obj.flag) {
                    this.lineNumber = obj.lineNo;
                    this.sendToPresenter("locateLyric", {
                        totalTime: this.player.audio.duration * 1000,
                        currentTime: time * 1000,
                        lineNumber: obj.lineNo
                    });
                }
            }
        }
    }, {
        key: 'seek',
        value: function seek(time) {}
    }, {
        key: 'destroy',
        value: function destroy() {
            this.destroyPresenter();
        }
    }]);
    return NativeSubtitle;
}();

var EXTEND_STYLE$1 = {
    'blue': {
        'className': 'style--pop-blue'
    }
};

/**
 * @class APlayer 音频组件
 * @description 音频 基本功能
 */

var APlayer = function () {
    function APlayer(options) {
        classCallCheck(this, APlayer);

        if (!options) {
            throw new Error("至少需要一个参数");
            return;
        }
        this.options = handleOption$1(options);
        this.tran = new i18n$1(this.options.lang).tran;
        this.events = new Events$1();
        this.container = this.options.container;
        this.subtitle = null;
        this.loaded = false;
        this.durationCache = 0; //缓存的总时长

        var src = this.options.audio.url ? this.options.audio.url : this.container.getAttribute('src') ? this.container.getAttribute('src') : this.container.querySelector('source').getAttribute('src');
        var lyric = this.options.audio.lyric ? this.options.audio.lyric : this.container.getAttribute('lyric');
        var title = this.options.audio.title ? this.options.audio.title : this.container.getAttribute('title') ? this.container.getAttribute('title') : utils$1.getFileName(src);
        var duration = this.options.audio.duration ? this.options.audio.duration : this.container.getAttribute('duration') ? this.container.getAttribute('duration') : 0;

        this.options.audio.url = src;
        this.options.audio.lyric = lyric;
        this.options.audio.title = title;
        this.options.audio.duration = duration == "undefined" ? 0 : duration;
    }

    createClass(APlayer, [{
        key: 'getDuration',
        value: function getDuration() {
            if (!isNaN(this.player.audio.duration) && this.player.audio.duration) {
                return this.player.audio.duration;
            } else if (this.options.audio.duration) {
                return this.options.audio.duration;
            } else {
                return this.durationCache;
            }
        }

        /**
         * @method change 改变options
         * @returns {void} 
         */

    }, {
        key: 'change',
        value: function change(options) {
            if (!options) {
                throw new Error("至少需要一个参数");
                return;
            }

            if (this.subtitle != null) {
                this.subtitle.hide();
            }

            this.pause();
            this.seek(0);

            this.options = handleOption$1(options);
            this.tran = new i18n$1(this.options.lang).tran;
            // this.events = new events();

            this.options.audio.url = options.audio.url;
            this.options.audio.lyric = options.audio.lyric;
            this.options.container = this.container;

            var title = options.audio.title ? options.audio.title : utils$1.getFileName(options.audio.url);
            this.options.audio.title = title ? title : this.options.audio.title ? this.options.audio.title : utils$1.getFileName(src);
            this.options.audio.duration = options.audio.duration ? options.audio.duration : 0;

            this.ended = false;
            this.loaded = false;
            this.player.texts.title.innerText = this.options.audio.title;
            this.updatePlayTime(0, this.options.audio.duration);

            this.player.audio.src = options.audio.url;
            this.player.audio.load();
        }

        /**
         * @method render 渲染组件
         * @returns {void} 
         */

    }, {
        key: 'render',
        value: function render() {
            var _this = this;

            var resetAudio = function resetAudio() {
                while (_this.container.hasChildNodes()) {
                    _this.container.removeChild(_this.container.firstChild);
                }
                _this.container.removeAttribute("controls");
                // this.container.setAttribute("crossorigin", "anonymous");
                _this.container.setAttribute("cid", "player-audio");
                //防止触发重新加载事件
                if (_this.container.getAttribute("src") != _this.options.audio.url) {
                    _this.container.setAttribute("src", _this.options.audio.url);
                }
                _this.container.setAttribute("title", _this.options.audio.title);
                _this.container.setAttribute("lyric", _this.options.audio.lyric);
                _this.container.setAttribute("class", "player-audio player-audio-current");
                _this.container.setAttribute("preload", _this.options.preload);
                _this.container.setAttribute("duration", _this.options.audio.duration);
            };
            resetAudio();

            //渲染html
            var videoHtml = html$2.main(this.options, this.tran);
            var styleWidth = this.options.width == null ? '' : 'style="width:' + this.options.width + '"';
            var topContains = utils$1.createDom('<span class="component__audio" cid="component-audio" ' + styleWidth + ' />');
            topContains.innerHTML = videoHtml;
            utils$1.insertAfter(topContains, this.container);
            var videoContains = topContains.querySelector("span[cid='audio-mediaelement']");
            videoContains.appendChild(this.container);
            this.container = topContains;
            this.componentAudio = this.container;
            this.parentContainer = this.componentAudio.parentNode;

            var doms = html$2.getDoms(this.container);
            this.player = doms;

            // 扩展皮肤，增加相应类名
            if (this.options.extendStyle && EXTEND_STYLE$1[this.options.extendStyle]) {
                this.componentAudio.classList.add(EXTEND_STYLE$1[this.options.extendStyle].className);
            }

            var that = this;

            var mouseEvents = utils$1.isMobile ? {
                down: 'touchstart mousedown',
                move: 'touchmove mousemove',
                up: 'touchend mouseup',
                over: 'touchstart',
                out: 'touchend',
                tap: 'touchstart mousedown'
            } : {
                down: 'mousedown touchstart',
                move: 'mousemove touchmove',
                up: 'mouseup touchend',
                over: 'mouseover',
                out: 'mouseout',
                tap: 'mousedown touchstart'
            };

            var barWidth = void 0;
            this.updateBar = function (type, percentage, direction) {
                percentage = percentage > 0 ? percentage : 0;
                percentage = percentage < 1 ? percentage : 1;
                _this.player.bars[type].style[direction] = percentage * 100 + '%';
            };

            var lastPlayPos = 0;
            var currentPlayPos = 0;
            var bufferingDetected = false;
            window.requestAnimationFrame = function () {
                return window.requestAnimationFrame || window.webkitRequestAnimationFrame || window.mozRequestAnimationFrame || window.oRequestAnimationFrame || window.msRequestAnimationFrame || function (callback) {
                    window.setTimeout(callback, 1000 / 60);
                };
            }();

            var setCheckLoadingTime = function setCheckLoadingTime() {
                _this.checkLoading = setInterval(function () {
                    if (_this.player.audio != null) {
                        currentPlayPos = _this.player.audio.currentTime;
                        if (!bufferingDetected && currentPlayPos === lastPlayPos && !_this.player.audio.paused) {
                            _this.container.classList.add('player-loading');
                            bufferingDetected = true;
                        }
                        if (bufferingDetected && currentPlayPos > lastPlayPos && !_this.player.audio.paused) {
                            _this.container.classList.remove('player-loading');
                            bufferingDetected = false;
                        }
                        lastPlayPos = currentPlayPos;
                    }
                }, 100);
            };

            var clearCheckLoadingTime = function clearCheckLoadingTime() {
                clearInterval(_this.checkLoading);
            };

            this.playedTime = false;
            this.animationFrame = function () {
                if (_this.playedTime) {
                    _this.updateBar('played', _this.player.audio.currentTime / _this.getDuration(), 'width');
                    _this.updatePlayTime(_this.player.audio.currentTime, _this.getDuration());
                }
                if (_this.animationFrame != null) {
                    window.requestAnimationFrame(_this.animationFrame);
                }
            };
            window.requestAnimationFrame(this.animationFrame);

            this.setTime = function (type) {
                if (!type) {
                    _this.playedTime = true;
                    setCheckLoadingTime();
                } else {
                    _this[type + 'Time'] = true;
                    if (type === 'played') {
                        setCheckLoadingTime();
                    }
                }
            };
            this.clearTime = function (type) {
                if (!type) {
                    _this.playedTime = false;
                    clearCheckLoadingTime();
                } else {
                    _this[type + 'Time'] = false;
                    if (type === 'played') {
                        clearCheckLoadingTime();
                    }
                }
            };

            var stopPropagation = function stopPropagation(e) {
                if (document.all) {
                    //只有ie识别
                    e.cancelBubble = true;
                } else {
                    e.stopPropagation();
                }
            };

            var cancelBubble = function cancelBubble(e) {
                //取消事件的默认动作
                e.preventDefault();
                stopPropagation(e);
            };

            var playerToggele = function playerToggele(e) {
                var e = e || window.event;
                cancelBubble(e);
                that.toggle();
            };

            utils$1.addEventHandler(document, mouseEvents.tap, utils$1.throttle(function () {
                if (_this.player != null && _this.player.volume != null) {
                    _this.player.volume.controler.classList.remove("ui-btn-active");
                }
            }, 500));

            utils$1.addEventHandler(this.player.buttons.play, mouseEvents.tap, utils$1.throttle(playerToggele, 500));
            utils$1.addEventHandler(this.player.buttons.layerPlay, mouseEvents.tap, utils$1.throttle(playerToggele, 500));

            if (!utils$1.isMobile) {
                utils$1.addEventHandler(this.player.buttons.play, "mouseover", function () {
                    _this.player.buttons.play.classList.add("ui-btn-hover");
                });
                utils$1.addEventHandler(this.player.buttons.play, "mouseout", function () {
                    _this.player.buttons.play.classList.remove("ui-btn-hover");
                });

                utils$1.addEventHandler(this.player.volume.controler, "mouseover", function () {
                    _this.player.buttons.volume.classList.add("ui-btn-hover");
                    _this.player.volume.controler.classList.add("ui-btn-active");
                    //音量向下显示
                    if (_this.container.offsetTop < 20) {
                        // this.player.                    
                    }
                });

                utils$1.addEventHandler(this.player.volume.controler, "mouseout", function () {
                    _this.player.buttons.volume.classList.remove("ui-btn-hover");
                    _this.player.volume.controler.classList.remove("ui-btn-active");
                });

                utils$1.addEventHandler(this.player.buttons.subtitle, "mouseover", function () {
                    _this.player.buttons.subtitle.classList.add("ui-btn-hover");
                });
                utils$1.addEventHandler(this.player.buttons.subtitle, "mouseout", function () {
                    _this.player.buttons.subtitle.classList.remove("ui-btn-hover");
                });
            }

            var getPercentage = function getPercentage(e) {
                var e = e || window.event;
                var clientX = e.changedTouches ? e.changedTouches[0].clientX : e.clientX;
                var percentage = (clientX - utils$1.getOffset(_this.player.bars.rail).left) / _this.getScale() / barWidth;
                percentage = percentage > 0 ? percentage : 0;
                percentage = percentage < 1 ? percentage : 1;
                return percentage;
            };

            var thumbMove = function thumbMove(e) {
                var e = e || window.event;
                cancelBubble(e);
                if (1 == e.which || 0 == e.which) {
                    var percentage = getPercentage(e);
                    _this.updateBar('played', percentage, 'width');
                    var duration = _this.getDuration();
                    _this.updatePlayTime(percentage * duration, duration);
                }
            };

            var thumbUp = function thumbUp(e) {
                var e = e || window.event;
                cancelBubble(e);
                if (1 == e.which || 0 == e.which) {
                    var percentage = getPercentage(e);
                    _this.updateBar('played', percentage, 'width');
                    var duration = _this.getDuration();
                    _this.seek(parseFloat(_this.player.bars.played.style.width) / 100 * duration);
                    _this.setTime();
                    utils$1.removeEventHandler(document, mouseEvents.up, thumbUp);
                    utils$1.removeEventHandler(document, mouseEvents.move, thumbMove);
                    document.onmousemove = null;
                    document.onmouseup = null;
                }
            };

            utils$1.addEventHandler(this.player.bars.rail, mouseEvents.down, function (e) {
                var e = e || window.event;
                cancelBubble(e);
                if (1 == e.which || 0 == e.which) {
                    barWidth = _this.player.bars.rail.clientWidth;
                    _this.clearTime();
                    utils$1.addEventHandler(document, mouseEvents.move, thumbMove);
                    utils$1.addEventHandler(document, mouseEvents.up, thumbUp);
                }
            });

            this.player.pop.container.style.fontSize = document.body.clientWidth / 80 + 'px';

            var distanceX = 0,
                distanceY = 0;

            if (this.options.audio.lyric) {
                var toggleSubtitle = function toggleSubtitle(e) {
                    var e = e || window.event;
                    cancelBubble(e);
                    if (!_this.player.buttons.subtitle.classList.contains("click_disabled")) {
                        _this.subtitle.toggle();
                        _this.subtitle.seek(_this.player.audio.currentTime);
                    }
                };

                utils$1.addEventHandler(this.player.buttons.subtitleContainer, mouseEvents.tap, utils$1.throttle(toggleSubtitle, 500));

                if (!utils$1.isMobile) {
                    utils$1.addEventHandler(this.player.buttons.closeSubtitle, mouseEvents.tap, utils$1.throttle(toggleSubtitle, 500));

                    var subtitleUp = function subtitleUp(e) {
                        var e = e || window.event;
                        cancelBubble(e);
                        if (1 == e.which || 0 == e.which) {
                            utils$1.removeEventHandler(document, mouseEvents.up, subtitleUp);
                            utils$1.removeEventHandler(document, mouseEvents.move, _subtitleMove);
                            document.onmousemove = null;
                            document.onmouseup = null;
                        }
                    };
                    var _subtitleMove = function _subtitleMove(e) {
                        var e = e || window.event;
                        cancelBubble(e);
                        if (1 == e.which || 0 == e.which) {
                            var clientX = e.changedTouches ? e.changedTouches[0].clientX : e.clientX;
                            var clientY = e.changedTouches ? e.changedTouches[0].clientY : e.clientY;
                            var left = clientX - distanceX < 0 ? 0 : clientX - distanceX;
                            var top = clientY - distanceY < 30 ? 30 : clientY - distanceY;

                            if (left + _this.player.pop.wrap.clientWidth > document.body.clientWidth) {
                                left = document.body.clientWidth - _this.player.pop.wrap.clientWidth;
                            }

                            if (top + _this.player.pop.wrap.clientHeight > document.body.clientHeight) {
                                top = document.body.clientHeight - _this.player.pop.wrap.clientHeight;
                            }
                            _this.player.pop.wrap.style.left = left + 'px';
                            _this.player.pop.wrap.style.top = top + 'px';
                        }
                    };

                    utils$1.addEventHandler(this.player.pop.wrap, mouseEvents.down, function (e) {
                        var e = e || window.event;
                        cancelBubble(e);
                        if (1 == e.which || 0 == e.which) {
                            var clientX = e.changedTouches ? e.changedTouches[0].clientX : e.clientX;
                            var clientY = e.changedTouches ? e.changedTouches[0].clientY : e.clientY;

                            distanceX = clientX - _this.player.pop.wrap.offsetLeft;
                            distanceY = clientY - _this.player.pop.wrap.offsetTop;

                            utils$1.addEventHandler(document, mouseEvents.move, _subtitleMove);
                            utils$1.addEventHandler(document, mouseEvents.up, subtitleUp);
                        }
                    });
                }
            }

            /**
             * control volume
             */
            this.switchVolumeIcon = function () {
                if (_this.volume() > 0) {
                    _this.player.buttons.volume.classList.remove('audio-icon-volume-no');
                    _this.player.buttons.volume.classList.add('audio-icon-volume');
                } else {
                    _this.player.buttons.volume.classList.remove('audio-icon-volume');
                    _this.player.buttons.volume.classList.add('audio-icon-volume-no');
                }
            };
            var volumeMove = function volumeMove(e) {
                var e = e || window.event;
                cancelBubble(e);
                if (1 == e.which || 0 == e.which) {
                    var clientY = e.changedTouches ? e.changedTouches[0].clientY : e.clientY;
                    // var offsetTop = utils.getOffset(this.player.volume.sildeBar).top;
                    var offsetTop = _this.player.volume.sildeBar.getBoundingClientRect().top;
                    var percentage = 1 - (clientY - offsetTop) / _this.getScale() / _this.player.bars.volumeSilde.clientHeight;
                    _this.volume(percentage, 'height');
                }
            };
            var volumeUp = function volumeUp(e) {
                var e = e || window.event;
                cancelBubble(e);
                if (1 == e.which || 0 == e.which) {
                    utils$1.removeEventHandler(document, mouseEvents.up, volumeUp);
                    utils$1.removeEventHandler(document, mouseEvents.move, volumeMove);
                    document.onmousemove = null;
                    document.onmouseup = null;
                }
            };

            utils$1.addEventHandler(this.player.volume.sildeContent, mouseEvents.tap, utils$1.throttle(function (e) {
                var e = e || window.event;
                cancelBubble(e);
                if (1 == e.which || 0 == e.which) {
                    var clientY = e.changedTouches ? e.changedTouches[0].clientY : e.clientY;
                    var offsetTop = utils$1.getOffset(_this.player.volume.sildeBar).top;
                    var offsetTop = _this.player.volume.sildeBar.getBoundingClientRect().top;
                    var percentage = 1 - (clientY - offsetTop) / _this.getScale() / _this.player.bars.volumeSilde.clientHeight;
                    _this.volume(percentage, 'height');
                }
            }, 500));

            utils$1.addEventHandler(this.player.volume.sildeContent, mouseEvents.down, function (e) {
                var e = e || window.event;
                cancelBubble(e);
                if (1 == e.which || 0 == e.which) {
                    utils$1.addEventHandler(document, mouseEvents.move, volumeMove);
                    utils$1.addEventHandler(document, mouseEvents.up, volumeUp);
                }
            });

            utils$1.addEventHandler(this.player.buttons.volume, mouseEvents.tap, utils$1.throttle(function (e) {
                var e = e || window.event;
                cancelBubble(e);
                if (!_this.player.volume.sildeBarHidden) {
                    _this.player.volume.controler.classList.add("ui-btn-active");
                }
                if (_this.player.audio.muted) {
                    _this.player.audio.muted = false;
                    _this.switchVolumeIcon();
                    _this.updateBar('volume', _this.volume(), 'height');
                } else {
                    _this.player.audio.muted = true;
                    _this.player.buttons.volume.classList.remove('audio-icon-volume');
                    _this.player.buttons.volume.classList.add('audio-icon-volume-no');
                    _this.updateBar('volume', 0, 'height');
                }
            }, 500));

            utils$1.addEventHandler(this.player.buttons.screen, mouseEvents.tap, utils$1.throttle(function (e) {
                var e = e || window.event;
                cancelBubble(e);
                _this.toggleScreen();
            }, 500));

            // set duration time

            if (this.player.audio.duration !== 1) {
                if (isNaN(this.player.audio.duration)) {
                    this.updatePlayTime(0, this.options.audio.duration);
                } else {
                    this.updatePlayTime(this.player.audio.duration, this.player.audio.duration);
                }
            }

            // autoplay
            if (this.options.autoplay) {
                this.play();
            } else {

                this.pause();
            }

            this.volume(this.options.volume, 'height');

            this.initAudio(this.player.audio, this.options.audio.type);
        }
    }, {
        key: 'getScale',
        value: function getScale() {
            var scale = this.container.getBoundingClientRect().width / this.container.offsetWidth || 1;
            return scale;
        }

        /**
         * @method getStatus get audio status
         * @returns {bool} 是否暂停
         */

    }, {
        key: 'getStatus',
        value: function getStatus() {
            return this.player.audio.paused;
        }

        /**
         * @method setLock 锁定
         * @param {lock} lock 锁定：true, 不锁定 ：false
         * @returns {void} void
         */

    }, {
        key: 'setLock',
        value: function setLock(lock) {
            if (lock) {
                this.componentAudio.classList.add('pointer-events-none');
            } else {
                this.componentAudio.classList.remove('pointer-events-none');
            }

            if (typeof this.options.onSetLock !== 'undefined') {
                this.options.onSetLock(lock);
            }
        }

        /**
         * @method end seek audiomethod end seek audio
         * @param {time} time  eg: 80(s)
         * @returns {void} void
         */

    }, {
        key: 'end',
        value: function end(time) {
            if (this.player.audio != null) {
                this.seek(this.getDuration());
            }
        }

        /**
         * @method Seek seek video
         * @param {time} time  eg: 80(s)
         * @returns {void} void
         */

    }, {
        key: 'seek',
        value: function seek(time) {
            if (this.player.audio != null) {
                if (!this.loaded) {
                    this.player.audio.pause();
                    this.player.audio.load();
                    this.loaded = true;
                }

                time = Math.max(time, 0);
                var duration = this.getDuration();
                if (duration) {
                    time = Math.min(time, duration);
                }

                this.player.audio.currentTime = time;

                this.updateBar('played', time / duration, 'width');
                this.updatePlayTime(time, duration);

                if (this.subtitle != null) {
                    this.subtitle.seek(time);
                }

                if (typeof this.options.onSeek !== 'undefined') {
                    this.options.onSeek(time);
                }
            }
        }

        /**
         * @method play 播放音频
         * @returns {void} void
         */

    }, {
        key: 'play',
        value: function play() {
            if (this.player.audio != null) {
                if (!this.loaded) {
                    // this.player.audio.src = this.options.audio.src;
                    this.player.audio.pause();
                    this.player.audio.load();
                    this.loaded = true;
                }
                this.player.audio.play();
                this.setTime();
                if (!this.player.audio.paused) {
                    this.player.buttons.play.classList.remove('audio-icon-play');
                    this.player.buttons.play.classList.add('audio-icon-pause');
                    this.player.buttons.layerPlayBtn.classList.add('hide_dom');
                    this.container.classList.add('player-playing');
                }
                if (this.subtitle != null) {
                    this.subtitle.play();
                }
            }

            if (typeof this.options.onPlay !== 'undefined') {
                this.options.onPlay();
            }
        }

        /**
         * @method pause 暂停音频
         * @returns {void} void
         */

    }, {
        key: 'pause',
        value: function pause() {
            if (this.player.audio != null) {
                var state = this.player.audio.paused;

                this.ended = false;
                this.player.buttons.play.classList.remove('audio-icon-pause');
                this.player.buttons.play.classList.add('audio-icon-play');
                this.player.buttons.layerPlayBtn.classList.remove('hide_dom');
                if (!state) {
                    this.player.audio.pause();
                }
                this.clearTime();
                this.container.classList.remove('player-playing');

                if (this.subtitle != null) {
                    this.subtitle.pause();
                }
                if (typeof this.options.onPause !== 'undefined' && !state) {
                    this.options.onPause();
                }
            }
        }

        /**
         * @method hideLyric 隐藏歌词
         * @returns {void} void
         */

    }, {
        key: 'hideLyric',
        value: function hideLyric() {
            if (this.subtitle != null) {
                this.subtitle.hide();
            }
        }

        /**
         * @method syncLyric 同步歌词
         * @returns {void} void
         */

    }, {
        key: 'syncLyric',
        value: function syncLyric() {
            if (this.subtitle != null) {
                this.subtitle.syncLyric();
            }
        }

        /**
         * @method volume 设置音量
         * @param {percentage} 音量 eg: (0.1-1)
         * @returns {void} void
         */

    }, {
        key: 'volume',
        value: function volume(percentage) {
            var direction = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'width';

            percentage = parseFloat(percentage);
            if (!isNaN(percentage)) {
                percentage = percentage > 0 ? percentage : 0;
                percentage = percentage < 1 ? percentage : 1;
                this.updateBar('volume', percentage, direction);
                var formatPercentage = (percentage * 100).toFixed(0) + '%';
                this.player.bars.volume.style.height = formatPercentage;

                this.player.audio.volume = percentage;
                if (this.player.audio.muted) {
                    this.player.audio.muted = false;
                }
                this.switchVolumeIcon();
                if (typeof this.options.onVolumeChange !== 'undefined') {
                    this.options.onVolumeChange(percentage);
                }
            }

            return this.player.audio.volume;
        }

        /**
         * @method changeVolume 设置音量
         * @param {percentage} 音量 eg: (0.1-1)
         * @returns {void} void
         */

    }, {
        key: 'changeVolume',
        value: function changeVolume(percentage) {
            this.volume(percentage, 'height');
        }

        /**
         * @method toggle 播放/暂停音频
         * @returns {void} void
         */

    }, {
        key: 'toggle',
        value: function toggle() {
            if (this.player.audio.paused) {
                this.play();
            } else {
                this.pause();
            }
        }
    }, {
        key: 'toggleScreen',
        value: function toggleScreen() {
            if (this.isFullScreen()) {
                this.exitFullScreen();
            } else {
                this.enterFullScreen();
            }
        }

        /**
         * 进入全屏
         */

    }, {
        key: 'enterFullScreen',
        value: function enterFullScreen() {
            // fullContainer 分为 body 和指定其他容器
            if (this.options.fullContainer.tagName.toUpperCase() === 'BODY') {
                this.componentAudio.classList.add('component__audio--fullscreen');
            } else {
                this.componentAudio.classList.add('component__audio--full-specified-container');
            }
            this.player.buttons.screen.classList.add('audio-icon-exit-screen');
            this.player.buttons.screen.classList.remove('audio-icon-enter-screen');
            this.nullContainer = utils$1.createDom('<div cid="component-audio-null" />');
            utils$1.insertAfter(this.nullContainer, this.componentAudio);
            this.options.fullContainer.appendChild(this.componentAudio);
        }

        /**
         * 退出全屏
         */

    }, {
        key: 'exitFullScreen',
        value: function exitFullScreen() {
            if (this.options.fullContainer.tagName.toUpperCase() === 'BODY') {
                this.componentAudio.classList.remove('component__audio--fullscreen');
            } else {
                this.componentAudio.classList.remove('component__audio--full-specified-container');
            }
            this.player.buttons.screen.classList.add('audio-icon-enter-screen');
            this.player.buttons.screen.classList.remove('audio-icon-exit-screen');
            utils$1.insertAfter(this.componentAudio, this.nullContainer);
            this.parentContainer.removeChild(this.nullContainer);
        }
    }, {
        key: 'isFullScreen',
        value: function isFullScreen() {
            return this.componentAudio.classList.contains('component__audio--fullscreen') || this.componentAudio.classList.contains('component__audio--full-specified-container');
        }
    }, {
        key: 'on',
        value: function on(name, callback) {
            this.events.on(name, callback);
        }

        /**
         * @method updatePlayTime 更新播放时间
         * @param {star} current  eg: 02:29 
         * @param {total} current  eg: 02:29 
         * @returns {void} void
         */

    }, {
        key: 'updatePlayTime',
        value: function updatePlayTime(star, total) {
            this.player.texts.time.innerHTML = star ? utils$1.secondToTime(star) : utils$1.secondToTime(0);
            this.player.texts.total.innerHTML = total ? utils$1.secondToTime(total) : utils$1.secondToTime(this.options.audio.duration);
        }

        /**
         * @method Seek seek video
         * @param {audio} audio 音频audio
         * @param {type} type  
         * @returns {void} void
         */

    }, {
        key: 'initAudio',
        value: function initAudio(audio, type) {
            var _this2 = this;

            var that = this;
            // this.setLock(true)
            /**
             * audio events
             */
            this.on('durationchange', function () {
                if (audio.duration !== 1) {
                    // compatibility: Android browsers will output 1 at first
                    // this.setLock(false);
                    if (!isNaN(audio.duration)) {
                        that.durationCache = audio.duration;
                    }
                    var currentTime = audio.currentTime || 0;
                    _this2.updateBar('played', currentTime / audio.duration, 'width');
                    _this2.updatePlayTime(currentTime, audio.duration);
                }
            });

            // audio end
            this.ended = false;
            this.on('ended', function () {
                _this2.updateBar('played', 1, 'width');
                if (!_this2.loop) {
                    _this2.ended = true;
                    _this2.seek(0);
                    _this2.pause();
                } else {
                    _this2.seek(0);
                    _this2.play();
                }

                if (_this2.subtitle != null) {
                    _this2.subtitle.end();
                }

                if (typeof _this2.options.onEnd !== 'undefined') {
                    _this2.options.onEnd();
                }
            });

            this.on('play', function () {
                if (_this2.player.audio.paused) {
                    _this2.play();
                }
            });

            this.on('pause', function () {
                if (!_this2.player.audio.paused) {
                    _this2.pause();
                }
            });

            this.on('error', function () {
                _this2.setLock(true);
                throw new Error("加载失败");
            });

            this.on('timeupdate', function () {
                if (_this2.subtitle != null) {
                    _this2.subtitle.sync(_this2.player.audio.currentTime);
                }
                if (typeof _this2.options.onTimeUpdate !== 'undefined') {
                    _this2.options.onTimeUpdate(_this2.player.audio.currentTime);
                }
            });

            this.on('canplaythrough', function () {
                console.log('The file is loaded and ready to play!');
            });

            this.on('progress', function () {
                console.log('download progress');
            });

            var _loop = function _loop(i) {
                audio.addEventListener(_this2.events.audioEvents[i], function () {
                    if (_this2.events && _this2.events.trigger) {
                        _this2.events.trigger(_this2.events.audioEvents[i]);
                    }
                });
            };

            for (var i = 0; i < this.events.audioEvents.length; i++) {
                _loop(i);
            }

            if (this.options.audio.lyric) {
                if (utils$1.isMobile || utils$1.isMobile1) {
                    this.subtitle = new NativeSubtitle(this.player.pop.container, this.player, this.options, this.events, this.tran);
                    this.subtitle.registerShowLyricBoxCallback(function (data) {
                        that.subtitle.showLyricBoxCallback(data);
                    });
                    this.subtitle.registerLocateLyricCallback(function (time) {
                        that.subtitle.locateLyricCallback(time);
                    });
                } else {
                    this.subtitle = new Subtitle(this.player.pop.container, this.player, this.options, this.events, this.tran);
                }
            } else {
                this.player.buttons.subtitle.classList.add("click_disabled");
            }
        }

        /**
         * @method getView 获取组件componentVideo
         * @returns {void} void
         */

    }, {
        key: 'getView',
        value: function getView() {
            return this.componentAudio;
        }

        /**
         * @method reset 重置
         * @returns {void} void
         */

    }, {
        key: 'reset',
        value: function reset() {
            if (this.player.audio != null) {
                this.pause();
            }
            if (this.subtitle != null) {
                this.subtitle.hide();
            }
        }

        /**
         * @method offset 位置
         * @returns {void} void
         */

    }, {
        key: 'offset',
        value: function offset(left, top) {
            if (this.subtitle != null) {
                this.subtitle.offset(left, top);
            }
        }

        /**
         * @method reset 重置
         * @returns {void} void
         */

    }, {
        key: 'resize',
        value: function resize() {
            this.events.trigger('resize');
        }

        /**
         * @method destroy 销毁
         * @returns {void} void
         */

    }, {
        key: 'destroy',
        value: function destroy() {
            this.pause();
            clearTimeout(this.hideTime);
            if (this.subtitle != null) {
                this.subtitle.destroy();
            }
            this.player.audio.src = '';
            this.container.innerHTML = '';
            this.events.trigger('destroy');

            for (var key in this) {
                if (this.hasOwnProperty(key) && key !== 'paused') {
                    delete this[key];
                }
            }
        }
    }]);
    return APlayer;
}();

var _$2 = {
    TOP_VIEWER: null,
    INSTANCES: {},
    clear: function clear(key) {
        var instance = this.INSTANCES[key];
        if (instance) {
            instance.destroy();
        }
        this.INSTANCES[key] = null;
        delete this.INSTANCES[key];
    },
    getKeyId: function getKeyId(key) {
        return key.split('|')[1];
    }
};

var AudioPlayer = {
    setLoggerLevel: function setLoggerLevel(level) {},
    render: function render(container, options) {
        container = container.length > 0 ? container[0] : container;
        for (var key in _$2.INSTANCES) {
            if (!document.querySelector('[data-id="' + _$2.getKeyId(key) + '"]')) {
                _$2.clear(key);
            }
        }
        options = options || {};
        if (container) {
            var items = container.nodeName.toLowerCase() == 'audio' ? [container] : container.getElementsByTagName('audio');
            $.each(items, function (item, index) {
                var dataId = $.getRandomId('media');
                var mediaId = 'audio';
                var containerId = options.containerId || '';
                mediaId = mediaId + '-' + containerId + '-' + index;

                var key = mediaId + '|' + dataId;
                var newOptions = $.extend({}, options, {
                    dataId: dataId,
                    mediaId: mediaId,
                    key: key
                });

                var instance = new VPlayer(newOptions);
                instance.render();
                _$2.INSTANCES[key] = instance;
            });
        } else {
            Logger.debug('no container');
        }
        return _$2.INSTANCES;
    },
    destroy: function destroy() {},
    reset: function reset() {},
    setLock: function setLock(lock, view) {},
    getMedia: function getMedia() {},
    mediaOnFullScreenChange: function mediaOnFullScreenChange() {
        //no use
    },
    Audio: {
        create: function create(options) {
            return new APlayer(options);
        }
    }
};

var methods = ['assets', 'clear', 'count', 'debug', 'dir', 'dirxml', 'error', 'exception', 'group', 'groupCollapsed', 'groupEnd', 'info', 'log', 'markTimeline', 'profile', 'profileEnd', 'table', 'time', 'timeEnd', 'timeline', 'timelineEnd', 'timeStamp', 'trace', 'warn'];

var LEVEL = {
    DEBUG: 'debug',
    INFO: 'info',
    WARN: 'warn',
    ERROR: 'error',
    DEBUG_VALUE: 3,
    INFO_VALUE: 2,
    WARN_VALUE: 1,
    ERROR_VALUE: 0,
    UNKNOWN: -1
};

var console$1 = window.console || {};

var length = methods.length;

while (length--) {
    var method = methods[length];
    if (!console$1[method]) {
        console$1[method] = function () {};
    }
}

var Logger$1 = {

    $level: 0,
    setLevel: function setLevel(level) {
        switch (level) {
            case LEVEL.DEBUG:
                this.$level = LEVEL.DEBUG_VALUE;
                break;
            case LEVEL.INFO:
                this.$level = LEVEL.INFO_VALUE;
                break;
            case LEVEL.WARN:
                this.$level = LEVEL.WARN_VALUE;
                break;
            case LEVEL.ERROR:
                this.$level = LEVEL.ERROR_VALUEl;
                break;
            default:
                this.$level = LEVEL.UNKNOWN;
        }
    },
    debug: function debug() {
        if (this.$level >= LEVEL.DEBUG_VALUE) {
            console$1.log.apply(console$1, arguments);
        }
    },
    info: function info() {
        if (this.$level >= LEVEL.INFO_VALUE) {
            console$1.log.apply(console$1, arguments);
        }
    },
    warn: function warn() {
        if (this.$level >= LEVEL.WARN_VALUE) {
            console$1.log.apply(console$1, arguments);
        }
    },
    error: function error() {
        if (this.$level >= LEVEL.ERROR_VALUE) {
            console$1.log.apply(console$1, arguments);
        }
    }
};

var DEFAULT = {
    src: '', //地址，支持字符串
    modal: true, //弹窗是否可以模态
    lock: false, // 锁定窗口 
    env: '', //环境 当前只有pc 和mobile 两种 所以暂时没有使用
    // minWidth:50,//没有使用
    // minHeight:50,//没有使用
    // maxWidth: '',//没有使用
    // maxHeight: '',//没有使用
    lang: 'zh', //语言,
    inline: false, //支持ckeditor使用span来做包装标签
    defaultSrc: '', //加载失败默认图片,
    style: '', //支持容器样式修改，
    eventAll: true, //是否支持整 个容器监听事件
    origin: false, //显示原始图片
    originMaxHeight: '', //显示原始图片最大高度（需要带单位，如200px\12em）
    originMaxWidth: '',
    zoomIcon: undefined, //都没传时，使用默认的显示逻辑
    mobileViewer: {
        ImgStyle: '' //预览窗口图片样式
    }
};

var Template = {
    view: '<div class="ndui-insert-pictures" style="width: 256px; height: 176px">' + '<div class="insert-pictures__wrap">' + '<span class="insert-pictures__zoom"></span>' + '<span class="insert-pictures__loading"></span>' + '</div>' + '</div>',
    view2: '<span class="ndui-insert-pictures" style="width: 256px; height: 176px">' + '<span class="insert-pictures__wrap">' + '<span class="insert-pictures__zoom"></span>' + '<span class="insert-pictures__loading"></span>' + '</span>' + '</span>',
    picItem: '<li class="pop-pic-list_li" style="background-image: url(\'{imgSrc}\')" data-index="{index}"></li> ',
    popwindow: '<div class="ndui-pop-insert-pictures">\n            <div class="pop-insert-pictures__close-inclass hide_dom disable-full"></div>\n            <div class="pop-insert-pictures__head">\n                <span class="head__oper head__close hide_dom"></span>\n                <span class="head__oper head__full {hideClassName}"></span>\n                <span class="head__title">\u8FD9\u662F\u56FE\u7247\u540D\u79F0</span>\n            </div>\n            <div class="pop-insert-pictures__body">\n                <img src="" alt="" >\n                <span class="pic__oper oper--left disable-full" style="display:none"></span>\n                <span class="pic__oper oper--right disable-full" style="display:none"></span>\n            </div>\n            <div class="pop-pic-list disable-full hide_dom">\n                \n            </div>\n            <div class="pop-pic-extend clearfix hide_dom">\n                <div class="pop-pic_nums">\n                    <span class="pic-text current js_count_current">2</span>\n                    <span class="pic-text">/</span>\n                    <span class="pic-text js_count_total">3</span>\n                </div>\n                <div class="pop-pic__description">\n                    <span class="pic-text _description-text">\u6587\u4EF6\u5927\u5C0F\uFF1A178kb</span>\n                </div>\n                <div class="pop-pic-btn-full exit {hideClassName}"></div>\n                <div class="pop-pic-scale disable-full">\n                    <div class="pop-pic-scale__less"></div>\n                    <div class="pop-pic-scale__rail">\n                        <i class="pop-pic-scale__mark mark1"></i>                \n                        <i class="pop-pic-scale__mark mark2"></i>                \n                        <i class="pop-pic-scale__mark mark3"></i>                \n                        <div class="pop-pic-scale__bar" style="left: 20%;"></div>\n                    </div>\n                    <div class="pop-pic-scale__more"></div>\n                </div>\n            </div>\n            <span class="insert-pictures__loading"></span>\n        </div>',
    mask: '<div class="ndui-pop-insert-pictures-mask"></div>',
    mobilefull: '<div class="ndui-pop-insert-pictures-pad">\n            <div class="pop-insert-pictures__body">\n                <img src="" alt="">\n            </div>\n            <span class="insert-pictures__loading"></span>\n        </div>'
};

var REGEXP_SPACES = /\s+/;
var REGEXP_SUFFIX = /^(width|height|left|top|marginLeft|marginTop)$/;
var REGEXP_TRIM = /^\s+(.*)\s+$/;

// Utilities
var objectProto = Object.prototype;
var toString = objectProto.toString;
function typeOf(obj) {
  return toString.call(obj).slice(8, -1).toLowerCase();
}

function isString(str) {
  return typeof str === 'string';
}

function isNumber(num) {
  return typeof num === 'number' && !isNaN(num);
}



function isObject(obj) {
  return (typeof obj === 'undefined' ? 'undefined' : _typeof(obj)) === 'object' && obj !== null;
}



function isFunction(fn) {
  return typeOf(fn) === 'function';
}

function isArray(arr) {
  return Array.isArray ? Array.isArray(arr) : typeOf(arr) === 'array';
}





function trim(str) {
  if (isString(str)) {
    str = str.trim ? str.trim() : str.replace(REGEXP_TRIM, '1');
  }

  return str;
}

function each(obj, callback) {
  if (obj && isFunction(callback)) {
    var i = void 0;

    if (isArray(obj) || isNumber(obj.length) /* array-like */) {
        var length = obj.length;

        for (i = 0; i < length; i++) {
          if (callback.call(obj, obj[i], i, obj) === false) {
            break;
          }
        }
      } else if (isObject(obj)) {
      Object.keys(obj).forEach(function (key) {
        callback.call(obj, obj[key], key, obj);
      });
    }
  }

  return obj;
}

function extend(obj) {
  for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  if (isObject(obj) && args.length > 0) {
    if (Object.assign) {
      return Object.assign.apply(Object, [obj].concat(args));
    }

    args.forEach(function (arg) {
      if (isObject(arg)) {
        Object.keys(arg).forEach(function (key) {
          obj[key] = arg[key];
        });
      }
    });
  }

  return obj;
}

function proxy(fn, context) {
  for (var _len2 = arguments.length, args = Array(_len2 > 2 ? _len2 - 2 : 0), _key2 = 2; _key2 < _len2; _key2++) {
    args[_key2 - 2] = arguments[_key2];
  }

  return function () {
    for (var _len3 = arguments.length, args2 = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      args2[_key3] = arguments[_key3];
    }

    return fn.apply(context, args.concat(args2));
  };
}

function setStyle(element, styles) {
  var style = element.style;

  each(styles, function (value, property) {
    if (REGEXP_SUFFIX.test(property) && isNumber(value)) {
      value += 'px';
    }

    style[property] = value;
  });
}



function hasClass(element, value) {
  return element.classList ? element.classList.contains(value) : element.className.indexOf(value) > -1;
}

function addClass(element, value) {
  if (!value) {
    return;
  }

  if (isNumber(element.length)) {
    each(element, function (elem) {
      addClass(elem, value);
    });
    return;
  }

  if (element.classList) {
    element.classList.add(value);
    return;
  }

  var className = trim(element.className);

  if (!className) {
    element.className = value;
  } else if (className.indexOf(value) < 0) {
    element.className = className + ' ' + value;
  }
}

function removeClass(element, value) {
  if (!value) {
    return;
  }

  if (isNumber(element.length)) {
    each(element, function (elem) {
      removeClass(elem, value);
    });
    return;
  }

  if (element.classList) {
    element.classList.remove(value);
    return;
  }

  if (element.className.indexOf(value) >= 0) {
    element.className = element.className.replace(value, '');
  }
}











function removeListener(element, type, handler) {
  var types = trim(type).split(REGEXP_SPACES);

  if (types.length > 1) {
    each(types, function (t) {
      removeListener(element, t, handler);
    });
    return;
  }

  if (element.removeEventListener) {
    element.removeEventListener(type, handler, false);
  } else if (element.detachEvent) {
    element.detachEvent('on' + type, handler);
  }
}

function addListener(element, type, _handler, once) {
  var types = trim(type).split(REGEXP_SPACES);
  var originalHandler = _handler;

  if (types.length > 1) {
    each(types, function (t) {
      addListener(element, t, _handler);
    });
    return;
  }

  if (once) {
    _handler = function handler() {
      for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
        args[_key4] = arguments[_key4];
      }

      removeListener(element, type, _handler);

      return originalHandler.apply(element, args);
    };
  }

  if (element.addEventListener) {
    element.addEventListener(type, _handler, false);
  } else if (element.attachEvent) {
    element.attachEvent('on' + type, _handler);
  }
}



function getEvent(event) {
  var e = event || window.event;

  // Fix target property (IE8)
  if (!e.target) {
    e.target = e.srcElement || document;
  }

  if (!isNumber(e.pageX) && isNumber(e.clientX)) {
    var eventDoc = event.target.ownerDocument || document;
    var doc = eventDoc.documentElement;
    var body = eventDoc.body;

    e.pageX = e.clientX + ((doc && doc.scrollLeft || body && body.scrollLeft || 0) - (doc && doc.clientLeft || body && body.clientLeft || 0));
    e.pageY = e.clientY + ((doc && doc.scrollTop || body && body.scrollTop || 0) - (doc && doc.clientTop || body && body.clientTop || 0));
  }

  return e;
}

function getOffset(element) {
  var doc = document.documentElement;
  var box = element.getBoundingClientRect();

  return {
    left: box.left + ((window.scrollX || doc && doc.scrollLeft || 0) - (doc && doc.clientLeft || 0)),
    top: box.top + ((window.scrollY || doc && doc.scrollTop || 0) - (doc && doc.clientTop || 0))
  };
}



function getByClass(element, className) {
  return element.getElementsByClassName ? element.getElementsByClassName(className) : element.querySelectorAll('.' + className);
}





function closest(element, selector) {
  if (Element.prototype.closest) {
    return element.closest(selector);
  } else {
    // IE
    var el = this;
    Element.prototype.matches = Element.prototype.msMatchesSelector || Element.prototype.webkitMatchesSelector;
    if (!document.documentElement.contains(el)) return null;
    do {
      if (el.matches(selector)) return el;
      el = el.parentElement;
    } while (el !== null);
    return null;
  }
}







// Force reflow to enable CSS3 transition


// e.g.: http://domain.com/path/to/picture.jpg?size=1280×960 -> picture.jpg












function createDom(str) {
  if (typeOf(str) === 'string') {
    var dom = document && document.createElement('div');
    dom.innerHTML = str;
    return dom.firstChild;
  }
  return str;
}

function contains(root, el) {
  if (root.compareDocumentPosition) return root === el || !!(root.compareDocumentPosition(el) & 16);
  if (root.contains && el.nodeType === 1) {
    return root.contains(el) && root !== el;
  }
  while (el = el.parentNode) {
    if (el === root) return true;
  }return false;
}

function getPosOfEvent(ev) {
  ev = ev.originalEvent ? ev.originalEvent : ev;
  if (ev.type && isTouch(ev.type)) {
    var posi = [];
    var src = null;
    var eventList = void 0;
    if (ev.touches.length > 0) {
      eventList = ev.touches;
    } else {
      eventList = ev.changedTouches;
    }

    for (var t = 0, len = eventList.length; t < len; t++) {
      src = eventList[t];
      posi.push({
        x: src.pageX,
        y: src.pageY
      });
    }
    return posi;
  } else {
    return [{
      x: ev.pageX,
      y: ev.pageY
    }];
  }
}

function isMobile() {
  return window.navigator.userAgent.match(/Android|iPhone|iPad|iPod|BlackBerry|IEMobile/i);
}

function getRandomId(name) {
  return name + (Math.random() * 10 + new Date() * 1).toString().replace('.', '');
}

function isTouch(type) {
  return (/^(touchend|touchcancel|touchstart|touchmove)$/.test(type)
  );
}

function getCss(domObj, attribute) {
  var style = domObj.style[attribute];
  if (style) {
    return style;
  }
  return domObj.currentStyle ? domObj.currentStyle[attribute] : document.defaultView.getComputedStyle(domObj, '')[attribute];
}

function templateReplace(tmpl, data) {
  return tmpl.replace(/{\s*([\w\-\/]+)\s*}/g, function (m, $1) {
    return data[$1] == null ? '' : data[$1];
  });
}

function getWidth(el) {
  var width = el.offsetWidth;
  if (!width) {
    width = parseInt(getCss(el, 'width'));
  }
  return width;
}

var _$3 = {
    IMAGE_MIN_WIDTH: 50,
    IMAGE_MIN_HEIGHT: 50,
    IMAGE_MAX_WIDTH: 256,
    IMAGE_MAX_HEIGHT: 176,
    SPECIAL_WIDTH: 20,
    SPECIAL_HEIGHT: 80,
    VIEWER_MIN_WIDTH: 400,
    VIEWER_MIN_HEIGHT: 300,
    TOP_VIEWER: null,
    IMAGE_INSTANCES: {},
    clear: function clear(key) {
        var imageview = this.IMAGE_INSTANCES[key];
        if (imageview) {
            imageview.destroy();
        }
        this.IMAGE_INSTANCES[key] = null;
        delete this.IMAGE_INSTANCES[key];
    },
    getKeyId: function getKeyId(key) {
        return key.split('|')[1];
    }
};

var DEFAULT$1 = {
    fullContainer: document.body, // 全屏预览的容器
    src: '', //传入地址，数组或字符串
    currentIndex: 0, //当前位置的索引
    modal: true, //是否显示模态窗口，背景不可操作
    lock: false, //是否锁定，事件无法执行
    editable: true, //是否可编辑
    enableFull: true, // 是否支持全屏
    width: 400, //限定宽度，暂时不使用
    height: 300, // 同宽度
    maxRatio: 10, //最大放大限定 暂时没有用
    swip: false, // 是否可以左右切换
    extendStyle: '', // blue
    resize: true, // 是否允许拖动修改大小
    showScaleBar: false, // 缩放是否按照加减法。false 默认2倍。true 为加减
    hasDecPicList: false, // 是否显示描述栏和图片列表
    showTitle: false, // 是否图片标题
    lang: 'zh', //语言,
    defaultSrc: '' //加载失败时候的默认图片
};

/**
 * Created by ylf on 2017/2/4.
 * 常用方法
 */

var utils$2 = {};

utils$2.getDistance = function (pos1, pos2) {
    var x = pos2.x - pos1.x,
        y = pos2.y - pos1.y;
    return Math.sqrt(x * x + y * y);
};

utils$2.getFingers = function (ev) {
    return ev.touches ? ev.touches.length : 1;
};

utils$2.calScale = function (pstart, pmove, intensity) {
    if (pstart.length >= 2 && pmove.length >= 2) {
        var disStart = this.getDistance(pstart[1], pstart[0]),
            disEnd = this.getDistance(pmove[1], pmove[0]),
            scale = (disEnd - disStart) * intensity;
        return (disEnd - scale) / disStart;
    }
    return 1;
};

utils$2.getPosOfEvent = function (ev) {
    if (this.isTouch(ev.type)) {
        var posi = [];
        var src = null;
        var eventList = void 0;
        if (ev.touches.length > 0) {
            eventList = ev.touches;
        } else {
            eventList = ev.changedTouches;
        }

        for (var t = 0, len = eventList.length; t < len; t++) {
            src = eventList[t];
            posi.push({
                x: src.pageX,
                y: src.pageY
            });
        }
        return posi;
    } else {
        return [{
            x: ev.pageX,
            y: ev.pageY
        }];
    }
};

utils$2.isDown = function (type) {
    return type === 'touchstart' || type === 'mousedown';
};
utils$2.isUp = function (type) {
    return (/^(touchend|touchcancel|mouseup|mouseleave|mouseout)$/.test(type)
    );
};
utils$2.isMove = function (type) {
    return type === 'mousemove' || type === 'touchmove';
};
utils$2.isTouch = function (type) {
    return (/^(touchend|touchcancel|touchstart|touchmove)$/.test(type)
    );
};

utils$2.isTouchEnd = function (ev) {
    return ev.type === 'touchend' || ev.type === 'touchcancel';
};

utils$2.isTouchMove = function (ev) {
    return ev.type === 'touchmove';
};

utils$2.proxy = function (fuc, context) {
    return function () {
        var args = Array.prototype.slice.call(arguments, 0);
        fuc.apply(context, args);
    };
};

utils$2.isMobile = window.navigator.userAgent.match(/Android|iPhone|iPad|iPod|BlackBerry|IEMobile/i);

utils$2.hasTouch = 'ontouchend' in window || typeof navigator.maxTouchPoints === "number" && navigator.maxTouchPoints > 0;

utils$2.extend = function (obj, obj2) {
    for (var key in obj2) {
        if (obj2.hasOwnProperty(key)) {
            obj[key] = obj2[key];
        }
    }
    return obj;
};

utils$2.getUuid = function () {
    var s = [],
        hexDigits = '0123456789abcdef',
        uuid;

    for (var i = 0; i < 36; i++) {
        s[i] = hexDigits.substr(Math.floor(Math.random() * 0x10), 1);
    }

    s[14] = '4'; // bits 12-15 of the time_hi_and_version field to 0010
    s[19] = hexDigits.substr(s[19] & 0x3 | 0x8, 1); // bits 6-7 of the clock_seq_hi_and_reserved to 01
    s[8] = s[13] = s[18] = s[23] = '-';

    uuid = s.join('');
    return uuid;
};

utils$2.isElementBody = function (ele) {
    if (ele instanceof HTMLElement) {
        return ele.tagName.toUpperCase() === 'BODY';
    }
    return false;
};

var Config = {
    evtElem: document.body,
    tap: true,
    doubleTap: true,
    drag: true,
    pinch: true,
    hold: true,
    tapMaxDistance: 50,
    tapTime: 600,
    holdTime: 1000,
    doubleTapMaxInterval: 300,
    doubleTapMaxDistance: 16,
    pinchIntensity: 0.8,
    eventPrefix: ''
};

var EVENT = {
    PINCH_START: 'pinchstart',
    PINCH: 'pinch',
    PINCH_END: 'pinchend',
    DRAG_START: 'dragstart',
    DRAG: 'drag',
    DRAG_END: 'dragend',
    HOLD: 'hold',
    TAP: 'avitap',
    DOUBLE_TAP: 'dbltap',
    FOCUS: 'mfocus',
    BLUR: 'mblur'
};

/**
 * 底层事件绑定
 *
 **/

var Engine = {
    trigger: function trigger(el, evt, detail) {
        var e = void 0,
            opt = {
            bubbles: true,
            cancelable: true,
            detail: detail || {}
        };

        if (typeof CustomEvent === 'function') {
            e = new CustomEvent(evt, opt);
            el.dispatchEvent(e);
        } else {
            e = document.createEvent('CustomEvent');
            e.initCustomEvent(evt, true, true, detail);
            el.dispatchEvent(e);
        }
    },
    bind: function bind(el, evt, handler) {
        //let that = this
        var proxy = function proxy(e) {
            //兼容处理，click这些原生事件
            e.originalEvent = e;

            for (var key in e.detail) {
                if (e.detail.hasOwnProperty(key) && key !== 'type') {
                    e[key] = e.detail[key];
                }
            }

            var returnVal = handler.call(e.target, e);

            if (typeof returnVal !== 'undefined' && !returnVal) {
                e.stopPropagation();
                e.preventDefault();
            }
        };

        handler.proxy = proxy;
        var evts = evt.match(/\S+/g);
        evts.forEach(function (e) {
            el.addEventListener(e, handler.proxy, false);
        });
    },
    unbind: function unbind(el, evt, handler) {
        var evts = evt.match(/\S+/g);
        evts.forEach(function (e) {
            el.removeEventListener(e, handler.proxy, false);
        });
    },
    on: function on() {
        var args = Array.prototype.slice.apply(arguments, [0]),
            elem = typeof args[0] === 'string' ? document.querySelectorAll(args[0]) : args[0],
            evt = args[1],
            handler = args[2];

        this.bind(elem, evt, handler);
    }
};

Engine.off = Engine.unbind;

/**
 * Created by ylf on 2017/2/4.
 */
var Core = function () {
    function Core(config) {
        classCallCheck(this, Core);

        this.uuid = utils$2.getUuid();
        this._config = utils$2.extend(Object.create(Config), config);
        this._downStart = false;
        this._pinchStart = false;
        this._dragStart = false;
        this._tap = false;
        this._startTime = 0;
        this._startPos = null;
        this._movePos = null;
        this._endPos = null;
        this._lastTarget = null;
        this._dragElem = null;
        this._holdTimer = 0;

        this.bindEvent();
    }

    createClass(Core, [{
        key: 'getConfig',
        value: function getConfig() {
            return this._config;
        }
    }, {
        key: 'reset',
        value: function reset() {
            this._downStart = false;
            this._pinchStart = false;
            this._dragStart = false;
            this._tap = false;
            this._startTime = 0;
            this._startPos = null;
            this._movePos = null;
            this._endPos = null;
            this._dragElem = null;
            clearTimeout(this._holdTimer);
            this._holdTimer = 0;
        }
    }, {
        key: 'bindEvent',
        value: function bindEvent() {
            var that = this,
                mouseEvents = ['mouseup', 'mousedown', 'mousemove', 'mouseleave'],
                touchEvents = ['touchstart', 'touchmove', 'touchend', 'touchcancel'],
                events = mouseEvents,
                config = that._config;

            if (utils$2.isMobile) {
                events = touchEvents;
            } else if (utils$2.hasTouch) {
                events = touchEvents.concat(mouseEvents);
            }

            that.handler = utils$2.proxy(that.handler, this);
            events.forEach(function (evt) {
                config.evtElem.addEventListener(evt, that.handler, false);
            });
        }
    }, {
        key: 'downHandler',
        value: function downHandler(evt, target) {
            Logger$1.debug('downHandler');
            this._downStart = true;
            if (!this._startPos || this._startPos.length < 2) {
                this._startPos = utils$2.getPosOfEvent(evt);
            }
            this._startTime = new Date().getTime();

            this.focus(evt);
            this.hold(evt);

            this._lastTarget = target;
        }
    }, {
        key: 'moveHandler',
        value: function moveHandler(evt) {
            Logger$1.debug('moveHandler');
            this._movePos = utils$2.getPosOfEvent(evt);

            if (utils$2.getFingers(evt) >= 2) {
                this.pinch(evt);
            } else {
                this.drag(evt);
            }
        }
    }, {
        key: 'upHandler',
        value: function upHandler(evt) {
            Logger$1.debug('upHandler');
            this._endPos = utils$2.getPosOfEvent(evt);

            if (this._pinchStart) {
                Logger$1.debug('deal with upHandler');
                this.pinch(evt);
            } else if (this._dragStart) {
                Logger$1.debug('deal with drag');
                this.drag(evt);
            } else if (this._tap) {
                Logger$1.debug('deal with tap');
                this.tap(evt);
            }

            this.reset();
        }
    }, {
        key: 'handler',
        value: function handler(evt) {
            var type = evt.type,
                target = evt.target;

            if (utils$2.isDown(type)) {
                this.downHandler(evt, target);
            } else if (utils$2.isUp(type)) {
                if (!this._downStart) {
                    return;
                }
                if (target == this._lastTarget) {
                    this._tap = true;
                    Logger$1.debug('_tap is true');
                }
                this._dragStart = false;
                this._pinchStart = false;
                this.upHandler(evt);
            } else if (utils$2.isMove(type)) {
                if (!this._downStart || !this._startPos) {
                    return;
                }
                Logger$1.debug('_move is true');
                this.moveHandler(evt);
            }
        }
    }, {
        key: 'tap',
        value: function tap(evt) {

            var time = new Date().getTime(),
                elem = evt.target,
                config = this._config,
                subTime = time - this._startTime,
                distance = utils$2.getDistance(this._startPos[0], this._endPos[0]);
            Logger$1.debug('tap judge'); //
            if (subTime < config.holdTime && distance <= config.tapMaxDistance && subTime <= config.tapTime) {
                Logger$1.debug(' tap start');
                if (this._startTime - this.__lastTapTime <= config.doubleTapMaxInterval && this.__lastTapPos && utils$2.getDistance(this.__lastTapPos, this._endPos[0]) <= config.doubleTapMaxDistance && this.__lastTapElem == elem) {
                    Logger$1.debug('double tap start');
                    this.trigger(elem, evt, EVENT.DOUBLE_TAP);
                    this.__lastTapTime = 0;
                    this.__lastTapPos = null;
                    this.__lastTapElem = null;
                } else {
                    Logger$1.debug('tap trigger ');
                    this.trigger(elem, evt, EVENT.TAP);
                    this.__lastTapTime = time;
                    this.__lastTapPos = this._endPos[0];
                    this.__lastTapElem = elem;
                }
            } else {
                Logger$1.debug("tap hold long time");
            }
        }
    }, {
        key: 'drag',
        value: function drag(evt) {
            if (!this._startPos) {
                return;
            }

            var elem = evt.target;

            if (!this._dragStart) {
                var startDis = utils$2.getDistance(this._startPos[0], this._movePos[0]);
                if (startDis > 1) {
                    this._dragElem = elem;
                    Logger$1.debug('drag trigger ');
                    this.trigger(elem, evt, EVENT.DRAG_START);
                    this.trigger(elem, evt, EVENT.DRAG);
                    this._dragStart = true;
                }
            } else {
                if (utils$2.isMove(evt.type)) {
                    this.trigger(this._dragElem, evt, EVENT.DRAG);
                } else if (utils$2.isUp(evt.type)) {
                    this.trigger(this._dragElem, evt, EVENT.DRAG_END);
                }
            }
        }
    }, {
        key: 'hold',
        value: function hold(evt) {
            var that = this;
            clearTimeout(this._holdTimer);
            this._holdTimer = setTimeout(function () {
                if (!that._startPos) {
                    return;
                }
                var distance = utils$2.getDistance(that._startPos[0], that._movePos ? that._movePos[0] : that._startPos[0]);
                if (that._config.tapMaxDistance < distance) {
                    return;
                }

                that.trigger(evt.target, evt, EVENT.HOLD, {
                    type: EVENT.HOLD,
                    originEvent: evt,
                    fingersCount: utils$2.getFingers(evt),
                    position: that._startPos[0]
                });
            }, this._config.holdTime);
        }
    }, {
        key: 'pinch',
        value: function pinch(evt) {
            if (!this._downStart) {
                return;
            }
            if (utils$2.getFingers(evt) < 2 && !utils$2.isTouchEnd(evt)) {
                return;
            }

            var config = this._config,
                scale = utils$2.calScale(this._startPos, this._movePos, config.pinchIntensity),
                fingersCount = utils$2.getFingers(evt),
                eventObj = {
                originEvent: evt,
                scale: scale,
                fingersCount: fingersCount
            };

            if (!this._pinchStart) {
                this._pinchStart = true;
                this.trigger(evt.target, evt, EVENT.PINCH_START, eventObj);
            } else if (utils$2.isTouchMove(evt)) {
                this.trigger(evt.target, evt, EVENT.PINCH, eventObj);
            } else if (utils$2.isTouchEnd(evt)) {
                this.trigger(evt.target, evt, EVENT.PINCH_END, eventObj);
            }
        }
    }, {
        key: 'focus',
        value: function focus(evt) {
            var elem = evt.target,
                lastElem = this._lastTarget;

            if (lastElem !== elem && lastElem) {
                this.trigger(lastElem, evt, EVENT.BLUR);
                this.trigger(elem, evt, EVENT.FOCUS);
            } else if (!lastElem) {
                this.trigger(elem, evt, EVENT.FOCUS);
            }
        }
    }, {
        key: 'trigger',
        value: function trigger(elem, originalEvt, evt, detail) {
            detail = detail || {
                originalEvent: originalEvt
            };
            var prefix = this._config.eventPrefix;
            if (prefix) {
                evt = prefix + evt;
            }
            Engine.trigger(elem, evt, detail, this._config.eventPrefix);
        }
    }]);
    return Core;
}();

/**
 *  create by ylf 2017/2/6
 */

var EventApi = function () {
    function EventApi(config) {
        classCallCheck(this, EventApi);

        this.core = new Core(config);
    }

    createClass(EventApi, null, [{
        key: 'on',
        value: function on() {
            var args = Array.prototype.slice.apply(arguments, [0]);
            //this.core.engine.on.apply(this.core.engine, args)
            Engine.on.apply(Engine, args);
        }
    }, {
        key: 'off',
        value: function off() {
            var args = Array.prototype.slice.apply(arguments, [0]);
            Engine.off.apply(Engine, args);
        }
    }]);
    return EventApi;
}();

/**
 * this is a event delegate 
 * created by ccr
 */
var registerEvents = void 0;
var self$1 = void 0; //不使用逶拖的方式来注册事件，否则会造成事件无法回收的问题
/**
 * 全局事件订阅
 */
var Events$2 = {
    eventList: {},
    down: false,
    key: '',
    init: function init() {
        self$1 = this;
        if (!registerEvents) {
            registerEvents = new EventApi({
                evtElem: document.body
            });
        }
        return registerEvents;
    },
    bind: function bind() {
        self$1 = this;
        EventApi.on(document.body, 'avitap dragstart drag dragend pinchstart pinch pinchend', self$1.handler);
    },

    unbind: function unbind() {
        self$1 = this;
        EventApi.off(document.body, 'avitap dragstart drag dragend pinchstart pinch pinchend', self$1.handler);
    },
    registerEvent: function registerEvent(obj, events) {
        if (obj == null || events == null) {
            return;
        }
        events = events.split(' ');
        for (var i = 0, len = events.length; i < len; i++) {
            var eventName = events[i];
            if (self$1.eventList[eventName] && self$1.eventList[eventName].indexOf(obj) < 0) {
                self$1.eventList[eventName].push(obj);
            } else if (self$1.eventList[eventName] == null || self$1.eventList[eventName].length == 0) {
                self$1.eventList[eventName] = [obj];
            }
        }
    },
    unregisterEvent: function unregisterEvent(obj, events) {
        if (obj == null || events == null) {
            return;
        }
        events = events.split(' ');
        for (var i = 0, len = events.length; i < len; i++) {
            var eventName = events[i];
            var index = self$1.eventList[eventName].indexOf(obj);
            if (index > -1) {
                self$1.eventList[eventName].splice(index, 1);
            }
        }
    },
    handler: function handler(event) {
        event = getEvent(event);
        var target = event.target;
        var objs = self$1.eventList[event.type] || [];
        for (var i = 0, len = objs.length; i < len; i++) {
            var obj = objs[i];
            if (obj.container && contains(obj.container, target)) {
                obj.$handle(event);
            }
        }
        return;
    }
};

var MobileViewer = function () {
    function MobileViewer(options) {
        classCallCheck(this, MobileViewer);

        this.options = extend({}, DEFAULT$1, options);
        Events$2.registerEvent(this, 'avitap dragstart drag dragend pinchstart pinch pinchend');
        this.$createHtml();
        this.$render();
        this.fullScreenChange = this.options.onFullScreenChange || this.options.eventHandler && this.options.eventHandler.fullScreenChange;
        var self = this;
        this.$orientationChange = proxy(self.$orientationChange, self);
        window.addEventListener("onorientationchange" in window ? "orientationchange" : "resize", this.$orientationChange, false);
    }

    createClass(MobileViewer, [{
        key: '$render',
        value: function $render() {
            this.viewerData = {
                width: 0,
                height: 0,
                src: ''
            };
            this.viewerData.src = this.options.src;
            this.fullWidth = document.documentElement.clientWidth;
            this.fullHeight = document.documentElement.clientHeight;
            this.cacheLastStatus = {};
            this.$init();
            this.close();
        }
    }, {
        key: '$init',
        value: function $init() {
            var _this = this;

            var self = this;
            self.$loadImage(function (w, h) {
                self.viewerData.width = w;
                self.viewerData.height = h;
                _this.$resize();
                addClass(self.loading, 'hide-dom');
            });
            this.viewerImg.src = this.viewerData.src;
        }
    }, {
        key: '$resize',
        value: function $resize() {
            //横向
            var self = this;
            var _w = self.viewerData.width;
            var _h = self.viewerData.height;
            var w = void 0,
                h = void 0;
            this.fullWidth = document.documentElement.clientWidth;
            this.fullHeight = document.documentElement.clientHeight;

            if (_w / _h > this.fullWidth / this.fullHeight && _w > this.fullWidth) {
                w = this.fullWidth;
                h = _h * w / _w;
            } else if (_h > this.fullHeight) {
                h = this.fullHeight;
                w = _w * h / _h;
            } else {
                h = _h;
                w = _w;
            }
            self.viewerImg.style.width = w + 'px';
            self.viewerData.width = w;
            self.viewerData.height = h;
            var left = void 0,
                top = void 0;
            top = (this.fullHeight - h) / 2;
            left = (this.fullWidth - w) / 2;
            self.viewerImg.style.top = top + 'px';
            self.viewerImg.style.left = left + 'px';
        }
    }, {
        key: '$createHtml',
        value: function $createHtml() {
            var templateDom = Template.mobilefull;
            var templateMask = Template.mask;
            templateDom = createDom(templateDom);
            this.viewerImg = templateDom.querySelector('img');
            //设置外部闯入得样式
            if (this.options.mobileViewer.imgStyle) {
                this.viewerImg.setAttribute('style', this.options.mobileViewer.imgStyle);
            }
            this.loading = templateDom.querySelector('.insert-pictures__loading');
            this.container = templateDom;
            templateMask = createDom(templateMask);
            this.mask = templateMask;
            this.options.fullContainer.appendChild(templateMask);
            this.options.fullContainer.appendChild(templateDom);
        }
    }, {
        key: '$loadImage',
        value: function $loadImage(callback) {
            var self = this;
            var img = new Image();
            img.src = self.viewerData.src;
            if (img.complete) {
                if (callback) {
                    callback(img.width, img.height);
                }
            } else {
                img.onload = function () {
                    if (callback) {
                        callback(img.width, img.height);
                    }
                    img.onload = null;
                };
                img.onerror = function () {
                    self.viewerData.src = self.options.defaultSrc;
                    self.viewerImg.src = self.options.defaultSrc;
                    img.src = self.options.defaultSrc;
                    img.onerror = null;
                };
            }
        }
    }, {
        key: '$initOffset',
        value: function $initOffset(p, _w, _h, changeWidth) {
            var x = p && p.x,
                y = p && p.y;
            this.viewerImg.style.width = changeWidth + 'px';
            var w = this.viewerImg.clientWidth,
                h = this.viewerImg.clientHeight;
            var left = this.viewerImg.offsetLeft,
                top = this.viewerImg.offsetTop;
            Logger$1.debug("mobile full", _w, _h, this.fullHeight, this.fullWidth);
            if (p && w > this.fullWidth && h > this.fullHeight) {
                Logger$1.debug("mobile viewer finger center scale");
                var imgLeft = p.x - (p.x - left) * w / _w;
                var imgTop = p.y - (p.y - top) * h / _h;
                imgLeft = imgLeft > 0 ? 0 : imgLeft < this.fullWidth - w ? this.fullWidth - w : imgLeft;
                imgTop = imgTop > 0 ? 0 : imgTop < this.fullHeight - h ? this.fullHeight - h : imgTop;
                this.viewerImg.style.left = imgLeft + 'px';
                this.viewerImg.style.top = imgTop + 'px';
            } else {
                Logger$1.debug("mobile viewer center scale");
                this.viewerImg.style.left = (this.fullWidth - w) / 2 + 'px';
                this.viewerImg.style.top = (this.fullHeight - h) / 2 + 'px';
            }
        }
    }, {
        key: '$handle',
        value: function $handle(event) {
            var _this2 = this;

            event = getEvent(event);
            var target = event.target;
            var self = this;
            event ? event.stopPropagation() : window.event.cancelBubble = true;
            event.preventDefault();

            var originalEvent = event.originalEvent;
            originalEvent && (originalEvent.stopPropagation ? originalEvent.stopPropagation() : originalEvent.cancelBubble && (originalEvent.cancelBubble = true));
            originalEvent.preventDefault && originalEvent.preventDefault();

            var point = getPosOfEvent(event);

            var px = point[0].x;
            var py = point[0].y;
            switch (event.type) {
                case 'avitap':
                    Logger$1.debug('mobile avitap');
                    var time = setTimeout(function () {
                        self.close();
                        clearTimeout(time);
                    }, 10);
                    break;
                case 'dragstart':
                    Logger$1.debug('mobile dragstart');
                    if (target == this.viewerImg) {
                        if (this.viewerImg.clientWidth <= this.fullWidth && this.viewerImg.clientHeight <= this.fullHeight) {
                            return;
                        }
                        this.isDrag = true;
                        this.wheeling = false;
                        this.cacheLastStatus = this.cacheLastStatus || {};
                        this.cacheLastStatus.x = px;
                        this.cacheLastStatus.y = py;
                        this.cacheLastStatus.width = this.viewerImg.clientWidth;
                        this.cacheLastStatus.height = this.viewerImg.clientHeight;
                    }
                    break;
                case 'drag':
                    if (this.isDrag) {
                        //Logger.debug('mobile drag');
                        if (this.wheeling) {
                            return;
                        }
                        this.wheeling = true;
                        var id = setTimeout(function () {
                            _this2.wheeling = false;
                            clearTimeout(id);
                        }, 60);

                        var w = this.cacheLastStatus.width,
                            h = this.cacheLastStatus.height;

                        if (w > this.fullWidth) {
                            var imageLeft = px - this.cacheLastStatus.x + this.viewerImg.offsetLeft;
                            if (imageLeft <= 0 && imageLeft >= this.fullWidth - w) {
                                this.viewerImg.style.left = imageLeft + 'px';
                            }
                            this.$moveFlag = true;
                        }
                        if (h > this.fullHeight) {
                            var imageTop = py - this.cacheLastStatus.y + this.viewerImg.offsetTop;
                            if (imageTop <= 0 && imageTop >= this.fullHeight - h) {
                                this.viewerImg.style.top = imageTop + 'px';
                            }
                            this.$moveFlag = true;
                        }
                        if (this.$moveFlag) {
                            this.cacheLastStatus.x = px;
                            this.cacheLastStatus.y = py;
                        }
                    }

                    break;
                case 'dragend':
                    this.isDrag = false;
                    Logger$1.debug('mobile dragend');
                    break;
                case 'pinchstart':
                    if (target == this.viewerImg) {
                        this.isPinch = true;
                        this.wheeling = false;
                        this.cacheLastStatus = this.cacheLastStatus || {};
                        this.cacheLastStatus.width = this.viewerImg.clientWidth;
                        this.cacheLastStatus.height = this.viewerImg.clientHeight;
                    }
                    Logger$1.debug('mobile pinchstart');
                    break;
                case 'pinch':
                    if (this.isPinch) {
                        if (this.wheeling) {
                            return;
                        }
                        this.wheeling = true;
                        var _id = setTimeout(function () {
                            _this2.wheeling = false;
                            clearTimeout(_id);
                        }, 60);

                        //Logger.debug('mobile pinch');
                        var scale = event.scale || 1;
                        scale = parseInt(scale * 100) / 100;
                        var _w2 = void 0,
                            _h2 = void 0,
                            _w = void 0,
                            _h = void 0;

                        _w = this.cacheLastStatus.width;
                        _h = this.cacheLastStatus.height;

                        if (scale > 1) {
                            scale = scale * 1.5;
                        } else {
                            scale = scale * 1 / 1.5;
                        }

                        // Logger.debug('mobile scale' + scale);
                        _w2 = parseInt(_w * scale * 100) / 100;
                        _h2 = parseInt(_h * scale * 100) / 100;

                        // Logger.debug('mobile width' + w);
                        //Logger.debug('mobile height' + h);
                        var touchs = event.originEvent && event.originEvent.changedTouches;
                        var centerPoint = null;
                        if (touchs && touchs.length > 1) {
                            var point1 = getOffset(this.viewerImg);
                            centerPoint = {
                                x: (touchs[0].clientX + touchs[1].clientX) / 2,
                                y: (touchs[0].clientY + touchs[1].clientY) / 2
                            };
                            var flag = false,
                                changeWidth = void 0;
                            if (_w2 >= this.viewerData.width && _h2 >= this.viewerData.height && _w2 <= this.viewerData.width * 20 && _h2 < this.viewerData.height * 20) {
                                //设置 left top 值
                                //Logger.debug('mobile offset');
                                changeWidth = _w2;
                                flag = true;
                            } else if (_w2 < this.viewerData.width && _h2 < this.viewerData.height && _w >= this.viewerData.width && _h >= this.viewerData.height && _w <= this.viewerData.width * 20 && _h < this.viewerData.height * 20) {
                                changeWidth = this.viewerData.width;
                                //如果缩放时候小于最小值的时候取原大小
                                flag = true;
                            } else if (_w2 > _w && _h2 > _h && _w2 > this.viewerData.width * 20 && _h2 > this.viewerData.height * 20) {
                                //如果放大最大值的时候大于20倍时取最大值
                                changeWidth = this.viewerData.width * 20;
                                flag = true;
                            }
                            flag && this.$initOffset(centerPoint, _w, _h, changeWidth);
                        }
                    }
                    break;
                case 'pinchend':
                    this.isPinch = false;
                    Logger$1.debug('mobile pinchend');
                    break;
                default:
                    break;
            }
            return false;
        }
    }, {
        key: 'updateSrc',
        value: function updateSrc() {
            var arg = arguments;
            if (arg.length == 0) {
                return;
            }
            if (arg.length == 1) {
                var arg1 = arg[0];
                if (isString(arg1)) {
                    this.viewerData.src = this.options.src = arg1;
                    this.$init();
                }
            }
        }
    }, {
        key: 'open',
        value: function open() {
            setStyle(document.body, {
                "overflow": "hidden"
            });
            this.container && (this.container.style.display = '');
            if (this.mask && hasClass(this.mask, 'hide-dom')) {
                removeClass(this.mask, 'hide-dom');

                window.AviMediaPlayer && window.AviMediaPlayer.pauseAll();
                var videos = document.querySelectorAll('.component__video video');
                videos.forEach(function (item) {
                    item.style.display = 'none';
                });
                if (this.fullScreenChange) {
                    this.fullScreenChange(true);
                }
            }
        }
    }, {
        key: 'close',
        value: function close() {
            setStyle(document.body, {
                "overflow": ""
            });
            this.container.style.display = 'none';
            if (this.mask && !hasClass(this.mask, 'hide-dom')) {
                addClass(this.mask, 'hide-dom');
                var videos = document.querySelectorAll('.component__video video');
                videos.forEach(function (item) {
                    item.style.display = '';
                });
                if (this.fullScreenChange) {
                    this.fullScreenChange(false);
                }
            }
        }
    }, {
        key: '$orientationChange',
        value: function $orientationChange(event) {
            // 根据event.orientation|screen.orientation.angle等于0|180、90|-90度来判断横竖屏
            event.preventDefault();
            event.stopPropagation();
            Logger$1.debug('监听到屏幕方向改变');
            if (window.orientation == 180 || window.orientation == 0) {
                this.setVertical();
                Logger$1.debug('设置为竖屏幕');
            }
            if (window.orientation == 90 || window.orientation == -90) {
                this.setCross();
                Logger$1.debug('设置为横屏幕');
            }
            if (event.type == 'resize') {
                this.$resize();
            }
        }
    }, {
        key: 'setCross',
        value: function setCross() {
            this.container && addClass(this.container, 'show__cross');
            this.$resize();
        }
    }, {
        key: 'setVertical',
        value: function setVertical() {
            this.container && removeClass(this.container, 'show__cross');
            this.$resize();
        }
    }, {
        key: 'destroy',
        value: function destroy() {
            Logger$1.debug('销毁mobileviewer');
            var self = this;
            //事件解绑
            Events$2.unregisterEvent(this, 'avitap dragstart drag dragend pinchstart pinch pinchend');
            window.removeEventListener("onorientationchange" in window ? "orientationchange" : "resize", this.$orientationChange, false);
            this.mask.ontouchstart = null;
            //dom移除
            this.container && this.container.parentNode.removeChild(this.container);
            this.mask && this.mask.parentNode.removeChild(this.mask);
            //数据清空
            this.mask = null;
            this.viewerImg = null;
            this.loading = null;
            this.container = null;
            this.viewerData = null;
            if (_$3.TOP_VIEWER && _$3.TOP_VIEWER == this) {
                _$3.TOP_VIEWER = null;
            }
        }
    }]);
    return MobileViewer;
}();

var zh = {
    'preview': '预览',
    'max': '最大化',
    'min': '向下还原',
    'close': '关闭',
    'fileSize': '文件大小',
    'fullScreen': '全屏'
};
var en = {
    'preview': 'Preview',
    'max': 'Maximise',
    'min': 'Restore Down',
    'close': 'Close',
    'fileSize': '文件大小',
    'fullScreen': '全屏'
};

var Lang = {
    'zh': zh,
    'zh_cn': zh,
    'en': en,
    'en_us': en,
    'ar_sa': {
        'preview': 'تصفح',
        'max': 'أكبر',
        'min': 'استرجع',
        'close': 'إغلاق',
        'fileSize': '文件大小',
        'fullScreen': '全屏'
    },
    'ja_jp': {
        'preview': 'プレビュー',
        'max': '最大化',
        'min': '',
        'close': '关闭',
        'fileSize': '文件大小',
        'fullScreen': '全屏'
    },
    'ms_my': {
        'preview': 'Pratonton',
        'max': 'Maksimumkan',
        'min': 'Ke bawah untuk pulihkan',
        'close': 'Tutup',
        'fileSize': '文件大小',
        'fullScreen': '全屏'
    },
    'ru_ru': {
        'preview': 'Предварительный просмотр',
        'max': 'Развернуть',
        'min': 'Восстановить вниз',
        'close': 'Закрыть',
        'fileSize': '文件大小',
        'fullScreen': '全屏'
    },
    'th_th': {
        'preview': 'พรีวิว',
        'max': 'ขยายใหญ่สุด',
        'min': 'ย่อหน้าต่าง',
        'close': 'ปิด',
        'fileSize': '文件大小',
        'fullScreen': '全屏'
    },
    'tr_tr': {
        'preview': 'Önizleme',
        'max': 'Ekranı Kapla',
        'min': 'Geri yükleme',
        'close': 'Kapat',
        'fileSize': '文件大小',
        'fullScreen': '全屏'
    },
    'zh_hk': {
        'preview': '預覽',
        'max': '最大化',
        'min': '向下還原',
        'close': '關閉',
        'fileSize': '文件大小',
        'fullScreen': '全屏'
    }
};

var EXTEND_STYLE$2 = {
    'blue': {
        'className': 'style--pop-blue'
    }
};

var PcViewer = function () {
    function PcViewer(options) {
        classCallCheck(this, PcViewer);

        this.options = extend({}, DEFAULT$1, options);
        if (!this.options.fullContainer) {
            this.options.fullContainer = document.body;
        }
        Events$2.registerEvent(this, 'dragstart drag dragend pinchstart pinch pinchend');
        this.$createHtml();
        this.$render();
    }

    createClass(PcViewer, [{
        key: '$render',
        value: function $render() {
            this.viewerData = {
                src: '',
                width: 0,
                height: 0,
                scale: 1,
                minScale: 1,
                maxScale: 1,
                stableWidth: 0,
                stableHeight: 0,
                containerWidth: 0,
                containerHeight: 0
            };
            this.bound = 8; //距离外窗8px内可以显示缩放的鼠标型状
            this.padding = 2; //图片的边距
            this.viewerData.src = this.options.src;
            this.currentIndex = this.options.currentIndex;
            this.viewerData.width = this.options.width;
            this.viewerData.height = this.options.height;
            // 保存下来固定宽高
            this.viewerData.stableWidth = this.options.width;
            this.viewerData.stableHeight = this.options.height;
            this.isModal = this.options.modal;
            isObject(this.options.lang) ? this.langObject = this.options.lang : this.langObject = Lang[this.options.lang.toLowerCase()] || Lang['zh'];

            this.$initFullSize();
            if (this.options.src && typeOf(this.options.src) == 'array') {
                this.imgLength = this.options.src.length;
                this.currentIndex = this.currentIndex > this.imgLength - 1 ? this.imgLength - 1 : this.currentIndex;
                this.viewerData.src = this.options.src[this.currentIndex];
            } else if (typeOf(this.options.src) == 'string') {
                this.options.src = [this.options.src];
                this.currentIndex = 0;
                this.viewerData.src = this.options.src[this.currentIndex];
                this.imgLength = this.options.src.length;
            } else {
                Logger$1.error("缺少src参数");
                return;
            }
            this.isModal && this.$createMask();
            this.close();
            this.$init();
            this.$setPicSelect(this.currentIndex);
            this.$updateInfo(this.currentIndex);
        }

        /**
         *创建mask dom
         */

    }, {
        key: '$createMask',
        value: function $createMask() {
            var templateDom = Template.mask;
            templateDom = createDom(templateDom);
            addClass(templateDom, 'mask--transparent');
            var hasMask = getByClass(this.options.fullContainer, 'ndui-pop-insert-pictures-mask');
            //存在则不添加，复用旧的dom
            if (hasMask == null || hasMask.length < 1) {
                if (!utils$2.isElementBody(this.options.fullContainer)) {
                    addClass(templateDom, 'full-specified-container');
                }
                this.options.fullContainer.appendChild(templateDom);
                this.mask = templateDom;
            } else {
                this.mask = hasMask[0];
            }
        }

        /**
         * 实例模版事件
         */

    }, {
        key: '$createHtml',
        value: function $createHtml() {
            var self = this;
            var templateDom = templateReplace(Template.popwindow, {
                hideClassName: this.options.enableFull ? '' : 'hide_dom'
            });
            templateDom = createDom(templateDom);
            this.viewerImg = templateDom.querySelector('img');
            this.viewerClose = templateDom.querySelector('.head__oper.head__close');
            this.viewCloseInClass = templateDom.querySelector('.pop-insert-pictures__close-inclass');
            this.viewerFull = templateDom.querySelector('.head__oper.head__full');
            this.viewerFull2 = templateDom.querySelector('.pop-pic-btn-full');
            this.viewerHead = templateDom.querySelector('.pop-insert-pictures__head');
            this.viewerBody = templateDom.querySelector('.pop-insert-pictures__body');
            this.swipLeft = templateDom.querySelector('.pic__oper.oper--left');
            this.swipRight = templateDom.querySelector('.pic__oper.oper--right');
            this.loading = templateDom.querySelector('.insert-pictures__loading');
            this.mask = templateDom.querySelector('.ndui-pop-insert-pictures-mask');
            this.viewTitle = templateDom.querySelector('.head__title');
            this.viewExtend = templateDom.querySelector('.pop-pic-extend');
            this.viewDescription = templateDom.querySelector('.pop-pic__description');
            this.textDescription = templateDom.querySelector('._description-text');
            this.viewPicListOuter = templateDom.querySelector('.pop-pic-list');
            this.textCountCurrent = templateDom.querySelector('.js_count_current');
            this.textCountToal = templateDom.querySelector('.js_count_total');
            this.scaleRail = templateDom.querySelector('.pop-pic-scale__rail');
            this.scaleBar = templateDom.querySelector('.pop-pic-scale__bar');
            this.scaleLessBtn = templateDom.querySelector('.pop-pic-scale__less');
            this.scaleMoreBtn = templateDom.querySelector('.pop-pic-scale__more');

            // 是否要显示图片列表
            if (this.options.hasDecPicList) {
                templateDom.classList.add('has-header-extend-list');
                removeClass(this.viewExtend, 'hide_dom');
                removeClass(this.viewPicListOuter, 'hide_dom');
                if (Array.isArray(self.options.src) && self.options.src.length) {
                    this.$initPicList(self.options.src);
                } else {
                    this.$initPicList([self.options.src]);
                }
            }

            this.viewPicList = templateDom.querySelector('.pop-pic-list');

            // 扩展皮肤，增加相应类名
            if (this.options.extendStyle && EXTEND_STYLE$2[this.options.extendStyle]) {
                addClass(templateDom, EXTEND_STYLE$2[this.options.extendStyle].className);
            }
            // 是否显示标题
            if (!this.options.showTitle) {
                addClass(this.viewTitle, 'hide_dom');
            }
            // 关闭按钮是否放在底部
            if (this.options.closeInBottom) {
                removeClass(this.viewCloseInClass, 'hide_dom');
            } else {
                removeClass(this.viewerClose, 'hide_dom');
            }
            this.container = templateDom;
            if (!utils$2.isElementBody(this.options.fullContainer)) {
                addClass(templateDom, 'full-specified-container');
            }
            this.options.fullContainer.appendChild(templateDom);
            self.$handleMousemove = proxy(self.$handleMousemove, self);
            self.$handleWheel = proxy(self.$handleWheel, self);
            self.$handleDblclick = proxy(self.$handleDblclick, self);
            self.$handleCloseViewer = proxy(self.$handleCloseViewer, self);
            self.$handleTap = proxy(self.$handleTap, self);
            self.$handleListClick = proxy(self.$handleListClick, self);
            self.$scaleBarDown = proxy(self.$scaleBarDown, self);
            self.$scaleBarMove = proxy(self.$scaleBarMove, self);
            self.$scaleBarUp = proxy(self.$scaleBarUp, self);
            addListener(document, 'click', self.$handleCloseViewer);
            addListener(this.container, 'dblclick', self.$handleDblclick);
            addListener(this.container, 'mousedown mousemove touchstart touchend mouseup', self.$handleMousemove);
            addListener(this.container, 'avitap', self.$handleTap);
            addListener(this.container, 'mousewheel', self.$handleWheel);
            addListener(this.viewPicListOuter, 'avitap', self.$handleListClick);
            addListener(this.scaleBar, 'mousedown touchstart', self.$scaleBarDown);
            addListener(this.viewerClose, 'mouseover mouseleave touchstart touchend', self.$handleMousemove);
            addListener(this.viewCloseInClass, 'mouseover mouseleave touchstart touchend', self.$handleMousemove);
            addListener(this.viewerFull, ' mouseover mouseleave touchstart touchend', self.$handleMousemove);
        }
    }, {
        key: '$init',
        value: function $init(swipWidth) {
            var self = this;
            this.$loadImage(function (width, height) {
                self.viewerData.width = width;
                self.viewerData.height = height;

                var w = void 0,
                    h = void 0,
                    left = void 0,
                    top = void 0,
                    tmpHeight = void 0;
                var _h = self.fullBodyHeight - self.viewerPaddingHeight * 2;
                var _w = self.fullBodyWidth - self.viewerPaddingWidth * 2;

                if (swipWidth) {
                    var _h2 = self.fullBodyHeight - self.viewerPaddingHeight * 2;
                    w = swipWidth;
                    tmpHeight = height / width * w;
                    if (tmpHeight > _h2) {
                        h = _h2;
                    } else if (tmpHeight < _$3.VIEWER_MIN_HEIGHT) {
                        h = _$3.VIEWER_MIN_HEIGHT;
                    } else {
                        h = tmpHeight;
                    }
                } else if (self.viewerData.width < _$3.VIEWER_MIN_WIDTH || self.viewerData.height < _$3.VIEWER_MIN_HEIGHT) {
                    w = width < _$3.VIEWER_MIN_WIDTH ? _$3.VIEWER_MIN_WIDTH : width > _w ? _w : width;
                    h = height < _$3.VIEWER_MIN_HEIGHT ? _$3.VIEWER_MIN_HEIGHT : height > _h ? _h : height;
                } else {
                    if (width > _w || height > _h) {
                        if (width / height > _w / _h) {
                            if (width > _w) {
                                w = _w;
                            } else if (width < _$3.IMAGE_MIN_WIDTH) {
                                w = _$3.IMAGE_MIN_WIDTH;
                            }
                            h = height / width * w;
                            h = h < _$3.IMAGE_MIN_HEIGHT ? _$3.IMAGE_MIN_HEIGHT : h;
                        } else {
                            if (height > _h) {
                                h = _h;
                            } else if (h < _$3.IMAGE_MAX_HEIGHT) {
                                h = _$3.IMAGE_MIN_HEIGHT;
                            }
                            w = width / height * h;
                            w = w < _$3.IMAGE_MIN_WIDTH ? _$3.IMAGE_MIN_WIDTH : w;
                        }
                    } else {
                        w = width;
                        h = height;
                    }
                }
                self.viewerData.containerWidth = self.options.resize ? w : self.viewerData.stableWidth;
                self.viewerData.containerHeight = self.options.resize ? h : self.viewerData.stableHeight;

                if (self.fulled) {
                    self.$resize(self.fullBodyWidth, self.fullBodyHeight);
                } else {
                    swipWidth ? self.$resize(null, null, true) : self.$resize();
                }
                self.loading.setAttribute('class', 'hide-dom');
                // 图片加载后，初始化缩放比例
                self.$updateScaleBar(1);
                if (self.options.showScaleBar) {
                    self.viewerData.scale = 1;
                    self.viewerData.minScale = 0.2;
                    self.viewerData.maxScale = 1.8;
                } else {
                    self.viewerData.scale = self.viewerImg.clientWidth / self.viewerData.width;
                    self.viewerData.minScale = _$3.IMAGE_MIN_WIDTH / self.viewerData.width;
                    self.viewerData.maxScale = 10;
                }
            });
            this.viewerClose.setAttribute('title', this.langObject['close']);
            this.viewerFull.setAttribute('title', this.langObject['max']);
            this.viewerImg.src = this.viewerData.src;
            this.$initSwip();
            if (!swipWidth) {
                this.fulled = false;
                this.$initFullStatus();
            }
            this.$initSwipVisible();
        }
    }, {
        key: '$initSwipVisible',
        value: function $initSwipVisible() {
            if (this.options.src.length > 1) {
                this.swipLeft.style.display = '';
                this.swipRight.style.display = '';
                this.swipLeft.style.display = '';
                this.swipRight.style.display = '';
            } else {
                this.swipLeft.style.display = 'none';
                this.swipRight.style.display = 'none';
                this.swipLeft.style.display = 'none';
                this.swipRight.style.display = 'none';
            }
        }
    }, {
        key: '$initFullSize',
        value: function $initFullSize() {
            if (utils$2.isElementBody(this.options.fullContainer)) {
                this.fullWidth = document.documentElement.clientWidth;
                this.fullHeight = document.documentElement.clientHeight;
            } else {
                this.fullWidth = this.options.fullContainer.clientWidth;
                this.fullHeight = this.options.fullContainer.clientHeight;
            }
            this.fullBodyWidth = this.fullWidth - this.padding * 2;
            this.fullBodyHeight = this.fullHeight - this.padding * 2 - this.viewerHead.clientHeight;
            this.viewerPaddingWidth = this.fullBodyWidth * 0.1;
            this.viewerPaddingHeight = this.fullBodyHeight * 0.1;
        }

        /**
         * 初始化图片列表
         * @param {图片地址数组} srcArray 
         */

    }, {
        key: '$initPicList',
        value: function $initPicList(srcArray) {
            var templatePicItem = Template.picItem;
            var picListHtml = srcArray.map(function (item, index) {
                return templateReplace(templatePicItem, {
                    imgSrc: item,
                    index: index
                });
            }).join('');
            var templatePicListDom = createDom('<ul class="pop-pic-list_ul">' + picListHtml + '</ul>');

            var children = this.viewPicListOuter.children;
            for (var i = 0, n = children.length; i < n; i++) {
                children[i].remove();
            }
            this.viewPicListOuter.appendChild(templatePicListDom);
        }

        /**
         * 设置图片列表的选中态
         * @param {序号} selectCurrent 
         */

    }, {
        key: '$setPicSelect',
        value: function $setPicSelect(selectCurrent) {
            var doms = this.viewPicListOuter.querySelectorAll('.pop-pic-list_li');
            if (doms.length) {
                doms.forEach(function (item, index) {
                    if (index === selectCurrent) {
                        addClass(item, 'is-select');
                    } else {
                        removeClass(item, 'is-select');
                    }
                });
            }
        }
    }, {
        key: '$resize',
        value: function $resize(bodyWidth, bodyHeight, fixed) {
            this.$initFullSize();
            var w = void 0,
                h = void 0,
                left = void 0,
                top = void 0,
                self = this,
                _w = void 0,
                _h = void 0;
            var imgWidth = this.viewerData.width,
                imgHeight = this.viewerData.height;
            w = bodyWidth == null ? this.viewerData.containerWidth : bodyWidth;
            h = bodyHeight == null ? this.viewerData.containerHeight : bodyHeight;

            if (imgWidth > w) {
                _w = w;
                _h = imgHeight / imgWidth * w;
                imgWidth = _w;
                imgHeight = _h;
            } else if (imgHeight > h) {
                _h = h;
                _w = imgWidth / imgHeight * h;
                imgWidth = _w;
                imgHeight = _h;
            }

            left = w / 2 - imgWidth / 2;
            top = h / 2 - imgHeight / 2;
            this.viewerImg.removeAttribute('style');
            this.viewerImg.style.width = imgWidth + 'px';
            this.viewerImg.style.left = left + 'px';
            this.viewerImg.style.top = top + 'px';
            var oldHeight = this.viewerBody.clientHeight;
            this.viewerBody.style.width = w + 'px';
            this.viewerBody.style.height = h + 'px';

            left = this.fullBodyWidth / 2 - w / 2;
            top = this.fullBodyHeight / 2 - h / 2;
            //this.container.style.left = left + 'px';
            //this.container.style.top = top + 'px';

            if (fixed && oldHeight) {
                var containerTop = this.container.offsetTop - (h - oldHeight) / 2;
                this.container.style.top = containerTop + 'px';
            } else {
                this.container.style.left = left + 'px';
                this.container.style.top = top + 'px';
            }
        }
    }, {
        key: '$imageCenter',
        value: function $imageCenter(bodyWidth, bodyHeight) {
            var w = void 0,
                h = void 0,
                left = void 0,
                top = void 0;
            w = bodyWidth || this.viewerBody.clientWidth;
            h = bodyHeight || this.clientHeight.clientHeight;
            left = w / 2 - this.viewerImg.clientWidth / 2;
            top = h / 2 - this.viewerImg.clientHeight / 2;

            this.viewerImg.style.left = left + 'px';
            this.viewerImg.style.top = top + 'px';
            this.viewerBody.style.width = w + 'px';
            this.viewerBody.style.height = h + 'px';
        }
    }, {
        key: '$loadImage',
        value: function $loadImage(callback) {
            var self = this;
            var img = new Image();
            img.src = self.viewerData.src;
            if (img.complete) {
                callback && callback(img.width, img.height);
            } else {
                img.onload = function () {
                    callback && callback(img.width, img.height);
                    img.onload = null;
                };
                img.onerror = function () {
                    self.viewerData.src = self.options.defaultSrc;
                    self.viewerImg.src = self.options.defaultSrc;
                    img.src = self.options.defaultSrc;
                    img.onerror = null;
                };
            }
        }
    }, {
        key: '$handleTap',
        value: function $handleTap(event) {
            if (this.options.lock) {
                return;
            }
            event ? event.stopPropagation() : window.event.cancelBubble = true;
            event = getEvent(event);

            _$3.TOP_VIEWER && (_$3.TOP_VIEWER.container.style.zIndex = 100000);
            this.container.style.zIndex = 100001;
            _$3.TOP_VIEWER = this;
            var scale = this.viewerData.scale;
            switch (event.target) {
                case this.viewerClose:
                case this.viewCloseInClass:
                    this.close();
                    break;
                case this.viewerFull:
                case this.viewerFull2:
                    if (this.options.enableFull) {
                        this.full();
                    }
                    break;
                case this.swipLeft:
                    this.prev();
                    break;
                case this.swipRight:
                    this.next();
                    break;
                case this.scaleLessBtn:
                    scale = Math.max(0.2, Math.min(this.viewerData.scale - 0.2, 1.8));
                    this.$setScale(scale / this.viewerData.scale);
                    break;
                case this.scaleMoreBtn:
                    scale = Math.max(0.2, Math.min(this.viewerData.scale + 0.2, 1.8));
                    this.$setScale(scale / this.viewerData.scale);
                    break;
                default:
                    break;
            }
        }
    }, {
        key: '$handleCloseViewer',
        value: function $handleCloseViewer(event) {
            Logger$1.debug('handleCloseViewer');
            event = getEvent(event);
            if (event.target == this.mask && this.options.editable) {
                this.close();
            }
        }
    }, {
        key: '$handleListClick',
        value: function $handleListClick(event) {
            Logger$1.debug('handleListClick');
            event = getEvent(event);
            var index = event.target.getAttribute('data-index');
            index = index ? parseInt(index) : null;
            if (typeof index === 'number') {
                var resize = false;
                this.showByIndex(index, resize);
            }
        }
    }, {
        key: '$scaleBarDown',
        value: function $scaleBarDown(e) {
            e.preventDefault();
            e.stopPropagation();

            var rect = this.scaleRail.getBoundingClientRect();
            this.railLeft = rect.x;
            this.railWidth = rect.width;

            addListener(document.body, 'mousemove touchmove', this.$scaleBarMove);
            addListener(document.body, 'mouseup touchend', this.$scaleBarUp);
        }
    }, {
        key: '$scaleBarMove',
        value: function $scaleBarMove(e) {
            e.preventDefault();
            e.stopPropagation();
            var point = getPosOfEvent(e);
            var current = point[0].x;
            if (current < this.railLeft) {
                current = this.railLeft;
            } else if (current > this.railLeft + this.railWidth) {
                current = this.railLeft + this.railWidth;
            }
            var rate = (current - this.railLeft) / this.railWidth;
            var scaleTarget = (this.viewerData.maxScale - this.viewerData.minScale) * rate + this.viewerData.minScale;
            this.$setScale(scaleTarget / this.viewerData.scale);
        }
    }, {
        key: '$scaleBarUp',
        value: function $scaleBarUp(e) {
            e.preventDefault();
            e.stopPropagation();
            removeListener(document.body, 'mousemove touchmove', this.$scaleBarMove);
            removeListener(document.body, 'mouseup touchend', this.$scaleBarUp);
        }
    }, {
        key: '$handleDblclick',
        value: function $handleDblclick(event) {
            var node = closest(event.target, '.disable-full');
            if (!node && this.options.enableFull) {
                this.full();
            }
            // if (event.target === this.swipLeft || event.target === this.swipRight) {
            // return;
            // }
        }
    }, {
        key: '$handle',
        value: function $handle(event) {
            Logger$1.debug("pcviewer handler event:" + event.type);
            if (this.options.lock) {
                return;
            }
            event = getEvent(event);
            var originalEvent = event.originalEvent ? event.originalEvent : event;
            var point = getPosOfEvent(event);
            this.$px = point[0].x;
            this.$py = point[0].y;
            switch (event.type) {
                case 'avitap':
                    //this.$handleTap(event);
                    break;
                case 'dragstart':
                    event.preventDefault();
                    if (_$3.TOP_VIEWER) {
                        _$3.TOP_VIEWER.container.style.zIndex = 100000;
                    }
                    this.container.style.zIndex = 100001;
                    _$3.TOP_VIEWER = this;

                    if (event.target == this.viewerClose || event.target == this.viewCloseInClass || event.target == this.viewerFull || event.target == this.viewerFull2) {
                        return;
                    }
                    this.isDown = true;
                    this.cacheLastStatus = this.cacheLastStatus || {};
                    this.cacheLastStatus.px = this.$px;
                    this.cacheLastStatus.py = this.$py;
                    break;
                case 'drag':
                    event.preventDefault();
                    if (!this.isDown) {
                        return;
                    }
                    //不是最大屏幕且在边界
                    if (this.isBorder && this.options.resize && !hasClass(this.viewerFull, 'head__small')) {
                        Logger$1.debug(this.container.style.cursor);
                        var bh = void 0,
                            bw = void 0; //border height  border width
                        switch (this.isBorder) {
                            case 'n-resize':
                                bh = this.viewerBody.clientHeight - (this.$py - this.cacheLastStatus.py);
                                if (bh >= _$3.VIEWER_MIN_HEIGHT && bh <= this.fullBodyHeight) {
                                    this.viewerBody.style.height = bh + 'px';
                                    this.container.style.top = this.container.offsetTop + (this.$py - this.cacheLastStatus.py) + 'px';
                                }
                                break;
                            case 's-resize':
                                bh = this.viewerBody.clientHeight + this.$py - this.cacheLastStatus.py;
                                if (bh >= _$3.VIEWER_MIN_HEIGHT && bh <= this.fullBodyHeight) {
                                    this.viewerBody.style.height = bh + 'px';
                                }
                                break;
                            case 'w-resize':
                                bw = this.viewerBody.clientWidth - (this.$px - this.cacheLastStatus.px);
                                if (bw >= _$3.VIEWER_MIN_WIDTH && bw <= this.fullBodyWidth) {
                                    this.viewerBody.style.width = bw + 'px';
                                    this.container.style.left = this.container.offsetLeft + this.$px - this.cacheLastStatus.px + 'px';
                                }

                                break;
                            case 'e-resize':
                                bw = this.viewerBody.clientWidth + this.$px - this.cacheLastStatus.px;
                                if (bw >= _$3.VIEWER_MIN_WIDTH && bw <= this.fullBodyWidth) {
                                    this.viewerBody.style.width = bw + 'px';
                                }

                                break;
                            case 'nw-resize':
                                bh = this.viewerBody.clientHeight - (this.$py - this.cacheLastStatus.py);
                                if (bh >= _$3.VIEWER_MIN_HEIGHT && bh <= this.fullBodyHeight) {
                                    this.viewerBody.style.height = bh + 'px';
                                    this.container.style.top = this.container.offsetTop + (this.$py - this.cacheLastStatus.py) + 'px';
                                }
                                bw = this.viewerBody.clientWidth - (this.$px - this.cacheLastStatus.px);
                                if (bw >= _$3.VIEWER_MIN_WIDTH && bw <= this.fullBodyWidth) {
                                    this.viewerBody.style.width = bw + 'px';
                                    this.container.style.left = this.container.offsetLeft + this.$px - this.cacheLastStatus.px + 'px';
                                }
                                break;
                            case 'ne-resize':
                                bh = this.viewerBody.clientHeight - (this.$py - this.cacheLastStatus.py);
                                if (bh >= _$3.VIEWER_MIN_HEIGHT && bh <= this.fullBodyHeight) {
                                    this.viewerBody.style.height = bh + 'px';
                                    this.container.style.top = this.container.offsetTop + (this.$py - this.cacheLastStatus.py) + 'px';
                                }
                                bw = this.viewerBody.clientWidth + this.$px - this.cacheLastStatus.px;
                                if (bw >= _$3.VIEWER_MIN_WIDTH && bw <= this.fullBodyWidth) {
                                    this.viewerBody.style.width = bw + 'px';
                                }
                                break;
                            case 'sw-resize':
                                bw = this.viewerBody.clientWidth - (this.$px - this.cacheLastStatus.px);
                                if (bw >= _$3.VIEWER_MIN_WIDTH && bw <= this.fullBodyWidth) {
                                    this.viewerBody.style.width = bw + 'px';
                                    this.container.style.left = this.container.offsetLeft + this.$px - this.cacheLastStatus.px + 'px';
                                }
                                bh = this.viewerBody.clientHeight + this.$py - this.cacheLastStatus.py;
                                if (bh >= _$3.VIEWER_MIN_HEIGHT && bh <= this.fullBodyHeight) {
                                    this.viewerBody.style.height = bh + 'px';
                                }
                                break;
                            case 'se-resize':
                                bh = this.viewerBody.clientHeight + this.$py - this.cacheLastStatus.py;
                                if (bh >= _$3.VIEWER_MIN_HEIGHT) {
                                    this.viewerBody.style.height = bh + 'px';
                                }
                                bw = this.viewerBody.clientWidth + this.$px - this.cacheLastStatus.px;
                                if (bw >= _$3.VIEWER_MIN_WIDTH) {
                                    this.viewerBody.style.width = bw + 'px';
                                }
                                break;
                        }
                        this.$imageCenter(this.viewerBody.clientWidth, this.viewerBody.clientHeight);
                        this.cacheLastStatus.px = this.$px;
                        this.cacheLastStatus.py = this.$py;
                    }

                    if (this.isBorder) {
                        return;
                    }

                    if (event.target == this.viewerImg) {
                        var w = void 0,
                            h = void 0,
                            byW = void 0,
                            byH = void 0,
                            moveFlag = false;
                        w = this.viewerImg.clientWidth;
                        h = this.viewerImg.clientHeight;
                        byW = this.viewerBody.clientWidth;
                        byH = this.viewerBody.clientHeight;

                        if (w > byW) {
                            var moveToLeft = this.$px - this.cacheLastStatus.px + this.viewerImg.offsetLeft;
                            if (moveToLeft <= 0 && moveToLeft >= byW - w) {
                                this.viewerImg.style.left = moveToLeft + 'px';
                            }
                            moveFlag = true;
                        }
                        if (h > byH) {
                            var moveToTop = this.$py - this.cacheLastStatus.py + this.viewerImg.offsetTop;
                            if (moveToTop <= 0 && moveToTop >= byH - h) {
                                this.viewerImg.style.top = moveToTop + 'px';
                            }
                            moveFlag = true;
                        }
                        if (moveFlag) {
                            this.cacheLastStatus.px = this.$px;
                            this.cacheLastStatus.py = this.$py;
                            return;
                        }
                    }

                    if (hasClass(this.viewerFull, 'head__small')) {
                        return;
                    }

                    var docCenterX = this.fullBodyWidth / 2;
                    var docCenterY = this.fullBodyHeight / 2;

                    var containerLeft = this.$px - this.cacheLastStatus.px + this.container.offsetLeft;
                    var containerTop = this.$py - this.cacheLastStatus.py + this.container.offsetTop;
                    var centerX = containerLeft + this.container.clientWidth / 2;
                    var centerY = containerTop + this.container.clientHeight / 2;
                    if (centerX >= 0 && centerX <= this.fullBodyWidth) {
                        this.container.style.left = containerLeft + 'px';
                    } else if (Math.abs(this.$px - docCenterX) < Math.abs(this.cacheLastStatus.px - docCenterX)) {
                        this.container.style.left = containerLeft + 'px';
                    }

                    if (centerY >= 0 && centerY <= this.fullHeight) {
                        this.container.style.top = containerTop + 'px';
                    } else if (Math.abs(this.$py - docCenterY) < Math.abs(this.cacheLastStatus.py - docCenterY)) {
                        this.container.style.top = containerTop + 'px';
                    }

                    this.cacheLastStatus.px = this.$px;
                    this.cacheLastStatus.py = this.$py;
                    break;
                case 'dragend':
                    event.preventDefault();
                    this.isDown = false;
                    this.isBorder = '';
                    break;
                default:
                    break;
            }
        }
    }, {
        key: '$handleWheel',
        value: function $handleWheel(event) {
            var _this = this;

            event ? event.stopPropagation() : window.event.cancelBubble = true;
            event = getEvent(event);
            event.preventDefault();

            var target = event.target;
            var px = void 0,
                py = void 0;
            var point = getPosOfEvent(event);
            px = point[0].x;
            py = point[0].y;

            if (target == this.viewerImg || target == this.viewerBody) {
                Logger$1.debug('wheel');
                // Limit wheel speed to prevent zoom too fast
                if (this.wheeling) {
                    return;
                }

                this.wheeling = true;

                setTimeout(function () {
                    _this.wheeling = false;
                }, 50);

                var ratio = 1;
                var delta = 1;
                var scale = 1;

                if (event.deltaY) {
                    delta = event.deltaY > 0 ? 1 : -1;
                } else if (event.wheelDelta) {
                    delta = -event.wheelDelta / 120;
                } else if (event.detail) {
                    delta = event.detail > 0 ? 1 : -1;
                }
                if (this.options.showScaleBar) {
                    scale = this.viewerData.scale;
                    scale -= 0.2 * delta;
                    scale = Math.max(0.2, Math.min(scale, 1.8));
                    this.$setScale(scale / this.viewerData.scale, px, py);
                } else {
                    scale = -delta * ratio;
                    scale = Number(scale);

                    if (scale < 0) {
                        scale = 1 / (1 - scale);
                    } else {
                        scale = 1 + scale;
                    }
                    this.$setScale(scale, px, py);
                }
            }
        }
    }, {
        key: '$handleMousemove',
        value: function $handleMousemove(event) {
            event = getEvent(event);
            event.preventDefault();
            var originalEvent = event.originalEvent ? event.originalEvent : event;
            var point = getPosOfEvent(event);
            this.$px = point[0].x;
            this.$py = point[0].y;

            var isFull = this.fulled;
            var isEnableResize = this.options.resize;
            var isCtrBtn = false;
            var target = event.target;
            if (target == this.viewerClose || target == this.viewCloseInClass || target == this.viewerFull || target == this.viewerFull2) {
                isCtrBtn = true;
            }

            switch (event.type) {
                case 'touchstart':
                    if (isCtrBtn) {
                        addClass(target, 'head__oper--hover');
                    }
                case 'mousedown':
                    if (isFull || isCtrBtn || !isEnableResize) {
                        return;
                    }
                    Logger$1.debug(this.$px, this.$py);
                    this.isBorder = this.$borderCheck({
                        x: this.$px,
                        y: this.$py
                    });
                    break;
                case 'mousemove':
                    if (isFull || isCtrBtn || !isEnableResize) {
                        return;
                    }
                    this.$borderCheck({
                        x: this.$px,
                        y: this.$py
                    }, function () {
                        Logger$1.debug('done');
                    });
                    break;
                case 'mouseover':
                    if (isCtrBtn) {
                        addClass(target, 'head__oper--hover');
                    }
                    break;
                case 'mouseup':
                    this.container && (this.container.style.cursor = '');
                    break;
                case 'mouseleave':
                case 'touchend':
                    if (isCtrBtn) {
                        removeClass(target, 'head__oper--hover');
                    }
                    break;
                default:
                    break;
            }
        }
    }, {
        key: '$borderCheck',
        value: function $borderCheck(pointer, callback) {
            var px = pointer.x;
            var py = pointer.y;
            var parent = this.container;
            parent.style.cursor = '';
            var x = parent.offsetLeft;
            var y = parent.offsetTop;
            var w = parent.clientWidth;
            var h = parent.clientHeight;
            var plt = {
                x: x,
                y: y
            };
            var prt = {
                x: x + w,
                y: y
            };
            var pld = {
                x: x,
                y: y + h
            };
            var prd = {
                x: x + w,
                y: y + h
            };

            var pointers = [plt, prt, pld, prd];
            var dirs = ['nw-resize', 'ne-resize', 'sw-resize', 'se-resize'];
            var flag = true;
            for (var i = 0; i < pointers.length; i++) {
                if (Math.abs(px - pointers[i].x) < this.bound && Math.abs(py - pointers[i].y) < this.bound) {
                    parent.style.cursor = dirs[i];
                    flag = false;
                    callback && callback(px, py);
                    break;
                }
            }
            if (flag) {

                if (px > plt.x + this.bound && px < prd.x - this.bound && py > plt.y && py < plt.y + this.bound) {
                    //top
                    parent.style.cursor = 'n-resize';
                    callback && callback(px, py);
                    flag = false;
                } else if (px > plt.x + this.bound && px < prd.x - this.bound && py > prd.y - this.bound && py < prd.y) {
                    //down
                    parent.style.cursor = 's-resize';
                    callback && callback(px, py);
                    flag = false;
                } else if (px > plt.x && px < plt.x + this.bound && py > plt.y + this.bound && py < prd.y - this.bound) {
                    //left
                    parent.style.cursor = 'w-resize';
                    callback && callback(px, py);
                    flag = false;
                } else if (px > prd.x - this.bound && px < prd.x && py > plt.y + this.bound && py < prd.y - this.bound) {
                    //right
                    parent.style.cursor = 'e-resize';
                    callback && callback(px, py);
                    flag = false;
                }
            }
            return parent.style.cursor;
        }
    }, {
        key: '$initFullStatus',
        value: function $initFullStatus() {
            if (this.fulled) {
                this.viewerFull && removeClass(this.viewerFull, 'head__full');
                this.viewerFull && addClass(this.viewerFull, 'head__small');
                this.viewerFull2 && removeClass(this.viewerFull2, 'enter');
                this.viewerFull2 && addClass(this.viewerFull2, 'exit');
                addClass(this.container, 'pop__full');
            } else {
                this.viewerFull && removeClass(this.viewerFull, 'head__small');
                this.viewerFull && addClass(this.viewerFull, 'head__full');
                this.viewerFull2 && addClass(this.viewerFull2, 'enter');
                this.viewerFull2 && removeClass(this.viewerFull2, 'exit');
                removeClass(this.container, 'pop__full');
            }
        }
    }, {
        key: '$initSwip',
        value: function $initSwip() {
            if (this.currentIndex == 0) {
                addClass(this.swipLeft, 'oper--disabled');
            } else {
                removeClass(this.swipLeft, 'oper--disabled');
            }
            if (this.currentIndex == this.imgLength - 1) {
                addClass(this.swipRight, 'oper--disabled');
            } else {
                removeClass(this.swipRight, 'oper--disabled');
            }
        }

        /**
         * 设置缩放值
         * @param {*} scale 
         */

    }, {
        key: '$setScale',
        value: function $setScale(scale) {
            var px = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
            var py = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

            var viewBodyRect = this.viewerBody.getBoundingClientRect();
            px = px || viewBodyRect.width / 2 + viewBodyRect.x;
            py = py || viewBodyRect.height / 2 + viewBodyRect.y;

            var $offset = getOffset(this.viewerImg);
            var left = $offset.left;
            var top = $offset.top;
            $offset = getOffset(this.viewerImg.parentNode);
            var parentLeft = $offset.left;
            var parentTop = $offset.top;
            var tempWidth = this.viewerImg.clientWidth;
            var tempHeight = this.viewerImg.clientHeight;
            var bodyWidth = this.viewerBody.clientWidth;
            var bodyHeight = this.viewerBody.clientHeight;
            var imgHeight = this.viewerData.height;
            var imgWidth = this.viewerData.width;
            var constMinWdith = this.options.showScaleBar ? 0 : _$3.IMAGE_MIN_WIDTH;
            var constMinHeight = this.options.showScaleBar ? 0 : _$3.IMAGE_MIN_HEIGHT;
            tempWidth = tempWidth * scale;
            if (tempWidth > imgWidth * 10) {
                tempWidth = imgWidth * 10;
                scale = tempWidth / this.viewerImg.clientWidth; //重新计算放大的倍数
            } else if (imgHeight < constMinHeight || imgWidth < constMinWdith) {
                if (tempWidth < imgWidth) {
                    tempWidth = imgWidth;
                    scale = tempWidth / this.viewerImg.clientWidth;
                }
            } else if (tempWidth < constMinWdith) {
                tempWidth = constMinWdith;
                scale = tempWidth / this.viewerImg.clientWidth;
            }

            this.viewerImg.style.width = tempWidth + 'px';
            tempHeight = this.viewerImg.clientHeight;

            left = (left - px) * scale + px;
            top = (top - py) * scale + py;

            if (bodyHeight >= tempHeight || bodyWidth >= tempWidth) {
                left = bodyWidth / 2 - tempWidth / 2;
                top = bodyHeight / 2 - tempHeight / 2;

                this.viewerImg.style.left = left + 'px';
                this.viewerImg.style.top = top + 'px';
            } else {
                left = left - parentLeft;
                top = top - parentTop;

                left = left > 0 ? 0 : left;
                left = left < bodyWidth - tempWidth ? bodyWidth - tempWidth : left;

                top = top > 0 ? 0 : top;
                top = top < bodyHeight - tempHeight ? bodyHeight - tempHeight : top;

                this.viewerImg.style.left = left + 'px';
                this.viewerImg.style.top = top + 'px';
            }

            // 更新底部缩放展示条
            this.viewerData.scale *= scale;
            this.$updateScaleBar(this.viewerData.scale);
        }

        /**
         * 更新缩放至演示
         */

    }, {
        key: '$updateScaleBar',
        value: function $updateScaleBar(scale) {
            var offset = scale - this.viewerData.minScale;
            var styleLeft = 0;
            var arrange = this.viewerData.maxScale - this.viewerData.minScale;

            styleLeft = Math.abs(offset / arrange) * 100;
            this.scaleBar.style.left = styleLeft + '%';
        }

        /**
         * 更新图片描述信息
         * @param {*} index 
         */

    }, {
        key: '$updateInfo',
        value: function $updateInfo(index) {
            var info = this.options.imgInfo ? this.options.imgInfo[index] || {} : {};
            var sizeText = '';
            info.size = info.size > 0 ? info.size : 0;
            if (info.size > 1024 * 1024) {
                sizeText = Math.round(info.size / 1024 / 1024 * 100) / 100 + 'M';
            } else {
                sizeText = Math.round(info.size / 1024 * 100) / 100 + 'kb';
            }
            this.viewTitle.innerHTML = info.name ? info.name : '';
            this.textDescription.innerHTML = info.size ? this.langObject['fileSize'] + '：' + sizeText : '';
            this.textCountCurrent.innerHTML = index + 1;
            this.textCountToal.innerHTML = this.imgLength;
        }
    }, {
        key: 'open',
        value: function open() {
            var index = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;

            if (this.container) {
                this.fulled = false;
                this.$initFullStatus();
                this.container.style.display = '';
                if (this.options.modal) {
                    this.mask && removeClass(this.mask, 'hide-dom');
                }
                this.currentIndex = typeof index === 'number' && index >= 0 ? index : 0;

                var resize = true;
                this.showByIndex(this.currentIndex, resize);
                this.$resize();
            }
        }
    }, {
        key: 'close',
        value: function close() {
            this.container.style.display = 'none';
            if (this.options.modal) {
                addClass(this.mask, 'hide-dom');
            }
        }
    }, {
        key: 'full',
        value: function full() {
            if (!this.fulled) {
                this.fulled = true;
                this.cacheLastStatus = this.cacheLastStatus || {};
                this.cacheLastStatus.full = {
                    containerLeft: this.container.offsetLeft,
                    containerTop: this.container.offsetTop,
                    bodyWidth: this.viewerBody.clientWidth,
                    bodyHeight: this.viewerBody.clientHeight,
                    imgWidth: this.viewerImg.clientWidth,
                    imgLeft: this.viewerImg.offsetLeft,
                    imgTop: this.viewerImg.offsetTop,
                    scale: this.viewerData.scale
                };
                this.viewerData.scale = 1;
                this.viewerFull.setAttribute('title', this.langObject['min']);
                this.$resize(this.fullBodyWidth, this.fullBodyHeight);
            } else {
                this.fulled = false;
                this.viewerFull.setAttribute('title', this.langObject['max']);
                if (this.cacheLastStatus.full) {
                    this.container.style.left = this.cacheLastStatus.full.containerLeft + 'px';
                    this.container.style.top = this.cacheLastStatus.full.containerTop + 'px';
                    this.viewerBody.style.width = this.cacheLastStatus.full.bodyWidth + 'px';
                    this.viewerBody.style.height = this.cacheLastStatus.full.bodyHeight + 'px';
                    this.viewerImg.style.width = this.cacheLastStatus.full.imgWidth + 'px';
                    this.viewerImg.style.left = this.cacheLastStatus.full.imgLeft + 'px';
                    this.viewerImg.style.top = this.cacheLastStatus.full.imgTop + 'px';
                    this.viewerData.scale = this.cacheLastStatus.full.scale;
                } else {
                    this.$init();
                }
            }
            this.$initFullStatus();
            // 图片加载后，初始化缩放比例
            this.$updateScaleBar(1);
            if (this.options.showScaleBar) {
                this.viewerData.minScale = 0.2;
                this.viewerData.maxScale = 1.8;
            } else {
                this.viewerData.scale = this.viewerImg.clientWidth / this.viewerData.width;
                this.viewerData.minScale = _$3.IMAGE_MIN_WIDTH / this.viewerData.width;
                this.viewerData.maxScale = 10;
            }
            this.$updateScaleBar(this.viewerData.scale);
        }
    }, {
        key: 'prev',
        value: function prev() {
            this.currentIndex--;
            if (this.currentIndex >= 0) {
                this.viewerData.src = this.options.src[this.currentIndex];
                this.$init(this.viewerBody.clientWidth);
                //切换图片后重置
                this.cacheLastStatus && this.cacheLastStatus.full ? this.cacheLastStatus.full = null : null;
            } else {
                this.currentIndex = 0;
                addClass(this.swipLeft, 'oper--disabled');
            }
            this.$updateInfo(this.currentIndex);
            this.$setPicSelect(this.currentIndex);
        }
    }, {
        key: 'next',
        value: function next() {
            this.currentIndex++;
            if (this.currentIndex < this.imgLength) {
                this.viewerData.src = this.options.src[this.currentIndex];
                this.$init(this.viewerBody.clientWidth);
                //切换图片后重置
                this.cacheLastStatus && this.cacheLastStatus.full ? this.cacheLastStatus.full = null : null;
            } else {
                addClass(this.swipRight, 'oper--disabled');
                this.currentIndex = this.imgLength - 1;
            }
            this.$updateInfo(this.currentIndex);
            this.$setPicSelect(this.currentIndex);
        }
    }, {
        key: 'showByIndex',
        value: function showByIndex(current, resize) {
            if (typeof current !== 'number' || current < 0) return;
            this.currentIndex = current;
            if (this.currentIndex < this.imgLength) {
                this.viewerData.src = this.options.src[this.currentIndex];
                // 通过open 调用，resize 我为true，通过prev next 调用，resize = false
                if (resize) {
                    this.$init();
                } else {
                    this.$init(this.viewerBody.clientWidth);
                }
                //切换图片后重置
                this.cacheLastStatus && this.cacheLastStatus.full ? this.cacheLastStatus.full = null : null;
            } else {
                addClass(this.swipRight, 'oper--disabled');
                this.currentIndex = this.imgLength - 1;
            }
            this.$updateInfo(this.currentIndex);
            this.$setPicSelect(this.currentIndex);
        }

        /**
         * 外部可以直接更新
         */

    }, {
        key: 'updateSrc',
        value: function updateSrc() {
            var arg = arguments;
            if (arg.length == 0) {
                return;
            }
            if (arg.length == 1) {
                var arg1 = arg[0];
                if (isString(arg1)) {
                    // this.isSwip = false;
                    this.options.src[0] = arg1;
                    this.viewerData.src = arg1;
                    this.$init();
                } else if (isNumber(arg1) && arg1 >= 0 && arg1 < this.imgLength) {
                    this.viewerData.src = this.options.src[arg1];
                    this.currentIndex = arg1;
                    this.$init();
                } else if (isArray(arg1) && arg1.length > 0) {
                    // this.isSwip = true;
                    this.options.src = arg1;
                    this.viewerData.src = this.options.src[0];
                    this.currentIndex = 0;
                    this.imgLength = arg1.length;
                    this.$init();
                }
            } else if (arg.length >= 2) {
                var index = arg[0];
                var srcArray = arg[1];
                if (isNumber(index) && isArray(srcArray) && index >= 0 && index < srcArray.length) {
                    // this.isSwip = true;
                    this.options.src = srcArray;
                    this.viewerData.src = this.options.src[index];
                    this.imgLength = srcArray.length;
                    this.currentIndex = index;
                    this.$init();
                }
            }
            this.$initPicList(this.options.src);
        }
    }, {
        key: 'updateImageInfo',
        value: function updateImageInfo(infoList) {
            if (isArray(infoList) && infoList.length > 0) {
                this.options.imgInfo = infoList;
            }
        }
    }, {
        key: 'onResize',
        value: function onResize() {
            this.$initFullSize();
            this.$imageCenter(this.viewerBody.clientWidth, this.viewerBody.clientHeight);
        }
    }, {
        key: 'setLock',
        value: function setLock(lock) {
            this.options.lock = lock;
        }
    }, {
        key: 'destroy',
        value: function destroy() {
            var self = this;
            //事件解绑
            Events$2.unregisterEvent(this, 'dragstart drag dragend pinchstart pinch pinchend');
            this.container && removeListener(this.container, 'mousedown mousemove', self.$handleMousemove);
            this.viewerClose && removeListener(this.viewerClose, 'mouseover mouseleave', self.$handleMousemove);
            this.viewCloseInClass && removeListener(this.viewCloseInClass, 'mouseover mouseleave', self.$handleMousemove);
            this.viewerFull && removeListener(this.viewerFull, ' mouseover mouseleave', self.$handleMousemove);
            this.viewerFull2 && removeListener(this.viewerFull2, ' mouseover mouseleave', self.$handleMousemove);
            this.container && addListener(this.container, 'click', self.$handleTap);
            // dom元素移除
            this.container && this.container.parentNode.removeChild(this.container);
            this.mask && this.mask.parentNode && this.mask.parentNode.removeChild(this.mask);
            // 数据清空
            this.mask = null;
            this.container = null;
            this.viewerImg = null;
            this.viewerClose = null;
            this.viewCloseInClass = null;
            this.viewerFull = null;
            this.viewerFull2 = null;
            this.viewerHead = null;
            this.viewerBody = null;
            this.swipLeft = null;
            this.swipRight = null;
            this.loading = null;
            this.options = null;
            this.$handle = null;
            this.$handleMousemove = null;
            if (_$3.TOP_VIEWER && _$3.TOP_VIEWER == this) {
                _$3.TOP_VIEWER = null;
            }
            this.viewerData = null;
        }
    }]);
    return PcViewer;
}();

var VIEWER_TYPE = {
    MOBILE: 'mobile',
    PC: 'pc'
    /**
     * viewer  弹窗构造工厂
     */
};var Factory = {
    createViewer: function createViewer(options) {
        var env = options.env;
        switch (env) {
            case VIEWER_TYPE.MOBILE:
                return new MobileViewer(options);
                break;
            case VIEWER_TYPE.PC:
                return new PcViewer(options);;
                break;
            default:
                break;
        }
        return _.VIEWER;
    }
};

var ImageView = function () {
    function ImageView(container, options) {
        classCallCheck(this, ImageView);

        //兼容只传入options的方式
        if (!container && !options) {
            throw new Error("至少需要一个参数");
            return;
        } else if (!options && container) {
            options = container;
            container = null;
        } else if (options && container) {
            var reg = container.nodeName && 'IMGimg'.match(container.nodeName);
            if (!(reg && reg.length > 0)) {
                throw new Error("图片参数有误");
                return;
            }
        }
        this.container = container;
        options.width = options.newwidth > 0 ? options.newwidth : 0;
        options.height = options.newHeight > 0 ? options.newHeight : 0;
        options.userSet = this.container && this.container.getAttribute('data-userset') === "true";
        this.options = extend({}, DEFAULT, options);

        if (isMobile()) {
            this.env = 'mobile';
        } else {
            this.env = 'pc';
        }
        this.env = this.options.env || this.env;
        this.options.env = this.env;
        var src = '';

        if (this.options.src && typeOf(this.options.src) == 'string') {
            src = this.options.src;
        } else if (this.container) {
            src = this.container && this.container.getAttribute('src');
            this.options.src = src;
        }

        !this.options.src && Logger$1.debug('没有图片地址');
        this.imgData = {
            src: src,
            width: this.options.width,
            height: this.options.height,
            userSet: this.options.userSet
        };
        this.viewer = null;
    }

    createClass(ImageView, [{
        key: 'render',
        value: function render() {
            var reg = this.container && this.container.nodeName && 'IMGimg'.match(this.container.nodeName);
            if (reg && reg.length > 0) {
                this.container.removeAttribute('style');
                this.container.removeAttribute('class');
                this.container.removeAttribute('width');
                this.container.removeAttribute('height');
                this.container.style.display = 'none';
            }
            return this.$init();
        }
    }, {
        key: '$init',
        value: function $init() {
            var _this = this;

            var self = this;
            self.$createHtml();
            return new Promise(function (resolve /* , reject */) {
                _this.$loadImage(function () {

                    //如果图片太小
                    if (self.imgData.width < _$3.IMAGE_MIN_WIDTH && self.imgData.height < _$3.IMAGE_MIN_HEIGHT) {
                        self.container && (self.container.style.display = '');
                        setStyle(self.container, {
                            'border': '0'
                        });
                        setStyle(self.container, {
                            'width': self.imgData.width,
                            'height': self.imgData.height
                        });
                        _this.img.removeAttribute('style');

                        self.initMaxStyle();
                        addClass(self.scale, 'hide-dom');
                        addClass(self.loading, 'hide-dom');
                        self.$removeLister();
                        return;
                    }
                    self.initMaxStyle();
                    self.$renderHtml();
                    addClass(_this.loading, 'hide-dom');
                    resolve();
                    Logger$1.debug('loaded img');
                }, function () {
                    resolve();
                });
            });
        }
    }, {
        key: '$loadImage',
        value: function $loadImage(success, error) {
            var self = this;
            var img = new Image();
            img.src = self.imgData.src;
            if (img.complete && self.imgData) {
                self.imgData.width = self.imgData.width !== 0 ? self.imgData.width : img.width;
                self.imgData.height = self.imgData.height !== 0 ? self.imgData.height : img.height;
                if (success) {
                    success();
                }
            } else {
                img.onload = function () {
                    if (self.imgData) {
                        // self.imgData.width = img.width;
                        // self.imgData.height = img.height;
                        self.imgData.width = self.imgData.width !== 0 ? self.imgData.width : img.width;
                        self.imgData.height = self.imgData.height !== 0 ? self.imgData.height : img.height;
                        success ? success() : null;
                    }
                    img.onload = null;
                };
                img.onerror = function () {
                    if (self.imgData) {
                        self.img.src = self.options.defaultSrc;
                        self.options.defaultSrc && (img.src = self.options.defaultSrc);
                        self.imgData.src = self.options.defaultSrc;
                    }
                    error();
                    img.onerror = null;
                };
            }
        }
    }, {
        key: '$createHtml',
        value: function $createHtml() {
            var templateView = void 0,
                wrapView = void 0;
            templateView = this.options.inline ? Template.view2 : Template.view;
            templateView = createDom(templateView);
            wrapView = templateView.firstElementChild;
            if (_typeof(this.options.zoomIcon) !== (typeof undefined === 'undefined' ? 'undefined' : _typeof(undefined))) {
                if (this.options.zoomIcon == false) {
                    addClass(templateView.querySelector('.insert-pictures__zoom'), 'hide-dom');
                }
            } else if (this.env == 'mobile' && this.options.eventAll) {
                addClass(templateView.querySelector('.insert-pictures__zoom'), 'hide-dom');
            }

            if (this.container) {
                this.img = this.container;
                // wrapView && wrapView.insertBefore(this.container, wrapView.firstElementChild);
            } else {
                var img = createDom('<img/>');
                img.src = this.imgData.src;
                this.img = img;

                //wrapView.insertBefore(img, wrapView.firstElementChild);
            }
            this.container = templateView;

            if (this.img && this.img.parentElement) {
                this.img.parentElement.insertBefore(this.container, this.img);
            }

            if (this.options.origin) {
                addClass(this.container, 'origin');
            }

            wrapView && wrapView.insertBefore(this.img, wrapView.firstElementChild);

            this.loading = this.container.querySelector('.insert-pictures__loading');
            this.scale = this.container.querySelector('.insert-pictures__zoom');
            var self = this;
            self.$handle = proxy(self.$handle, self);
            if (!this.options.eventAll) {
                this.scale && addListener(this.scale, 'click', self.$handle);

                if ('ontouchend' in window) {
                    var startX = 0,
                        startY = 0;
                    this.scale && addListener(this.scale, 'touchstart', function (startEvent) {
                        startX = startEvent.changedTouches[0].clientX;
                        startY = startEvent.changedTouches[0].clientY;
                    });
                    this.scale && addListener(this.scale, 'touchend', function (endEvent) {
                        if (Math.abs(endEvent.changedTouches[0].clientX - startX) < 4 && Math.abs(endEvent.changedTouches[0].clientY - startY) < 4) {
                            self.$handle(endEvent);
                        }
                    });
                    // $.addListener(this.scale, 'touchmove', () => {
                    //     this.scale && $.addListener(this.scale, 'touchend', self.$handle);
                    // })
                }
            } else {
                this.container && addListener(this.container, 'click', self.$handle);
            }
            this.options.style && setStyle(this.container, this.options.style);
        }
    }, {
        key: '$renderHtml',
        value: function $renderHtml() {
            this.$resize();
        }
    }, {
        key: '$resize',
        value: function $resize() {
            var w = void 0,
                h = void 0,
                _w = void 0,
                _h = void 0;
            _w = this.imgData.width;
            _h = this.imgData.height;

            if (_w < _$3.IMAGE_MIN_HEIGHT && this.imgData.userSet === false) {
                w = _$3.IMAGE_MIN_WIDTH;
                h = _h <= _$3.IMAGE_MAX_HEIGHT ? _h : _$3.IMAGE_MAX_HEIGHT;
            } else if (_h < _$3.IMAGE_MIN_WIDTH && this.imgData.userSet === false) {
                h = _$3.IMAGE_MIN_HEIGHT;
                w = _w <= _$3.IMAGE_MAX_WIDTH ? _w : _$3.IMAGE_MAX_WIDTH;
            } else if (_w > _$3.IMAGE_MAX_WIDTH || _h > _$3.IMAGE_MAX_HEIGHT) {
                w = _$3.IMAGE_MAX_WIDTH;
                h = _$3.IMAGE_MAX_HEIGHT;
                if (this.imgData.userSet === true) {
                    if (_w > _h) {
                        addClass(this.container, 'img-w100');
                    } else {
                        addClass(this.container, 'img-h100');
                    }
                }
            } else {
                w = _w;
                h = _h;
                addClass(this.container, 'img-vertical-align-top');
            }
            this.container.style.width = w + 'px';
            this.container.style.height = h + 'px';
            this.options.style && setStyle(this.container, this.options.style);
            this.img.removeAttribute('style');
            this.initMaxStyle();
            if (this.options.origin) {
                this.container.removeAttribute('style');
            }
        }
    }, {
        key: 'initMaxStyle',
        value: function initMaxStyle() {
            if (this.options.origin) {
                if (this.options.originMaxHeight) {
                    var style = this.img.getAttribute('style') || '';
                    this.img.setAttribute('style', style + ';max-height:' + this.options.originMaxHeight);
                }
                if (this.options.originMaxWidth) {
                    var _style = this.img.getAttribute('style') || '';
                    this.img.setAttribute('style', _style + ';max-width:' + this.options.originMaxWidth);
                }
            }
        }
    }, {
        key: '$handle',
        value: function $handle(event) {
            event ? (event.stopPropagation(), event.preventDefault()) : (window.event.cancelBubble = true, window.event.returnValue == false);
            Logger$1.debug('click');
            if (this.options.lock) {
                return;
            }

            if (this.viewer == null) {
                var viewrOptions = extend({}, this.options, this.imgData);
                this.viewer = Factory.createViewer(viewrOptions);
                this.viewer.container.style.zIndex = 100001;
            } else if (this.env == 'mobile') {
                this.viewer.updateSrc(this.imgData.src);
            }
            this.viewer.open();
            //发送单击图片查看事件
            if (typeof this.options.onImageOpen !== 'undefined') {
                this.options.onImageOpen({
                    type: 'image'
                });
            }
            if (_$3.TOP_VIEWER) {
                _$3.TOP_VIEWER.container && (_$3.TOP_VIEWER.container.style.zIndex = 100000);
            }
            this.viewer.container && (this.viewer.container.style.zIndex = 100001);
            _$3.TOP_VIEWER = this.viewer;
        }
    }, {
        key: 'setLock',
        value: function setLock(lock) {
            this.options.lock = lock;
            if (this.options.lock) {
                this.container && setStyle(this.container, {
                    'pointer-events': 'none'
                });
            } else {
                this.container && setStyle(this.container, {
                    'pointer-events': ''
                });
            }
        }

        //获取默认单位

    }, {
        key: 'getUnit',
        value: function getUnit(str) {
            var px = '';
            if (typeof str === 'number') {
                px = 'px';
            }
            return px;
        }
    }, {
        key: 'resize',
        value: function resize(data) {
            if (this.options.origin) {
                if (data.width) {
                    var px = this.getUnit(data.width);
                    this.img.setAttribute('style', 'max-width: ' + data.width + px + ';width:' + data.width + px);
                } else if (data.height) {
                    var originWith = this.imgData.width;
                    var originHeight = this.imgData.height;
                    var height = data.height;

                    var _px = this.getUnit(height);
                    this.img.setAttribute('style', 'max-height: ' + height + _px + ';height:' + height + _px);

                    var currentWidth = getWidth(this.img);
                    if (parseInt(data.height) / currentWidth > originHeight / originWith && currentWidth > originWith) {
                        height = originHeight / originWith * currentWidth;
                        _px = 'px';
                    }
                    this.img.setAttribute('style', 'max-height: ' + height + _px + ';height:' + height + _px);
                }
            } else {
                if (data.width) {
                    this.container.style.width = 'width:' + data.width + this.getUnit(data.width);
                }
                if (data.height) {
                    this.container.style.height = 'height:' + data.height + this.getUnit(data.height);
                }
            }
        }
    }, {
        key: 'getView',
        value: function getView() {
            return this.container;
        }
    }, {
        key: 'reset',
        value: function reset() {
            this.off();
        }
    }, {
        key: 'on',
        value: function on() {
            this.viewer && this.viewer.open();
        }
    }, {
        key: 'off',
        value: function off() {
            this.viewer && this.viewer.close();
        }
    }, {
        key: '$removeLister',
        value: function $removeLister() {
            if (!this.options.eventAll) {
                this.scale && removeListener(this.scale, 'click', this.$handle);
                if ('ontouchend' in window) {
                    this.scale && removeListener(this.scale, 'touchend', self.$handle);
                }
            } else {
                this.container && removeListener(this.container, 'click', this.$handle);
            }
        }
    }, {
        key: 'destroy',
        value: function destroy() {
            Logger$1.debug('销毁imageveiewer');
            this.$removeLister();
            this.viewer && this.viewer.destroy();
            this.container = null;
            this.viewer = null;
            this.loading = null;
            this.scale = null;
            this.options = null;
            this.imgData = null;
        }
    }]);
    return ImageView;
}();

var ImagePlayer = {
    setLoggerLevel: function setLoggerLevel(level) {
        Logger$1.setLevel(level);
    },
    render: function render(container, options) {

        container = container.length > 0 ? container[0] : container; //添加对jq的适配

        for (var key in _$3.IMAGE_INSTANCES) {
            if (!document.querySelector('[data-id="' + _$3.getKeyId(key) + '"]')) {
                _$3.clear(key);
            }
        }
        _$3.TOP_VIEWER = null; //清空当前顶层数据
        _$3.VIEWER = null; //清空工厂

        options = options || {};
        Events$2.init();
        Events$2.unbind();
        Events$2.bind();
        if (container) {
            var imageItems = container.nodeName.toLowerCase() == 'img' ? [container] : container.getElementsByTagName('img');

            each(imageItems, function (item, index) {

                if (item.parentNode && hasClass(item.parentNode, 'ndui-insert-pictures') || item.parentNode && hasClass(item.parentNode, 'pop-insert-pictures__body')) {
                    return;
                }

                var dataId = getRandomId('media');

                var mediaId = 'image';
                var containerId = options.containerId || '';
                mediaId = mediaId + '-' + containerId + '-' + index;

                var key = mediaId + '|' + dataId;
                var newOptions = extend({}, options, {
                    dataId: dataId,
                    mediaId: mediaId,
                    key: key
                });

                var image = new ImageView(item, newOptions);
                image.render();
                _$3.IMAGE_INSTANCES[key] = image;
            });
        } else {
            Logger$1.debug('no container');
        }
        return _$3.IMAGE_INSTANCES;
    },

    destroy: function destroy() {
        Events$2.unbind();
        var instances = _$3.IMAGE_INSTANCES;
        for (var i = 0, len = instances.length; i < len; i++) {
            instances.destroy();
        }
        _$3.IMAGE_INSTANCES = {};
        _$3.TOP_VIEWER = null;
        _$3.VIEWER = null; //清空工厂
    },
    reset: function reset() {
        var instances = _$3.IMAGE_INSTANCES;
        for (var i = 0, len = instances.length; i < len; i++) {
            instances.off();
        }
    },
    setLock: function setLock(lock, view) {
        var instances = _$3.IMAGE_INSTANCES;
        for (var key in instances) {
            instances[key].setLock(lock);
        }
    },
    getMedia: function getMedia() {
        return _$3.IMAGE_INSTANCES;
    },
    mediaOnFullScreenChange: function mediaOnFullScreenChange() {
        //no use
    },
    Image: {
        create: function create(img, options) {
            Events$2.init();
            Events$2.unbind();
            Events$2.bind();
            return new ImageView(img, options);
        }
    },
    Viewer: {
        create: function create(options) {
            var env = '';
            if (isMobile()) {
                env = 'mobile';
            } else {
                env = 'pc';
            }
            options.env = options.env || env;
            Events$2.init();
            Events$2.unbind();
            Events$2.bind();
            var instance = Factory.createViewer(options);
            if (_$3.TOP_VIEWER) {
                _$3.TOP_VIEWER.container && (_$3.TOP_VIEWER.container.style.zIndex = 100000);
            }
            instance.container && (instance.container.style.zIndex = 100001);
            _$3.TOP_VIEWER = instance;
            return instance;
        }
    }
};

var methods$1 = ['assets', 'clear', 'count', 'debug', 'dir', 'dirxml', 'error', 'exception', 'group', 'groupCollapsed', 'groupEnd', 'info', 'log', 'markTimeline', 'profile', 'profileEnd', 'table', 'time', 'timeEnd', 'timeline', 'timelineEnd', 'timeStamp', 'trace', 'warn'];

var LEVEL$1 = {
    DEBUG: 'debug',
    INFO: 'info',
    WARN: 'warn',
    ERROR: 'error',
    DEBUG_VALUE: 3,
    INFO_VALUE: 2,
    WARN_VALUE: 1,
    ERROR_VALUE: 0,
    UNKNOWN: -1
};

var console$2 = window.console || {};

var length$1 = methods$1.length;

while (length$1--) {
    var method$1 = methods$1[length$1];
    if (!console$2[method$1]) {
        console$2[method$1] = function () {};
    }
}

var Logger$2 = {

    $level: 0,
    setLevel: function setLevel(level) {
        switch (level) {
            case LEVEL$1.DEBUG:
                this.$level = LEVEL$1.DEBUG_VALUE;
                break;
            case LEVEL$1.INFO:
                this.$level = LEVEL$1.INFO_VALUE;
                break;
            case LEVEL$1.WARN:
                this.$level = LEVEL$1.WARN_VALUE;
                break;
            case LEVEL$1.ERROR:
                this.$level = LEVEL$1.ERROR_VALUEl;
                break;
            default:
                this.$level = LEVEL$1.UNKNOWN;
        }
    },
    debug: function debug() {
        if (this.$level >= LEVEL$1.DEBUG_VALUE) {
            console$2.log.apply(console$2, arguments);
        }
    },
    info: function info() {
        if (this.$level >= LEVEL$1.INFO_VALUE) {
            console$2.log.apply(console$2, arguments);
        }
    },
    warn: function warn() {
        if (this.$level >= LEVEL$1.WARN_VALUE) {
            console$2.log.apply(console$2, arguments);
        }
    },
    error: function error() {
        if (this.$level >= LEVEL$1.ERROR_VALUE) {
            console$2.log.apply(console$2, arguments);
        }
    }
};

var DEFAULT$2 = {
    env: '',
    href: '',
    lock: false
};

var REGEXP_SPACES$1 = /\s+/;
var REGEXP_TRIM$1 = /^\s+(.*)\s+$/;

// Utilities
var objectProto$1 = Object.prototype;
var toString$1 = objectProto$1.toString;
function typeOf$1(obj) {
  return toString$1.call(obj).slice(8, -1).toLowerCase();
}

function isString$1(str) {
  return typeof str === 'string';
}

function isNumber$1(num) {
  return typeof num === 'number' && !isNaN(num);
}



function isObject$1(obj) {
  return (typeof obj === 'undefined' ? 'undefined' : _typeof(obj)) === 'object' && obj !== null;
}



function isFunction$1(fn) {
  return typeOf$1(fn) === 'function';
}

function isArray$1(arr) {
  return Array.isArray ? Array.isArray(arr) : typeOf$1(arr) === 'array';
}





function trim$1(str) {
  if (isString$1(str)) {
    str = str.trim ? str.trim() : str.replace(REGEXP_TRIM$1, '1');
  }

  return str;
}

function each$1(obj, callback) {
  if (obj && isFunction$1(callback)) {
    var i = void 0;

    if (isArray$1(obj) || isNumber$1(obj.length) /* array-like */) {
        var length = obj.length;

        for (i = 0; i < length; i++) {
          if (callback.call(obj, obj[i], i, obj) === false) {
            break;
          }
        }
      } else if (isObject$1(obj)) {
      Object.keys(obj).forEach(function (key) {
        callback.call(obj, obj[key], key, obj);
      });
    }
  }

  return obj;
}

function extend$1(obj) {
  for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  if (isObject$1(obj) && args.length > 0) {
    if (Object.assign) {
      return Object.assign.apply(Object, [obj].concat(args));
    }

    args.forEach(function (arg) {
      if (isObject$1(arg)) {
        Object.keys(arg).forEach(function (key) {
          obj[key] = arg[key];
        });
      }
    });
  }

  return obj;
}

function proxy$1(fn, context) {
  for (var _len2 = arguments.length, args = Array(_len2 > 2 ? _len2 - 2 : 0), _key2 = 2; _key2 < _len2; _key2++) {
    args[_key2 - 2] = arguments[_key2];
  }

  return function () {
    for (var _len3 = arguments.length, args2 = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      args2[_key3] = arguments[_key3];
    }

    return fn.apply(context, args.concat(args2));
  };
}







function addClass$1(element, value) {
  if (!value) {
    return;
  }

  if (isNumber$1(element.length)) {
    each$1(element, function (elem) {
      addClass$1(elem, value);
    });
    return;
  }

  if (element.classList) {
    element.classList.add(value);
    return;
  }

  var className = trim$1(element.className);

  if (!className) {
    element.className = value;
  } else if (className.indexOf(value) < 0) {
    element.className = className + ' ' + value;
  }
}

function removeClass$1(element, value) {
  if (!value) {
    return;
  }

  if (isNumber$1(element.length)) {
    each$1(element, function (elem) {
      removeClass$1(elem, value);
    });
    return;
  }

  if (element.classList) {
    element.classList.remove(value);
    return;
  }

  if (element.className.indexOf(value) >= 0) {
    element.className = element.className.replace(value, '');
  }
}











function removeListener$1(element, type, handler) {
  var types = trim$1(type).split(REGEXP_SPACES$1);

  if (types.length > 1) {
    each$1(types, function (t) {
      removeListener$1(element, t, handler);
    });
    return;
  }

  if (element.removeEventListener) {
    element.removeEventListener(type, handler, false);
  } else if (element.detachEvent) {
    element.detachEvent('on' + type, handler);
  }
}

function addListener$1(element, type, _handler, once) {
  var types = trim$1(type).split(REGEXP_SPACES$1);
  var originalHandler = _handler;

  if (types.length > 1) {
    each$1(types, function (t) {
      addListener$1(element, t, _handler);
    });
    return;
  }

  if (once) {
    _handler = function handler() {
      for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
        args[_key4] = arguments[_key4];
      }

      removeListener$1(element, type, _handler);

      return originalHandler.apply(element, args);
    };
  }

  if (element.addEventListener) {
    element.addEventListener(type, _handler, false);
  } else if (element.attachEvent) {
    element.attachEvent('on' + type, _handler);
  }
}





















// Force reflow to enable CSS3 transition


// e.g.: http://domain.com/path/to/picture.jpg?size=1280×960 -> picture.jpg












function createDom$1(str) {
  if (typeOf$1(str) === 'string') {
    var dom = document && document.createElement('div');
    dom.innerHTML = str;
    return dom.firstChild;
  }
  return str;
}







function isIOS() {
  return (/(ipad|ios|iphone|mac os)/i.test(navigator.userAgent)
  );
}

function isPPT() {
  return window.native && typeof window.native.invoke === 'function';
}
function isSmartClass() {
  function getUrlParam(url, name) {
    var pattern = new RegExp("[?&]" + name + "\=([^&#\/]+)", "g");
    var matcher = pattern.exec(url);
    var items = null;
    if (null != matcher) {
      try {
        items = decodeURIComponent(decodeURIComponent(matcher[1]));
      } catch (e) {
        try {
          items = decodeURIComponent(matcher[1]);
        } catch (e) {
          items = matcher[1];
        }
      }
    }
    return items;
  }
  return getUrlParam(location.href, "terminal") === "student" && window.Bridge && Bridge.require("IcrJsBridge");
}



function getRandomId$1(name) {
  return name + (Math.random() * 10 + new Date() * 1).toString().replace('.', '');
}

var _$4 = {
    LINK_INSTANCES: {},
    clear: function clear(key) {
        var linkview = this.LINK_INSTANCES[key];
        if (linkview) {
            linkview.destroy();
        }
        this.LINK_INSTANCES[key] = null;
        delete this.LINK_INSTANCES[key];
    },
    getKeyId: function getKeyId(key) {
        return key.split('|')[1];
    }
};

var DEFAULT$3 = {
    src: '', //传入地址，数组或字符串
    currentIndex: 0, //当前位置的索引
    lock: false, //是否锁定，事件无法执行
    lang: 'zh' //语言
};

var Template$1 = {
    popfullh5: '<div class="ndui-pop-link">' + '<div class="ndui-pop-link__head">' + '<div class="ndui-pop-link__close"><svg class="icon" style="width: 1em; height: 1em;vertical-align: middle;fill: currentColor;overflow: hidden;" viewBox="0 0 1024 1024" version="1.1" xmlns="http://www.w3.org/2000/svg" p-id="4087"><path d="M564.228924 510.419846l448.11096-448.10584c14.259147-14.248907 14.259147-37.36562 0-51.619646-14.259147-14.259147-37.3605-14.259147-51.614526 0L512.604158 458.805319 64.477838 10.69436c-14.259147-14.259147-37.37586-14.259147-51.619646 0-14.259147 14.248907-14.259147 37.36562 0 51.619646l448.12632 448.10584L12.868432 958.530806c-14.259147 14.254027-14.259147 37.3605 0 51.619646 7.132133 7.127013 16.470978 10.69052 25.809823 10.69052 9.343965 0 18.68793-3.563507 25.809823-10.69052l448.11608-448.10584 448.12632 448.10584c7.132133 7.127013 16.470978 10.69052 25.809823 10.69052 9.333725 0 18.67257-3.563507 25.804703-10.69052 14.259147-14.259147 14.259147-37.36562 0-51.619646L564.228924 510.419846z" p-id="4088"></path></svg></div>' + '<div class="ndui-pop-link__title"></div>' + ' </div>' + '<div class="ndui-pop-link__body">' + '<iframe class="ndui-pop-link__iframe" src="" sandbox="allow-scripts allow-same-origin">' + '</iframe>' + '</div>' + '</div>'
};

var Frame = function () {
    function Frame(options) {
        classCallCheck(this, Frame);

        if (options.singleton && Frame.instance) {
            return Frame.instance;
        }
        this.container = null;
        this.urlViewer = null;
        this.titleViewer = null;
        Frame.instance = this;
    }

    createClass(Frame, [{
        key: 'destroy',
        value: function destroy() {
            if (this.container) {
                this.container.parentNode.removeChild(this.container);
            }
        }
    }, {
        key: 'open',
        value: function open() {
            if (!this.container) {
                this.init();
            }
            if (this.container) {
                addClass$1(document.body, 'ndui-hide-overflow');
                this.container.style.display = 'block';
            }
        }
    }, {
        key: 'close',
        value: function close() {
            if (this.container) {
                removeClass$1(document.body, 'ndui-hide-overflow');
                this.container.style.display = 'none';
                this.clearIframe(this.urlViewer);
            }
        }
    }, {
        key: 'setUrl',
        value: function setUrl(url) {
            if (!this.container) {
                this.init();
            }
            if (this.container) {
                if (url !== this.urlViewer.src) {
                    this.resetIframe();
                }
                this.urlViewer.src = url;
                this.titleViewer.innerHTML = url;
            }
        }
    }, {
        key: 'init',
        value: function init() {
            this.initView();
            this.initEvent();
        }
    }, {
        key: 'initView',
        value: function initView() {
            var templateDom = Template$1.popfullh5;
            templateDom = createDom$1(templateDom);
            this.urlViewer = templateDom.querySelector('.ndui-pop-link__iframe');
            this.titleViewer = templateDom.querySelector('.ndui-pop-link__title');
            if (isIOS()) {
                this.urlViewer.setAttribute('scrolling', 'no');
                addClass$1(templateDom.querySelector('.ndui-pop-link__body'), 'ndui-pop-link__body--ios');
            }
            this.container = templateDom;
            document.body.appendChild(templateDom);
        }
    }, {
        key: 'initEvent',
        value: function initEvent() {
            var _this = this;

            addListener$1(this.container.querySelector('.ndui-pop-link__close'), 'click', function () {
                return _this.close();
            });
        }
    }, {
        key: 'resetIframe',
        value: function resetIframe() {
            var el = this.urlViewer.cloneNode();
            var parent = this.urlViewer.parentNode;
            el.src = '';
            this.clearIframe(this.urlViewer);
            parent.removeChild(this.urlViewer);
            parent.appendChild(el);
            this.urlViewer = el;
        }
    }, {
        key: 'clearIframe',
        value: function clearIframe(iframeEl) {
            // ??iframe ??
            if (iframeEl) {
                var iframeObj = iframeEl.contentWindow;
                iframeEl.src = 'about:blank';
                try {
                    iframeObj.document.write('');
                    iframeObj.document.clear();
                } catch (e) {}
            }
        }
    }]);
    return Frame;
}();

var SmartClassViewer = function () {
    function SmartClassViewer(options) {
        classCallCheck(this, SmartClassViewer);

        this.options = extend$1({}, DEFAULT$3, options);
        this.fullScreenChange = this.options.onFullScreenChange || this.options.eventHandler && this.options.eventHandler.fullScreenChange;
        this.frame = new Frame({ singleton: true });
    }

    createClass(SmartClassViewer, [{
        key: 'updateSrc',
        value: function updateSrc(url) {
            if (isString$1(url)) {
                this.frame.setUrl(url);
            }
        }
    }, {
        key: 'open',
        value: function open() {
            this.frame.open();
            if (this.fullScreenChange) {
                this.fullScreenChange(true);
            }
        }
    }, {
        key: 'close',
        value: function close() {
            this.frame.close();
            if (this.fullScreenChange) {
                this.fullScreenChange(false);
            }
        }
    }, {
        key: 'destroy',
        value: function destroy() {
            Logger.debug('销毁smartclassviewer');
            this.frame.destory();
        }
    }]);
    return SmartClassViewer;
}();

var PcViewer$2 = function () {
    function PcViewer(options) {
        classCallCheck(this, PcViewer);

        this.options = extend$1({}, DEFAULT$3, options);
    }

    createClass(PcViewer, [{
        key: 'open',
        value: function open() {
            window.open(this.options.src);
        }
    }, {
        key: 'close',
        value: function close() {}
    }, {
        key: 'destroy',
        value: function destroy() {
            this.options = null;
        }
    }]);
    return PcViewer;
}();

var PPTViewer = function () {
    function PPTViewer(options) {
        classCallCheck(this, PPTViewer);

        this.options = extend$1({}, DEFAULT$3, options);
    }

    createClass(PPTViewer, [{
        key: 'open',
        value: function open() {
            window.native.invoke("openUrl", { "url": this.options.src, "type": "default_browser" });
        }
    }, {
        key: 'close',
        value: function close() {}
    }, {
        key: 'destroy',
        value: function destroy() {
            this.options = null;
        }
    }]);
    return PPTViewer;
}();

var BridgeProxy = function () {
    if (window.Bridge && window.Bridge.require) {
        var linkmanager = Bridge.require("nd.exercises.linkmanager");
        if (linkmanager) {
            linkmanager = linkmanager.promise();
            return {
                enable: true,
                openUrl: function openUrl(url) {
                    linkmanager.open(url);
                }
            };
        } else {
            var appfactory = Bridge.require("sdp.appfactory");
            appfactory = appfactory && appfactory.promise();
            if (appfactory && typeof appfactory.openWithBrowser === 'function') {
                return {
                    enable: true,
                    openUrl: function openUrl(url) {
                        appfactory.openWithBrowser(url);
                    }
                };
            }
        }
    }
    return {
        enable: false,
        openUrl: function openUrl() {}
    };
}();

var AppFactoryViewer = function () {
    function AppFactoryViewer(options) {
        classCallCheck(this, AppFactoryViewer);

        this.options = extend$1({}, DEFAULT$3, options);
    }

    createClass(AppFactoryViewer, [{
        key: 'open',
        value: function open() {
            BridgeProxy.openUrl({ "url": this.options.src });
        }
    }, {
        key: 'close',
        value: function close() {}
    }, {
        key: 'destroy',
        value: function destroy() {
            this.options = null;
        }
    }], [{
        key: 'support',
        value: function support() {
            return BridgeProxy.enable;
        }
    }]);
    return AppFactoryViewer;
}();

var VIEWER_TYPE$1 = {
    SMARTCLASS: 'smartClass',
    APPFACTORY: 'appFactory',
    PC: 'pc',
    PPT: 'ppt'
    /**
     * viewer  弹窗构造工厂
     */
};var Factory$1 = {
    createViewer: function createViewer(options) {
        var env = options.env;
        switch (env) {
            case VIEWER_TYPE$1.SMARTCLASS:
                return new SmartClassViewer(options);
                break;
            case VIEWER_TYPE$1.APPFACTORY:
                return new AppFactoryViewer(options);
                break;
            case VIEWER_TYPE$1.PC:
                return new PcViewer$2(options);
                break;
            case VIEWER_TYPE$1.PPT:
                return new PPTViewer(options);
                break;
            default:
                return new PcViewer$2(options);
                break;
        }
        return _.VIEWER;
    }
};

var LinkView = function () {
    function LinkView(container, options) {
        classCallCheck(this, LinkView);

        //兼容只传入options的方式
        if (!container && !options) {
            throw new Error("至少需要一个参数");
        } else if (!options && container) {
            options = container;
            container = null;
        } else if (options && container) {
            var reg = container.nodeName && 'Aa'.match(container.nodeName);
            if (!(reg && reg.length > 0)) {
                throw new Error("链接参数有误");
            }
        }
        this.container = container;
        this.options = extend$1({}, DEFAULT$2, options);
        if (isPPT()) {
            this.env = 'ppt';
        } else if (isSmartClass()) {
            this.env = 'smartClass';
        } else if (AppFactoryViewer.support()) {
            this.env = 'appFactory';
        } else {
            this.env = 'pc';
        }
        this.options.env = this.options.env || this.env;

        var src = this.container && this.container.getAttribute('href');
        this.options.href = src;
        !this.options.href && Logger$2.debug('没有链接地址');
        this.linkData = {
            src: src
        };
        this.viewer = null;
    }

    createClass(LinkView, [{
        key: 'render',
        value: function render() {
            this.$init();
        }
    }, {
        key: '$init',
        value: function $init() {
            this.$handle = proxy$1(this.$handle, this);
            this.container && addListener$1(this.container, 'click', this.$handle);
        }
    }, {
        key: '$handle',
        value: function $handle(event) {
            event.stopPropagation();
            event.preventDefault();
            if (!event.currentTarget.classList.contains('component_link_in_question')) {
                return;
            }
            // Logger.debug('clickLink');
            if (this.options.lock || !this.options.href) {
                return;
            }
            if (this.viewer == null) {
                var viewrOptions = extend$1({}, this.options, this.linkData);
                this.viewer = Factory$1.createViewer(viewrOptions);
            }
            // if (this.env == 'smartClass') {
            this.viewer.updateSrc && this.viewer.updateSrc(this.linkData.src);
            // }
            this.viewer.open();
        }
    }, {
        key: 'setLock',
        value: function setLock(lock) {
            this.options.lock = !!lock;
        }
    }, {
        key: 'getView',
        value: function getView() {
            return this.container;
        }
    }, {
        key: 'reset',
        value: function reset() {
            this.off();
        }
    }, {
        key: 'on',
        value: function on() {
            this.viewer && this.viewer.open();
        }
    }, {
        key: 'off',
        value: function off() {
            this.viewer && this.viewer.close();
        }
    }, {
        key: '$removeLister',
        value: function $removeLister() {
            this.container && removeListener$1(this.container, 'click', this.$handle);
        }
    }, {
        key: 'destroy',
        value: function destroy() {
            Logger$2.debug('销毁linkveiewer');
            this.$removeLister();
            this.viewer && this.viewer.destroy();
            this.container = null;
            this.viewer = null;
            this.scale = null;
            this.options = null;
            this.linkData = null;
        }
    }]);
    return LinkView;
}();

var LinkPlayer = {
    setLoggerLevel: function setLoggerLevel(level) {
        Logger$2.setLevel(level);
    },
    render: function render(container, options) {

        container = container.length > 0 ? container[0] : container; //添加对jq的适配

        for (var key in _$4.LINK_INSTANCES) {
            if (!document.querySelector('[data-id="' + _$4.getKeyId(key) + '"]')) {
                _$4.clear(key);
            }
        }
        _$4.VIEWER = null; //清空工厂

        options = options || {};
        Events$2.init();
        Events$2.unbind();
        Events$2.bind();
        if (container) {
            var linkItems = container.nodeName.toLowerCase() == 'a' ? [container] : container.getElementsByTagName('a');

            each$1(linkItems, function (item, index) {

                // if (item.parentNode && $.hasClass(item.parentNode, 'ndui-insert-pictures') ||
                //     item.parentNode && $.hasClass(item.parentNode, 'pop-insert-pictures__body')) {
                //     return;
                // }

                var dataId = getRandomId$1('media');

                var mediaId = 'link';
                var containerId = options.containerId || '';
                mediaId = mediaId + '-' + containerId + '-' + index;

                var key = mediaId + '|' + dataId;
                var newOptions = extend$1({}, options, {
                    dataId: dataId,
                    mediaId: mediaId,
                    key: key
                });

                var link = new LinkView(item, newOptions);
                link.render();
                _$4.LINK_INSTANCES[key] = link;
            });
        } else {
            Logger$2.debug('no container');
        }
        return _$4.LINK_INSTANCES;
    },

    destroy: function destroy() {
        Events$2.unbind();
        var instances = _$4.LINK_INSTANCES;
        for (var i = 0, len = instances.length; i < len; i++) {
            instances.destroy();
        }
        _$4.LINK_INSTANCES = {};
        _$4.VIEWER = null; //清空工厂
    },
    reset: function reset() {
        var instances = _$4.LINK_INSTANCES;
        for (var i = 0, len = instances.length; i < len; i++) {
            instances.off();
        }
    },
    setLock: function setLock(lock, view) {
        var instances = _$4.LINK_INSTANCES;
        for (var key in instances) {
            instances[key].setLock(lock);
        }
    },
    getMedia: function getMedia() {
        return _$4.LINK_INSTANCES;
    },
    mediaOnFullScreenChange: function mediaOnFullScreenChange() {
        //no use
    },
    Link: {
        create: function create(link, options) {
            Events$2.init();
            Events$2.unbind();
            Events$2.bind();
            return new LinkView(link, options);
        }
    },
    Viewer: {
        create: function create(options) {
            var env = '';
            if (isPPT()) {
                env = 'ppt';
            } else if (isSmartClass()) {
                env = 'smartClass';
            } else if (AppFactoryViewer.support()) {
                env = 'appFactory';
            } else {
                env = 'pc';
            }
            options.env = options.env || env;
            Events$2.init();
            Events$2.unbind();
            Events$2.bind();
            var instance = Factory$1.createViewer(options);
            return instance;
        }
    }
};

/**
 * Created by ylf on 2015/12/9.
 */

var __Q$1 = function () {
    'use strict';

    var typeUndefined = typeof undefined === 'undefined' ? 'undefined' : _typeof(undefined);

    //兼容代码
    if (_typeof(String.prototype.trim) === typeUndefined) {
        String.prototype.trim = function () {
            return this.replace(/(^\s*)|(\s*$)/g, "");
        };
    }
    //兼容代码
    if (_typeof(Array.prototype.forEach) === typeUndefined) {
        Array.prototype.forEach = function (callback) {
            for (var i = 0; i < this.length; i++) {
                var item = this[i];
                callback.apply(this, [item, i, this]);
            }
        };
    }

    var addEventListener = function addEventListener(target, type, handler) {
        if (target.addEventListener) {
            target.addEventListener(type, handler, false);
        } else if (target.attachEvent) {
            type = 'on' + type;
            target.attachEvent(type, handler);
        } else {
            target['on' + type] = handler;
        }
    };
    var removeEventListener = function removeEventListener(target, type, handler, flag) {
        if (target.removeEventListener) {
            target.removeEventListener(type, handler, flag);
        } else if (target.detachEvent) {
            type = 'on' + type;
            target.detachEvent(type, handler);
        } else {}
    };

    var getStyle = function getStyle(domObj, attribute) {
        var style = domObj.style[attribute];
        if (style) {
            return style;
        }
        return domObj.currentStyle ? domObj.currentStyle[attribute] : document.defaultView.getComputedStyle(domObj, '')[attribute];
    };

    var _q = function _q(selector, context) {
        if ((typeof selector === 'undefined' ? 'undefined' : _typeof(selector)) === typeUndefined || selector == null) {
            return;
        }

        if (typeof context === 'string') {
            selector = context + ' ' + selector;
        }

        if ((typeof context === 'undefined' ? 'undefined' : _typeof(context)) === typeUndefined || selector === null || context === document.body) {
            context = document;
        }

        this._selector = (typeof selector === 'undefined' ? 'undefined' : _typeof(selector)) === 'object' ? selector : selector.trim();
        this._dom = [];
        this.length = 0;

        if ((typeof selector === 'undefined' ? 'undefined' : _typeof(selector)) === 'object') {
            this._dom[0] = selector;
            this.length = 1;
        }
        //#id selector /.class selector
        else if (this._selector.substring(0, 1) === '#' && this._selector.indexOf(' ') <= -1) {
                this._dom[0] = document.getElementById(this._selector.substring(1, this._selector.length));
                if (this._dom[0]) {
                    this.length = 1;
                }
            } else {
                this._dom = context.querySelectorAll(this._selector);
                this.length = this._dom.length;
            }
        return this;
    };
    _q.prototype = {
        each: function each(fn) {
            if (this.length > 0) {
                for (var i = 0; i < this.length; i++) {
                    var dom = this._dom[i];
                    (function () {
                        fn.apply(dom, [i]);
                    })(i);
                }
            }
        },
        get: function get$$1(i) {
            return this._dom[i];
        },
        show: function show() {
            this.each(function () {
                this.style.display = 'block';
            });
        },
        hide: function hide() {
            this.each(function () {
                this.style.display = 'none';
            });
        },
        isShow: function isShow() {
            return this.css('display') === 'block';
        },
        toggle: function toggle() {
            this.each(function () {
                if (getStyle(this, 'display') === 'block') {
                    this.style.display = 'none';
                } else {
                    this.style.display = 'block';
                }
            });
        },
        css: function css(name, value) {
            if ((typeof value === 'undefined' ? 'undefined' : _typeof(value)) === typeUndefined) {
                return getStyle(this._dom[0], name);
            } else {
                this.each(function () {
                    this.style[name] = value;
                });
            }
        },
        find: function find(selector) {
            return __Q(selector, this._dom[0]);
        },
        bind: function bind(event, fn) {
            this.on(event, '', fn);
        },
        addClass: function addClass(clsName) {
            clsName = clsName.trim();

            this.each(function () {
                var classValue = this.getAttribute('class');
                var cls = classValue != null ? classValue.split(' ') : '';
                var newCls = clsName;
                for (var j = 0; j < cls.length; j++) {
                    var c = cls[j].trim();
                    if (c === clsName || c === '') {
                        continue;
                    }
                    newCls += ' ' + c;
                }
                this.setAttribute('class', newCls);
            });
            return this;
        },
        hasClass: function hasClass(clsName) {
            clsName = clsName.trim();
            var hasCls = false;
            this.each(function () {
                var classValue = this.getAttribute('class');
                var cls = classValue != null ? classValue.split(' ') : '';
                for (var j = 0; j < cls.length; j++) {
                    if (cls[j].trim() === clsName) {
                        hasCls = true;
                        break;
                    }
                }
            });
            return hasCls;
        },
        removeClass: function removeClass(clsName) {
            clsName = clsName.trim();
            this.each(function () {
                var classValue = this.getAttribute('class');
                var cls = classValue != null ? classValue.split(' ') : '';
                var newCls = '';
                for (var j = 0; j < cls.length; j++) {
                    var c = cls[j].trim();
                    if (c === clsName || c === '') {
                        continue;
                    }
                    newCls += ' ' + c;
                }
                this.setAttribute('class', newCls);
            });
        },
        toggleClass: function toggleClass(clsName) {
            this.each(function () {
                var cls = this.getAttribute('class').split(' ');
                var newCls = '';
                var hasCls = false;
                for (var j = 0; j < cls.length; j++) {
                    var c = cls[j].trim();
                    if (c === '') {
                        continue;
                    }
                    if (c === clsName) {
                        hasCls = true;
                        continue;
                    }
                    newCls += ' ' + c;
                }

                if (!hasCls) {
                    newCls += ' ' + clsName;
                }
                this.setAttribute('class', newCls);
            });
        },
        attr: function attr(_attr, val) {
            if (typeof val !== 'undefined') {
                this.each(function () {
                    if (typeof val !== 'undefined') {
                        this.setAttribute(_attr, val);
                    }
                });
                return this;
            } else {
                return this._dom[0].getAttribute(_attr);
            }
        },
        removeAttr: function removeAttr(attr) {
            this.each(function () {
                if (attr instanceof Array) {
                    for (var i = 0; i < attr.length; i++) {
                        this.removeAttribute(attr[i]);
                    }
                } else {
                    this.removeAttribute(attr);
                }
            });
        },
        text: function text(txt) {
            if ((typeof txt === 'undefined' ? 'undefined' : _typeof(txt)) === (typeof undefined === 'undefined' ? 'undefined' : _typeof(undefined))) {
                throw new Error('only support set text');
            }
            this.each(function () {
                this.innerText = txt;
            });
        },
        width: function width() {
            if (arguments.length > 0) {
                throw new Error('only support get');
            }
            var width = this._dom[0].offsetWidth;
            if (!width) {
                width = parseInt(this.css('width'));
            }
            return width;
        },
        height: function height() {
            if (arguments.length > 0) {
                throw new Error('only support get');
            }
            var height = this._dom[0].offsetHeight;
            if (!height) {
                height = parseInt(this.css('height'));
            }
            return height;
        },
        offset: function offset() {
            var docElem,
                win,
                box = {
                top: 0,
                left: 0
            },
                elem = this._dom[0],
                doc = elem && elem.ownerDocument;

            if (!doc) {
                return;
            }

            docElem = doc.documentElement;
            // If we don't have gBCR, just use 0,0 rather than error
            // BlackBerry 5, iOS 3 (original iPhone)
            if (_typeof(elem.getBoundingClientRect) !== (typeof undefined === 'undefined' ? 'undefined' : _typeof(undefined))) {
                box = elem.getBoundingClientRect();
            }
            win = doc == doc.window ? doc : doc.nodeType === 9 ? doc.defaultView || doc.parentWindow : false;
            return {
                top: box.top + (win.pageYOffset || docElem.scrollTop) - (docElem.clientTop || 0),
                left: box.left + (win.pageXOffset || docElem.scrollLeft) - (docElem.clientLeft || 0)
            };
        },
        on: function on(events, selector, fn) {
            var that = this;
            if (typeof selector === 'function') {
                fn = selector;
                selector = '';
            }
            events.split(' ').forEach(function (event) {

                if (event.trim() === '') {
                    return;
                }

                var eventName = event.split('.')[0];

                //使用touch替代click
                if (event.split('.')[0] === 'click' && __Q.support.touch || eventName === 'qpTap') {
                    __Q.event.qpTap((typeof selector === 'undefined' ? 'undefined' : _typeof(selector)) !== typeUndefined && selector !== '' ? that.find(selector) : that, fn);
                    return;
                }

                if (eventName === 'still-click') {
                    eventName = 'click';
                }

                if ((typeof selector === 'undefined' ? 'undefined' : _typeof(selector)) !== typeUndefined && selector !== '') {
                    that.find(selector).each(function () {
                        addEventListener(this, eventName, fn);
                    });
                } else {
                    that.each(function () {
                        addEventListener(this, eventName, fn);
                    });
                }
            });
        },
        off: function off(events, selector, fn) {
            var that = this;
            if (typeof selector === 'function') {
                fn = selector;
                selector = '';
            }
            events.split(' ').forEach(function (event) {

                if (event.trim() === '') {
                    return;
                }

                if ((typeof selector === 'undefined' ? 'undefined' : _typeof(selector)) !== typeUndefined && selector !== '') {
                    that.find(selector).each(function () {
                        removeEventListener(this, event.split('.')[0], fn, false);
                    });
                } else {
                    that.each(function () {
                        removeEventListener(this, event.split('.')[0], fn, false);
                    });
                }
            });
        },
        before: function before(ele) {
            if (typeof ele === 'string') {
                //创建临时dom，存放view
                var $temp = __Q(document.createElement('div'));
                $temp.html(ele);

                this._dom[0].parentNode.appendChild($temp.get(0));
                var children = $temp.get(0).children;
                var len = children.length;
                var i = 0;
                while (children.length > 0) {
                    if (_typeof(children[0]) === 'object') {
                        this.before(children[0]);
                    }
                    i++;
                    if (i >= len) {
                        break;
                    }
                }

                $temp.remove();
            } else {
                this._dom[0].parentNode.insertBefore(ele, this._dom[0]);
            }
        },
        appendChild: function appendChild(ele) {
            this.each(function () {
                this.appendChild(ele);
            });
        },
        html: function html(_html) {
            if ((typeof _html === 'undefined' ? 'undefined' : _typeof(_html)) !== typeUndefined) {
                this._dom[0].innerHTML = _html;
            } else {
                return this._dom[0].innerHTML;
            }
        },
        remove: function remove() {
            if (this._dom[0].parentNode) {
                this._dom[0].parentNode.removeChild(this._dom[0]);
            }
        },
        closest: function closest(clsSelector) {
            clsSelector = clsSelector.trim();
            if (clsSelector.substring(0, 1) !== '.' || clsSelector.indexOf(' ') > -1) {
                throw new Error('only support simple class selector ');
            }
            var cur;
            for (cur = this._dom[0]; cur && typeof cur !== 'undefined' && cur.nodeName.toLocaleLowerCase() !== "body"; cur = cur.parentNode) {
                // Always skip document fragments
                if (cur.nodeType < 11 &&
                // Don't pass non-elements to Sizzle
                cur.nodeType === 1 && new __Q(cur).hasClass(clsSelector.substring(1, clsSelector.length))) {
                    return __Q(cur);
                }
            }
            return;
        },
        contains: function contains(childNode) {
            var parentNode = this._dom[0];
            if (parentNode.contains) {
                return parentNode !== childNode && parentNode.contains(childNode);
            } else {
                return !!(parentNode.compareDocumentPosition(childNode) & 16);
            }
        },
        data: function data(key, val) {
            //自动生成一个随机id Key;
            var id = this.attr('data-id');
            if (((typeof id === 'undefined' ? 'undefined' : _typeof(id)) === (typeof undefined === 'undefined' ? 'undefined' : _typeof(undefined)) || id === null) && typeof val !== 'undefined') {
                id = __Q.utils.getRandomId('media');
                this.attr('data-id', id);
            }

            if (typeof val !== 'undefined') {
                if (val === null) {
                    delete __Q.cache[id + key];
                } else {
                    __Q.cache[id + key] = val;
                }
            } else {
                return typeof __Q.cache[id + key] === 'undefined' ? '' : __Q.cache[id + key];
            }
        }
    };

    var __Q = function __Q(selector, context) {
        return new _q(selector, context);
    };

    __Q.noop = function () {};

    //扩展原型方法
    __Q.fn = function (name, fn) {
        _q.prototype[name] = fn;
    };

    __Q.utils = {};
    __Q.support = {};
    __Q.event = {};

    __Q.support.isMobile = function () {
        return navigator.userAgent.match(/Android|iPhone|iPad|iPod|BlackBerry|IEMobile/i) ? true : false;
    }();
    __Q.support.isiOS = navigator.userAgent.match(/iPhone|iPad|iPod/i) ? true : false;

    __Q.support.touch = function () {
        if ((typeof navigator === 'undefined' ? 'undefined' : _typeof(navigator)) === typeUndefined) {
            return false;
        }
        if ('ontouchend' in document && __Q.support.isMobile) {
            return true;
        }
        return false;
    }();

    /**
     * 下面是一些基础函数，解决mouseover与mouserout事件不停切换的问题（问题不是由冒泡产生的）
     */
    __Q.event.checkHover = function () {
        var getEvent = function getEvent(e) {
            return e || window.event;
        };

        var contains = function contains(parentNode, childNode) {
            if (parentNode.contains) {
                return parentNode !== childNode && parentNode.contains(childNode);
            } else {
                return !!(parentNode.compareDocumentPosition(childNode) & 16);
            }
        };
        return function (e, target) {
            if (getEvent(e).type === "mouseover" || getEvent(e).type === "mousemove") {
                return !contains(target, getEvent(e).relatedTarget || getEvent(e).fromElement) && (getEvent(e).relatedTarget || getEvent(e).fromElement) !== target;
            } else {
                return !contains(target, getEvent(e).relatedTarget || getEvent(e).toElement) && (getEvent(e).relatedTarget || getEvent(e).toElement) !== target;
            }
        };
    }();

    __Q.event.isWhitePad = function (e) {
        //白板使用笔来点击会触发这个事件
        return !!(e.sourceCapabilities && e.sourceCapabilities.firesTouchEvents);
    };

    __Q.event.stopPropagation = function (e) {
        if (e.stopPropagation) {
            e.stopPropagation();
        } else {
            e.returnValue = false;
        }
    };
    __Q.event.preventDefault = function (e) {
        if (e.preventDefault) {
            e.preventDefault();
        } else {
            e.returnValue = false;
        }
    };

    __Q.utils.getRandomId = function (name) {
        return name + (Math.random() * 10 + new Date().getMilliseconds()).toString().replace('.', '');
    };

    __Q.utils.resize = function (oWidth, oHeight, maxW, maxH, minW, minH) {
        var w = oWidth,
            h = oHeight,
            scale = 1;

        if (oWidth < minW) {
            scale = (minW / oWidth).toFixed(3);
            h = parseInt(scale * oHeight);
            w = minW;
        }
        if (h < minH) {
            scale = (minH / oHeight).toFixed(3);
            w = parseInt(scale * oWidth);
            h = minH;
        }

        if (maxW > 0 && w > maxW) {
            scale = (maxW / oWidth).toFixed(3);
            h = parseInt(scale * oHeight);
            w = maxW;
        }
        if (maxH > 0 && h > maxH) {
            scale = (maxH / oHeight).toFixed(3);
            w = parseInt(scale * oWidth);
            h = maxH;
        }
        return {
            w: w,
            h: h,
            scale: scale
        };
    };

    __Q.utils.getRequest = function () {
        var sSearch = location.search;
        var oParam = {};
        sSearch = sSearch.substr(1, sSearch.length);
        var params = sSearch.split('&');
        for (var i = 0; i < params.length; i++) {
            var paramExpr = params[i];
            var param = paramExpr.split('=');
            var key = param[0];
            if (key) {
                var value = decodeURIComponent(param[1]);
                oParam[key] = value;
            }
        }

        var fn = function fn(sParamName) {
            return oParam[sParamName] || '';
        };
        fn.get = function () {
            return oParam;
        };
        return fn;
    }();

    //获取给定目录的基本地址
    __Q.utils.getBasePath = function (path) {
        var hashIndex = path.lastIndexOf("#");
        if (hashIndex == -1) {
            hashIndex = path.length;
        }
        var queryIndex = path.indexOf("?");
        if (queryIndex == -1) {
            queryIndex = path.length;
        }
        var slashIndex = path.lastIndexOf("/", Math.min(queryIndex, hashIndex));
        return slashIndex >= 0 ? path.substring(0, slashIndex + 1) : "";
    };

    //jquery的extend方法
    __Q.extend = function () {
        var src,
            copyIsArray,
            copy,
            name,
            options,
            clone,
            target = arguments[0] || {},
            i = 1,
            length = arguments.length,
            deep = false;

        // Handle a deep copy situation
        if (typeof target === "boolean") {
            deep = target;
            target = arguments[1] || {};
            // skip the boolean and the target
            i = 2;
        }

        // Handle case when target is a string or something (possible in deep copy)
        if ((typeof target === 'undefined' ? 'undefined' : _typeof(target)) !== "object" && !jQuery.isFunction(target)) {
            target = {};
        }

        // extend jQuery itself if only one argument is passed
        if (length === i) {
            target = this;
            --i;
        }

        for (; i < length; i++) {
            // Only deal with non-null/undefined values
            if ((options = arguments[i]) != null) {
                // Extend the base object
                for (name in options) {
                    src = target[name];
                    copy = options[name];

                    // Prevent never-ending loop
                    if (target === copy) {
                        continue;
                    }

                    // Recurse if we're merging plain objects or arrays
                    if (deep && copy && (jQuery.isPlainObject(copy) || (copyIsArray = jQuery.isArray(copy)))) {
                        if (copyIsArray) {
                            copyIsArray = false;
                            clone = src && jQuery.isArray(src) ? src : [];
                        } else {
                            clone = src && jQuery.isPlainObject(src) ? src : {};
                        }

                        // Never move original objects, clone them
                        target[name] = jQuery.extend(deep, clone, copy);

                        // Don't bring in undefined values
                    } else if (copy !== undefined) {
                        target[name] = copy;
                    }
                }
            }
        }

        // Return the modified object
        return target;
    };

    __Q.cache = {}; //TODO:这个暂时这样处理。后续测试通过后修改为闭包

    __Q.browser = {};

    var uaBrowser = function uaBrowser(userAgent) {
        var ua = userAgent.toLowerCase();
        // Useragent RegExp
        var rwebkit = /(webkit)[ \/]([\w.]+)/;
        var ropera = /(opera)(?:.*version)?[ \/]([\w.]+)/;
        var rmsie = /(msie) ([\w.]+)/;
        var rmozilla = /(mozilla)(?:.*? rv:([\w.]+))?/;

        var match = rwebkit.exec(ua) || ropera.exec(ua) || rmsie.exec(ua) || ua.indexOf("compatible") < 0 && rmozilla.exec(ua) || [];
        return {
            browser: match[1] || "",
            version: match[2] || "0"
        };
    };

    if ((typeof navigator === 'undefined' ? 'undefined' : _typeof(navigator)) !== typeUndefined) {
        var browserMatch = uaBrowser(navigator.userAgent);
        if (browserMatch.browser) {
            __Q.browser[browserMatch.browser] = true;
            __Q.browser.version = browserMatch.version;
        }
        __Q.browser.crosswalk = /(crosswalk)/.test(navigator.userAgent.toLowerCase());
    }

    //if (typeof  exports !== typeUndefined) {
    //    exports.__Q = __Q;
    //}
    if ((typeof module === 'undefined' ? 'undefined' : _typeof(module)) === "object" && module && _typeof(module.exports) === "object") {
        // Expose __Q as module.exports in loaders that implement the Node
        // module pattern (including browserify). Do not create the global, since
        // the user will be storing it themselves locally, and globals are frowned
        // upon in the Node module world.
        module.exports = __Q;
    }

    return __Q;
}();

/**
 * Created by ylf on 2015/12/13.
 */

var methods$2 = ['assert', 'clear', 'count', 'debug', 'dir', 'dirxml', 'error', 'exception', 'group', 'groupCollapsed', 'groupEnd', 'info', 'log', 'markTimeline', 'profile', 'profileEnd', 'table', 'time', 'timeEnd', 'timeline', 'timelineEnd', 'timeStamp', 'trace', 'warn'];
var length$2 = methods$2.length;
var _console = window.console = window.console || {};
var method$2 = null;

while (length$2--) {
    method$2 = methods$2[length$2];
    // Only stub undefined methods.
    if (!_console[method$2]) {
        _console[method$2] = __Q.noop;
    }
}

//logger level值(debug:3, info:2, warn:1, error: 0, close: -1)
var _logger = {
    _level: 0,
    _console: _console,
    setLevel: function setLevel(level) {
        switch (level) {
            case 'debug':
                this._level = 3;
                break;
            case 'info':
                this._level = 2;
                break;
            case 'warn':
                this._level = 1;
                break;
            case 'error':
                this._level = 0;
                break;
            default:
                this._level = -1;
        }
    },
    debug: function debug() {
        if (this._level >= 3) {
            this._console.log.apply(this._console, arguments);
        }
    },
    info: function info(msg) {
        if (this._level >= 2) {
            this._console.info.apply(this._console, arguments);
        }
    },
    warn: function warn(msg) {
        if (this._level >= 1) {
            this._console.warn.apply(this._console, arguments);
        }
    },
    error: function error(msg) {
        if (this._level >= 0) {
            this._console.error.apply(this._console, arguments);
        }
    }
};

/**
 * Created by zph on 2017/10/12.
 */

function AudioPlayerPlugin(option) {
    option = option || {};
    return this.each(function () {
        var $this = __Q$1(this);

        var data = $this.data('my.mediaplayer');
        var options = __Q$1.extend({}, (typeof option === 'undefined' ? 'undefined' : _typeof(option)) === 'object' && option);
        //插件缓存
        if (!data) {
            options.container = $this.get(0);
            data = AudioPlayer.Audio.create(options);
            $this.data('my.mediaplayer', data);
        }
    });
}

__Q$1.fn('audioplayer', AudioPlayerPlugin);

/**
 * Created by ylf on 2015/12/9.
 */

function ImagePlayerPlugin(option) {
    option = option || {};
    return this.each(function () {
        var $this = __Q$1(this);

        var data = $this.data('my.mediaplayer');
        var options = __Q$1.extend({}, (typeof option === 'undefined' ? 'undefined' : _typeof(option)) === 'object' && option);
        //插件缓存
        if (!data) {
            data = ImagePlayer.Image.create($this.get(0), options);
            $this.data('my.mediaplayer', data);
        }
    });
}

__Q$1.fn('imgplayer', ImagePlayerPlugin);

/**
 * Created by lxq on 2019/8/22
 */

function LinkPlayerPlugin(option) {
    option = option || {};
    return this.each(function () {
        var $this = __Q$1(this);
        var data = $this.data('my.mediaplayer');
        var options = __Q$1.extend({}, (typeof option === 'undefined' ? 'undefined' : _typeof(option)) === 'object' && option);
        if (option.dataId) {
            $this.attr('data-id', option.dataId);
        }
        //插件缓存
        if (!data) {
            data = LinkPlayer.Link.create($this.get(0), options);
            $this.data('my.mediaplayer', data);
        } else if (option.dataId) {
            data.options.dataId = option.dataId;
            $this.data('my.mediaplayer', data);
        }
    });
}

__Q$1.fn('linkplayer', LinkPlayerPlugin);

/**
 * Created by zph on 2015/12/9.
 */

function VideoPlayerPlugin(option) {
    option = option || {};
    return this.each(function () {
        var $this = __Q$1(this);

        var data = $this.data('my.mediaplayer');
        var options = __Q$1.extend({}, (typeof option === 'undefined' ? 'undefined' : _typeof(option)) === 'object' && option);
        //插件缓存
        if (!data) {
            options.container = $this.get(0);
            data = VideoPlayer.Video.create(options);
            $this.data('my.mediaplayer', data);
        }
    });
}

__Q$1.fn('videoplayer', VideoPlayerPlugin);

var DefaultOption = {
    lang: 'zh_cn',
    mutuallyExclusive: true,
    //mediaMargin: '10px 0px',//旧版参数，需要兼容
    containerId: '', //渲染外层id，用于标识多媒体，投影端同步使用
    style: '', //容器样式
    img: {
        render: false,
        renderImmediately: false, //不立即加载
        width: 0,
        height: 0,
        src: ''
    },
    link: {
        render: true,
        href: ''
    },
    video: {
        render: true,
        style: '' //容器样式
    },
    audio: {
        render: true,
        style: '' //容器样式
    },
    onPlay: null,
    onPause: null,
    onEnd: null,
    onSeek: null,
    onVolumeChange: null,
    onTimeUpdate: null,
    onFullScreenChange: null,
    onImageOpen: null
};
var copyOption = function copyOption(option) {
    //渲染的多媒体资源
    option = option || {};
    var imgOption = __Q$1.extend({}, DefaultOption.img, option.img || {});
    var videoOption = __Q$1.extend({}, DefaultOption.video, option.video || {});
    var audioOption = __Q$1.extend({}, DefaultOption.audio, option.audio || {});
    var linkOption = __Q$1.extend({}, DefaultOption.link, option.link || {});

    option = __Q$1.extend({}, DefaultOption, option || {});
    option.img = imgOption;
    option.video = videoOption;
    option.audio = audioOption;
    option.link = linkOption;

    if (typeof option.containerId !== 'string') {
        option.containerId = '';
    }

    return option;
};

/**
 * Created by ylf on 2015/12/13.
 * 互动课堂第三方API
 */

var isApp = typeof AndroidInterface !== "undefined";
var bridgeCallNative = typeof Bridge !== 'undefined' && Bridge && Bridge.callNative;
var isAppNative = isApp && bridgeCallNative;
var STUDENT = 'STUDENT';
var TEACHER = 'TEACHER';

var isStudent = false;
var isTeacher = false;
var isProjector = false;

var userInfo = null;

var callNative = function callNative(method, params) {
    try {
        return Bridge.callNative("com.nd.pad.icr.ui.IcrJsBridge", method, params || {});
    } catch (ex) {
        _logger.warn('mediaplayer call native error', ex);
    }
};

//获取当前app用户信息
var getCurrentUserInfo = function getCurrentUserInfo() {
    if (isAppNative && !userInfo) {
        var res = callNative("getCurrentUserInfo", {});
        if (typeof Promise !== 'undefined') {
            return Promise.resolve(res).then(function (result) {
                if (result) {
                    userInfo = result;
                    isStudent = result.userType === STUDENT;
                    isTeacher = result.userType === TEACHER;
                    isProjector = result.isProjector;
                }
            });
        }
    }
};

var setViewerModuleVisibility = function setViewerModuleVisibility(show) {
    if (isAppNative) {
        var result = getCurrentUserInfo();
        if (result && result.then) {
            result.then(function () {
                if (isStudent) {
                    //隐藏学生端view
                    var res = callNative("setPresenterVisibility", {
                        instanceId: "studentDraft",
                        visible: show
                    }); //草稿按钮栏
                    if (typeof Promise !== 'undefined') {
                        Promise.resolve(res).then(function (result) {
                            _logger.debug(result);
                        });
                    }
                }
            });
        }
    }
};

var InteractAppApi = {
    setViewerModuleVisibility: setViewerModuleVisibility
};

//播放互相排斥
var mutuallyExclusivePlay = function mutuallyExclusivePlay(mediaObj) {
    var that = window.AviMediaPlayer;
    var mediaId = mediaObj.mediaId + '|' + mediaObj.dataId;
    for (var key in that._$medias) {
        if (mediaId !== key && that._$medias[key]) {
            if (that._$medias[key].data('my.mediaplayer').pause) {
                that._$medias[key].data('my.mediaplayer').pause();
            }
        }
    }
};

var makeMediaEventOption = function makeMediaEventOption(option, eventHandler, mediaId, index, mediaType, dataId) {

    var eventOption = {};

    var params = {
        mediaId: mediaId,
        index: index,
        mediaType: mediaType
    };

    var onPlay = option.onPlay;
    eventOption.onPlay = function () {
        var newParams = __Q$1.extend({}, params, {
            dataId: dataId
        });
        if (option.mutuallyExclusive) {
            mutuallyExclusivePlay(newParams);
        }
        if (onPlay) {
            onPlay(newParams);
        }
        if (eventHandler.play) {
            eventHandler.play(newParams);
        }
    };

    var onPause = option.onPause;
    eventOption.onPause = function () {
        var newParams = __Q$1.extend({}, params);
        if (onPause) {
            onPause(newParams);
        }
        if (eventHandler.pause) {
            eventHandler.pause(newParams);
        }
    };

    var onEnd = option.onEnd;
    eventOption.onEnd = function (val) {
        var newParams = __Q$1.extend({}, params);
        if (onEnd) {
            onEnd(newParams);
        }
        if (eventHandler.ended) {
            eventHandler.ended(newParams);
        }
    };

    var onSeek = option.onSeek;
    eventOption.onSeek = function (val) {
        var newParams = __Q$1.extend({}, params, {
            seeked: val
        });
        if (onSeek) {
            onSeek(newParams);
        }
        if (eventHandler.seeked) {
            eventHandler.seeked(newParams);
        }
    };

    var onVolumeChange = option.onVolumeChange;
    eventOption.onVolumeChange = function (val) {
        var newParams = __Q$1.extend({}, params, {
            volume: val
        });
        if (onVolumeChange) {
            onVolumeChange(newParams);
        }
        if (eventHandler.volumeChange) {
            eventHandler.volumeChange(newParams);
        }
    };

    var onTimeUpdate = option.onTimeUpdate;
    eventOption.onTimeUpdate = function (val) {
        var newParams = __Q$1.extend({}, params, {
            currentTime: val
        });
        if (onTimeUpdate) {
            onTimeUpdate(newParams);
        }
        if (eventHandler.timeupdate) {
            eventHandler.timeupdate(newParams);
        }
    };

    var onFullScreenChange = option.onFullScreenChange;
    eventOption.onFullScreenChange = function (val) {
        if (onFullScreenChange) {
            onFullScreenChange(val);
        }
        if (eventHandler.fullScreenChange) {
            eventHandler.fullScreenChange(val);
        }

        //隐藏原生view
        InteractAppApi.setViewerModuleVisibility(!val);
    };

    // 监听单击图片查看事件
    var onImageOpen = option.onImageOpen;
    eventOption.onImageOpen = function (val) {
        if (onImageOpen) {
            onImageOpen(val);
        }
        if (eventHandler.imageOpen) {
            eventHandler.imageOpen(val);
        }
    };

    return eventOption;
};

/**
 * Created by ylf on 2015/12/9.
 */

//空方法
var _emptyFunc = function _emptyFunc() {};

var sendEventToPlayer = function sendEventToPlayer(name, data) {
    if (window.player && player.fireEvent) {
        player.fireEvent(name, data);
    }
};

var mediaArry = ['audio', 'video', 'img', 'a'];

var mediaAlias = {
    'a': 'link'
};

window.AviMediaPlayer = {
    _$medias: {}, //{data-media|data-id:{data}}
    _eventHandler: {
        play: _emptyFunc,
        pause: _emptyFunc,
        ended: _emptyFunc,
        timeupdate: _emptyFunc,
        seeked: _emptyFunc,
        volumeChange: _emptyFunc,
        fullScreenChange: _emptyFunc,
        imageOpen: _emptyFunc
    },
    _getMedia: function _getMedia(mediaId) {
        var that = this;
        for (var key in this._$medias) {
            if (that._getKeyMedia(key) === mediaId) {
                var $media = this._$medias[key];
                return $media;
            }
        }
        return null;
    },
    _getKeyByMediaId: function _getKeyByMediaId(mediaId) {
        var that = this;
        for (var key in this._$medias) {
            if (that._getKeyMedia(key) === mediaId) {
                return key;
            }
        }
        return null;
    },
    _getKey: function _getKey($renderBody) {
        return $renderBody.attr('data-media') + '|' + $renderBody.attr('data-id');
    },
    _getKeyId: function _getKeyId(key) {
        return key.split('|')[1];
    },
    _getKeyMedia: function _getKeyMedia(key) {
        return key.split('|')[0];
    },
    _getKeyType: function _getKeyType(key) {
        return key.split('-')[0];
    },
    _clear: function _clear(rkey) {
        var that = this;
        if (that._$medias[rkey]) {
            var mediaPlayer = that._$medias[rkey].data('my.mediaplayer');
            if (mediaPlayer) {
                mediaPlayer.destroy();
            }
            that._$medias[rkey].data('my.mediaplayer', null);
            delete that._$medias[rkey];
        }
    },
    setLoggerLevel: function setLoggerLevel(level) {
        _logger.setLevel(level);
        ImagePlayer.setLoggerLevel(level);
    },
    /**
     * 销毁内存对象,并解绑事件
     * @param obj id或view对象。不传直接销毁dom不存在的内存对象
     */
    destroy: function destroy(obj) {
        var that = this;

        //解绑事件
        for (var key in that._eventHandler) {
            that._eventHandler[key] = _emptyFunc;
        }

        //清除指定容器内的对象
        if (arguments.length > 0) {
            var type = typeof obj === 'undefined' ? 'undefined' : _typeof(obj);
            var $view = null;
            if (type === 'string') {
                $view = __Q$1('#' + obj);
            } else if (type === 'object') {
                if ((typeof jQuery === 'undefined' ? 'undefined' : _typeof(jQuery)) !== (typeof undefined === 'undefined' ? 'undefined' : _typeof(undefined)) && obj instanceof jQuery) {
                    $view = __Q$1(obj[0]);
                } else {
                    $view = __Q$1(obj);
                }
            }
            if ($view.length === 0) {
                return;
            }
            var $renderBodys = $view.find('.qp_media');
            for (var i = 0; i < $renderBodys.length; i++) {
                var rkey = that._getKey(__Q$1($renderBodys.get(i)));
                that._clear(rkey);
            }
            if (!$renderBodys || $renderBodys.length === 0) {
                //删除不存在dom内的缓存对象
                for (var mediakey in that._$medias) {
                    if (!document.querySelector('[data-id="' + that._getKeyId(mediakey) + '"]')) {
                        that._clear(mediakey);
                    }
                }
            }
        } else {
            //删除不存在dom内的缓存对象
            for (var mediakey in that._$medias) {
                if (!document.querySelector('[data-id="' + that._getKeyId(mediakey) + '"]')) {
                    that._clear(mediakey);
                }
            }
        }
    },
    reset: function reset() {
        for (var mediakey in this._$medias) {
            if (this._$medias[mediakey]) {
                var mediaPlayer = this._$medias[mediakey].data('my.mediaplayer');
                if (mediaPlayer && mediaPlayer.reset) {
                    mediaPlayer.reset();
                }
            }
        }
        // exitFull();
    },
    resize: function resize(data) {
        for (var mediakey in this._$medias) {
            if (this._$medias[mediakey]) {
                var mediaPlayer = this._$medias[mediakey].data('my.mediaplayer');
                if (mediaPlayer && mediaPlayer.resize) {
                    mediaPlayer.resize(data);
                }
            }
        }
    },
    setLock: function setLock(lock, view) {
        if (view) {
            var type = typeof view === 'undefined' ? 'undefined' : _typeof(view);
            var $view = null;
            if (type === 'string') {
                $view = __Q$1('#' + view);
            } else if (type === 'object') {
                if ((typeof jQuery === 'undefined' ? 'undefined' : _typeof(jQuery)) !== (typeof undefined === 'undefined' ? 'undefined' : _typeof(undefined)) && view instanceof jQuery) {
                    $view = __Q$1(view[0]);
                } else {
                    $view = __Q$1(view);
                }
            }
            if ($view.length === 0) {
                return;
            }
            var $renderBodys = $view.find('.qp_media');
            for (var i = 0; i < $renderBodys.length; i++) {
                var rkey = this._getKey(__Q$1($renderBodys.get(i)));
                if (this._$medias[rkey]) {
                    var mediaPlayer = this._$medias[rkey].data('my.mediaplayer');
                    if (mediaPlayer && mediaPlayer.setLock) {
                        mediaPlayer.setLock(lock);
                    }
                }
            }
        } else {
            for (var mediakey in this._$medias) {
                if (this._$medias[mediakey]) {
                    var mediaPlayer = this._$medias[mediakey].data('my.mediaplayer');
                    if (mediaPlayer && mediaPlayer.setLock) {
                        mediaPlayer.setLock(lock);
                    }
                }
            }
        }
    },
    render: function render($view, option, callback) {
        var that = this;

        var totalMediaNumber = 0;
        var areadyMedias = window.AviMediaPlayer.getMedia();
        for (var key in areadyMedias) {
            if (areadyMedias[key]) {
                totalMediaNumber++;
            }
        }

        //删除缓存
        $view.querySelector && $view.querySelector("audio") && $view.querySelector("audio").setAttribute("data-id", __Q$1.utils.getRandomId('media'));
        $view.querySelector && $view.querySelector("video") && $view.querySelector("video").setAttribute("data-id", __Q$1.utils.getRandomId('media'));
        for (var key in that._$medias) {
            var mediaObj = that._$medias[key].data('my.mediaplayer');
            if (mediaObj && mediaObj.getView && mediaObj.getView()) {
                if (!mediaObj.getView().parentNode) {
                    that._clear(key);
                }
            } else if (!document.querySelector('[data-id="' + that._getKeyId(key) + '"]')) {
                that._clear(key);
            }
        }

        var option = copyOption(option);
        // 多媒体资源图片的promise
        var promiseArray = [];
        //渲染的多媒体资源
        var instances = {};

        var $medias = {};
        var container = $view.length > 0 ? $view[0] : $view;
        var mediaReg = /^(img|a|video|audio)$/i;
        if (container.nodeName && mediaReg.test(container.nodeName)) {
            $medias[container.nodeName.toLocaleLowerCase()] = __Q$1(container);
        } else {
            for (var i = 0, l = mediaArry.length; i < l; i++) {
                $medias[mediaAlias[mediaArry[i]] || mediaArry[i]] = __Q$1(container).find(mediaArry[i]);
            }
        }
        //给icplayer发送开始事件
        var eventId = utils$2.getUuid();
        sendEventToPlayer('LoadResourceStart', {
            id: eventId
        });

        for (var type in $medias) {
            if (!$medias.hasOwnProperty(type)) {
                continue;
            }
            $medias[type].each(function (index) {
                if (!option[type].render) {
                    return;
                }

                index += totalMediaNumber;

                var $media = __Q$1(this),
                    $renderBody = $media.closest('.qp_media');

                //已经渲染过，不进行二次渲染
                if ($renderBody && $renderBody.length > 0 && $renderBody.get(0) && !$renderBody.hasClass('component_link_in_question')) {
                    //由于超链接类型里面可能会插入img等媒体资源所以这里加了个特殊判断
                    return;
                }

                var instance = null,
                    identifierIndex = option.containerId + '-' + index,
                    dataId = __Q$1.utils.getRandomId('media'),
                    mediaId = type + '-' + identifierIndex,
                    key = mediaId + '|' + dataId,
                    eventOption = makeMediaEventOption(option, that._eventHandler, mediaId, identifierIndex, type, dataId);

                var mediaOption = __Q$1.extend({}, option[type], {
                    lang: option.lang, dataId: dataId
                }, eventOption);

                // mediaOption.preload = (index >= 50) ? 'none' :'metadata'
                mediaOption.preload = index >= 50 ? 'none' : 'auto';

                //create
                $media[type + 'player'](mediaOption);

                instance = $media.data('my.mediaplayer');
                if (instance) {
                    // instance.render();                                        
                    promiseArray.push(instance.render());
                    instance.setLock(option.lock);
                }
                var $insView = __Q$1(instance.getView());
                $insView.addClass('qp_media');

                $insView.attr('data-id', dataId);
                $insView.attr('data-media', type + '-' + identifierIndex);

                that._$medias[key] = $media;
                //返回实例
                instances[key] = instance;
            });
        }
        Promise.all(promiseArray).then(function () {
            sendEventToPlayer('LoadResourceComplete', {
                id: eventId
            });
            callback && callback();
        }).catch(function () {
            sendEventToPlayer('LoadResourceComplete', {
                id: eventId
            });
            callback && callback();
            console.log('图片资源error');
        });
        return instances;
    },
    change: function change(mediaId, option) {
        var $media = this._getMedia(mediaId);
        if ($media && $media.get(0)) {

            var dataId = this._getKeyByMediaId(mediaId);
            var keyId = this._getKeyId(dataId);

            var instance = $media.data('my.mediaplayer');
            if (instance) {
                instance.change(option);
            }
            var $insView = __Q$1(instance.getView());
            $insView.addClass('qp_media');

            $insView.attr('data-id', keyId);
            $insView.attr('data-media', mediaId);

            this._$medias[dataId] = $media;
        }
    },
    mediaOnStart: function mediaOnStart(callback) {
        if (typeof callback === 'function') {
            this._eventHandler.play = callback;
        }
    },
    mediaOnPause: function mediaOnPause(callback) {
        if (typeof callback === 'function') {
            this._eventHandler.pause = callback;
        }
    },
    mediaOnEnded: function mediaOnEnded(callback) {
        if (typeof callback === 'function') {
            this._eventHandler.ended = callback;
        }
    },
    mediaOnTimeupdate: function mediaOnTimeupdate(callback) {
        if (typeof callback === 'function') {
            this._eventHandler.timeupdate = callback;
        }
    },
    mediaOnSeeked: function mediaOnSeeked(callback) {
        if (typeof callback === 'function') {
            this._eventHandler.seeked = callback;
        }
    },
    mediaOnVolumeChange: function mediaOnVolumeChange(callback) {
        if (typeof callback === 'function') {
            this._eventHandler.volumeChange = callback;
        }
    },
    mediaOnImageOpen: function mediaOnImageOpen(callback) {
        if (typeof callback === 'function') {
            this._eventHandler.imageOpen = callback;
        }
    },
    getMedia: function getMedia(key) {
        if (key) {
            var $media = this._$medias[key];
            return $media;
        } else {
            return this._$medias;
        }
    },
    isPaused: function isPaused(mediaId) {
        var $media = this._getMedia(mediaId);
        if ($media && $media.get(0)) {
            return $media.data('my.mediaplayer').getStatus();
        }
    },
    end: function end(mediaId) {
        var $media = this._getMedia(mediaId);
        if ($media && $media.get(0)) {
            return $media.data('my.mediaplayer').end();
        }
    },
    play: function play(mediaId) {
        var $media = this._getMedia(mediaId);
        if ($media && $media.get(0)) {
            $media.data('my.mediaplayer').play();
        }
    },
    pause: function pause(mediaId) {
        var $media = this._getMedia(mediaId);
        if ($media && $media.get(0)) {
            $media.data('my.mediaplayer').pause();
        }
    },
    seek: function seek(mediaId, seeked) {
        var $media = this._getMedia(mediaId);
        if ($media && $media.get(0)) {
            $media.data('my.mediaplayer').seek(seeked == 0 ? 0.01 : seeked);
        }
    },
    changeVolume: function changeVolume(mediaId, volume) {
        var $media = this._getMedia(mediaId);
        if ($media && $media.get(0)) {
            $media.data('my.mediaplayer').changeVolume(volume);
        }
    },
    offset: function offset(mediaId, left, top) {
        var $media = this._getMedia(mediaId);
        if ($media && $media.get(0)) {
            $media.data('my.mediaplayer').offset(left, top);
        }
    },
    syncLyricAll: function syncLyricAll(lyric) {
        var that = this;
        for (var key in that._$medias) {
            var $media = that._$medias[key];
            if ($media && $media.get(0)) {
                if (!$media.data('my.mediaplayer').options.audio) {
                    continue;
                }
                if ($media.data('my.mediaplayer').options.audio.lyric == lyric) {
                    $media.data('my.mediaplayer').syncLyric && $media.data('my.mediaplayer').syncLyric();
                }
            }
        }
    },
    hideLyricAll: function hideLyricAll() {
        var that = this;
        for (var key in that._$medias) {
            var $media = that._$medias[key];
            if ($media && $media.get(0)) {
                $media.data('my.mediaplayer').hideLyric && $media.data('my.mediaplayer').hideLyric();
            }
        }
    },
    fullScreenStyleAll: function fullScreenStyleAll(option) {
        var that = this;
        for (var key in that._$medias) {
            var $media = that._$medias[key];
            if ($media && $media.get(0)) {
                $media.data('my.mediaplayer').fullScreenStyle && $media.data('my.mediaplayer').fullScreenStyle(option);
            }
        }
    },
    pauseAll: function pauseAll(obj) {
        var that = this;
        if ((typeof obj === 'undefined' ? 'undefined' : _typeof(obj)) !== (typeof undefined === 'undefined' ? 'undefined' : _typeof(undefined))) {
            var type = typeof obj === 'undefined' ? 'undefined' : _typeof(obj);
            var $view = null;
            if (type === 'string') {
                $view = __Q$1('#' + obj);
            } else if (type === 'object') {
                if ((typeof jQuery === 'undefined' ? 'undefined' : _typeof(jQuery)) !== (typeof undefined === 'undefined' ? 'undefined' : _typeof(undefined)) && obj instanceof jQuery) {
                    $view = __Q$1(obj[0]);
                } else {
                    $view = __Q$1(obj);
                }
            }
            if ($view.length === 0) {
                return;
            }
            var $renderBodys = $view.find('.qp_media');
            for (var i = 0; i < $renderBodys.length; i++) {
                var rkey = that._getKey(__Q$1($renderBodys.get(i)));
                if (that._$medias[rkey]) {
                    var mediaPlayer = that._$medias[rkey].data('my.mediaplayer');
                    if (mediaPlayer && mediaPlayer.pause) {
                        mediaPlayer.pause();
                    }
                }
            }
        } else {
            for (var key in that._$medias) {
                var $media = that._$medias[key];
                if ($media && $media.get(0) && $media.data('my.mediaplayer') && $media.data('my.mediaplayer').pause) {
                    $media.data('my.mediaplayer').pause();
                }
            }
        }
    },

    mediaOnFullScreenChange: function mediaOnFullScreenChange(callback) {
        if (typeof callback === 'function') {
            this._eventHandler.fullScreenChange = callback;
        }
    },
    Image: {
        create: function create(img, options) {
            return ImagePlayer.Image.create(img, options);
        }
    },
    ImageViewer: {
        create: function create(options) {
            return ImagePlayer.Viewer.create(options);
        }
    },
    Link: {
        create: function create(link, options) {
            return LinkPlayer.Link.create(link, options);
        }
    },
    LinkViewer: {
        create: function create(options) {
            return LinkPlayer.Viewer.create(options);
        }
    },
    Video: {
        create: function create(options) {
            return VideoPlayer.Video.create(options);
        }
    },
    Audio: {
        create: function create(options) {
            return AudioPlayer.Audio.create(options);
        }
    }
};

})));

})(window,document)